/* Generated by re2c 0.16 */
#line 1 "src/wast-lexer.cc"
/*
 * Copyright 2016 WebAssembly Community Group participants
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "src/wast-lexer.h"

#include <cassert>
#include <cstdio>

#include "config.h"

#include "src/lexer-source.h"
#include "src/wast-parser.h"

#define YYMAXFILL 29

#define INITIAL_LEXER_BUFFER_SIZE (64 * 1024)

#define ERROR(...) parser->Error(GetLocation(), __VA_ARGS__)

#define BEGIN(c) cond = (c)
#define FILL(n)              \
  do {                       \
    if (Failed(Fill((n)))) { \
      RETURN(Eof);           \
    }                        \
  } while (0)

#define MAYBE_MALFORMED_UTF8(desc)                \
  if (!(eof_ && limit_ - cursor_ <= YYMAXFILL)) { \
    ERROR("malformed utf-8%s", desc);             \
  }                                               \
  continue

#define yytext (next_pos_)
#define yyleng (cursor_ - next_pos_)

/* p must be a pointer somewhere in the lexer buffer */
#define FILE_OFFSET(p) ((p) - (buffer_) + buffer_file_offset_)
#define COLUMN(p) (FILE_OFFSET(p) - line_file_offset_ + 1)

#define COMMENT_NESTING (comment_nesting_)
#define NEWLINE                               \
  do {                                        \
    line_++;                                  \
    line_file_offset_ = FILE_OFFSET(cursor_); \
  } while (0)

#define RETURN(token) return Token(GetLocation(), TokenType::token);

#define RETURN_LITERAL(token, literal)          \
  return Token(GetLocation(), TokenType::token, \
               MakeLiteral(LiteralType::literal))

#define RETURN_TYPE(token, type) \
  return Token(GetLocation(), TokenType::token, Type::type)

#define RETURN_OPCODE0(token) \
  return Token(GetLocation(), TokenType::token, Opcode::token)

#define RETURN_OPCODE(token, opcode) \
  return Token(GetLocation(), TokenType::token, Opcode::opcode)

#define RETURN_TEXT(token) \
  return Token(GetLocation(), TokenType::token, GetText())

#define RETURN_TEXT_AT(token, at) \
  return Token(GetLocation(), TokenType::token, GetText(at))

namespace wabt {

WastLexer::WastLexer(std::unique_ptr<LexerSource> source, string_view filename)
    : source_(std::move(source)),
      filename_(filename),
      line_(1),
      comment_nesting_(0),
      buffer_file_offset_(0),
      line_file_offset_(0),
      eof_(false),
      buffer_(nullptr),
      buffer_size_(0),
      marker_(nullptr),
      next_pos_(nullptr),
      cursor_(nullptr),
      limit_(nullptr) {}

WastLexer::~WastLexer() {
  delete[] buffer_;
}

// static
std::unique_ptr<WastLexer> WastLexer::CreateBufferLexer(string_view filename,
                                                        const void* data,
                                                        size_t size) {
  return MakeUnique<WastLexer>(MakeUnique<LexerSource>(data, size), filename);
}

Location WastLexer::GetLocation() {
  return Location(filename_, line_, COLUMN(next_pos_), COLUMN(cursor_));
}

Literal WastLexer::MakeLiteral(LiteralType type) {
  return Literal(type, GetText());
}

std::string WastLexer::GetText(size_t offset) {
  return std::string(yytext + offset, yyleng - offset);
}

Result WastLexer::Fill(size_t need) {
  if (eof_) {
    return Result::Error;
  }
  size_t free = next_pos_ - buffer_;
  assert(static_cast<size_t>(cursor_ - buffer_) >= free);
  // Our buffer is too small, need to realloc.
  if (free < need) {
    char* old_buffer = buffer_;
    size_t old_buffer_size = buffer_size_;
    size_t new_buffer_size =
        old_buffer_size ? old_buffer_size * 2 : INITIAL_LEXER_BUFFER_SIZE;
    // Make sure there is enough space for the bytes requested (need) and an
    // additional YYMAXFILL bytes which is needed for the re2c lexer
    // implementation when the eof is reached.
    while ((new_buffer_size - old_buffer_size) + free < need + YYMAXFILL)
      new_buffer_size *= 2;

    char* new_buffer = new char[new_buffer_size];
    if (limit_ > next_pos_) {
      memmove(new_buffer, next_pos_, limit_ - next_pos_);
    }
    buffer_ = new_buffer;
    buffer_size_ = new_buffer_size;
    next_pos_ = new_buffer + (next_pos_ - old_buffer) - free;
    marker_ = new_buffer + (marker_ - old_buffer) - free;
    cursor_ = new_buffer + (cursor_ - old_buffer) - free;
    limit_ = new_buffer + (limit_ - old_buffer) - free;
    buffer_file_offset_ += free;
    free += new_buffer_size - old_buffer_size;
    delete[] old_buffer;
  } else {
    // Shift everything down to make more room in the buffer.
    if (limit_ > next_pos_) {
      memmove(buffer_, next_pos_, limit_ - next_pos_);
    }
    next_pos_ -= free;
    marker_ -= free;
    cursor_ -= free;
    limit_ -= free;
    buffer_file_offset_ += free;
  }
  // Read the new data into the buffer.
  limit_ += source_->Fill(limit_, free);

  // If at the end of file, need to fill YYMAXFILL more characters with "fake
  // characters", that are not a lexeme nor a lexeme suffix. see
  // http://re2c.org/examples/example_03.html.
  if (limit_ < buffer_ + buffer_size_ - YYMAXFILL) {
    eof_ = true;
    // Fill with 0xff, since that is an invalid utf-8 byte.
    memset(limit_, 0xff, YYMAXFILL);
    limit_ += YYMAXFILL;
  }
  return Result::Ok;
}

Token WastLexer::GetToken(WastParser* parser) {
#line 183 "src/prebuilt/wast-lexer-gen.cc"

enum YYCONDTYPE {
	YYCOND_i,
	YYCOND_BAD_TEXT,
	YYCOND_LINE_COMMENT,
	YYCOND_BLOCK_COMMENT,
};

#line 180 "src/wast-lexer.cc"
  YYCONDTYPE cond = YYCOND_i;  // i is the initial state.

  for (;;) {
    next_pos_ = cursor_;
    
#line 198 "src/prebuilt/wast-lexer-gen.cc"
{
	unsigned char yych;
	unsigned int yyaccept = 0;
	if (cond < 2) {
		if (cond < 1) {
			goto YYCOND_i;
		} else {
			goto YYCOND_BAD_TEXT;
		}
	} else {
		if (cond < 3) {
			goto YYCOND_LINE_COMMENT;
		} else {
			goto YYCOND_BLOCK_COMMENT;
		}
	}
/* *********************************** */
YYCOND_BAD_TEXT:
	if ((limit_ - cursor_) < 5) FILL(5);
	yych = *cursor_;
	if (yych <= 0x7F) {
		if (yych <= '!') {
			if (yych == '\n') goto yy5;
			if (yych >= ' ') goto yy7;
		} else {
			if (yych <= '"') goto yy9;
			if (yych == '\\') goto yy11;
			goto yy7;
		}
	} else {
		if (yych <= 0xEF) {
			if (yych <= 0xC1) goto yy12;
			if (yych <= 0xDF) goto yy14;
			if (yych <= 0xE0) goto yy15;
			goto yy16;
		} else {
			if (yych <= 0xF0) goto yy17;
			if (yych <= 0xF3) goto yy18;
			if (yych <= 0xF4) goto yy19;
			goto yy12;
		}
	}
	++cursor_;
yy4:
#line 242 "src/wast-lexer.cc"
	{ ERROR("illegal character in string");
                                  continue; }
#line 246 "src/prebuilt/wast-lexer-gen.cc"
yy5:
	++cursor_;
	BEGIN(YYCOND_i);
#line 235 "src/wast-lexer.cc"
	{ ERROR("newline in string");
                                  NEWLINE;
                                  continue; }
#line 254 "src/prebuilt/wast-lexer-gen.cc"
yy7:
	++cursor_;
#line 234 "src/wast-lexer.cc"
	{ continue; }
#line 259 "src/prebuilt/wast-lexer-gen.cc"
yy9:
	++cursor_;
	BEGIN(YYCOND_i);
#line 241 "src/wast-lexer.cc"
	{ RETURN_TEXT(Text); }
#line 265 "src/prebuilt/wast-lexer-gen.cc"
yy11:
	yyaccept = 0;
	yych = *(marker_ = ++cursor_);
	if (yych <= 'f') {
		if (yych <= '/') {
			if (yych <= '!') {
				if (yych == '\n') goto yy4;
				goto yy20;
			} else {
				if (yych <= '"') goto yy7;
				if (yych == '\'') goto yy7;
				goto yy20;
			}
		} else {
			if (yych <= 'F') {
				if (yych <= '9') goto yy22;
				if (yych <= '@') goto yy20;
				goto yy22;
			} else {
				if (yych == '\\') goto yy7;
				if (yych <= '`') goto yy20;
				goto yy22;
			}
		}
	} else {
		if (yych <= 0x7F) {
			if (yych <= 'q') {
				if (yych == 'n') goto yy7;
				goto yy20;
			} else {
				if (yych == 's') goto yy20;
				if (yych <= 't') goto yy7;
				goto yy20;
			}
		} else {
			if (yych <= 0xEF) {
				if (yych <= 0xC1) goto yy4;
				if (yych <= 0xDF) goto yy23;
				if (yych <= 0xE0) goto yy25;
				goto yy26;
			} else {
				if (yych <= 0xF0) goto yy27;
				if (yych <= 0xF3) goto yy28;
				if (yych <= 0xF4) goto yy29;
				goto yy4;
			}
		}
	}
yy12:
	++cursor_;
yy13:
#line 244 "src/wast-lexer.cc"
	{ MAYBE_MALFORMED_UTF8(" in string"); }
#line 319 "src/prebuilt/wast-lexer-gen.cc"
yy14:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy13;
	if (yych <= 0xBF) goto yy7;
	goto yy13;
yy15:
	yyaccept = 1;
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x9F) goto yy13;
	if (yych <= 0xBF) goto yy30;
	goto yy13;
yy16:
	yyaccept = 1;
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x7F) goto yy13;
	if (yych <= 0xBF) goto yy30;
	goto yy13;
yy17:
	yyaccept = 1;
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x8F) goto yy13;
	if (yych <= 0xBF) goto yy31;
	goto yy13;
yy18:
	yyaccept = 1;
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x7F) goto yy13;
	if (yych <= 0xBF) goto yy31;
	goto yy13;
yy19:
	yyaccept = 1;
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x7F) goto yy13;
	if (yych <= 0x8F) goto yy31;
	goto yy13;
yy20:
	++cursor_;
yy21:
#line 238 "src/wast-lexer.cc"
	{ ERROR("bad escape \"%.*s\"",
                                        static_cast<int>(yyleng), yytext);
                                  continue; }
#line 362 "src/prebuilt/wast-lexer-gen.cc"
yy22:
	yych = *++cursor_;
	if (yych <= '@') {
		if (yych <= '/') goto yy21;
		if (yych <= '9') goto yy7;
		goto yy21;
	} else {
		if (yych <= 'F') goto yy7;
		if (yych <= '`') goto yy21;
		if (yych <= 'f') goto yy7;
		goto yy21;
	}
yy23:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy24;
	if (yych <= 0xBF) goto yy20;
yy24:
	cursor_ = marker_;
	if (yyaccept == 0) {
		goto yy4;
	} else {
		goto yy13;
	}
yy25:
	yych = *++cursor_;
	if (yych <= 0x9F) goto yy24;
	if (yych <= 0xBF) goto yy23;
	goto yy24;
yy26:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy24;
	if (yych <= 0xBF) goto yy23;
	goto yy24;
yy27:
	yych = *++cursor_;
	if (yych <= 0x8F) goto yy24;
	if (yych <= 0xBF) goto yy26;
	goto yy24;
yy28:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy24;
	if (yych <= 0xBF) goto yy26;
	goto yy24;
yy29:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy24;
	if (yych <= 0x8F) goto yy26;
	goto yy24;
yy30:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy24;
	if (yych <= 0xBF) goto yy7;
	goto yy24;
yy31:
	++cursor_;
	if ((yych = *cursor_) <= 0x7F) goto yy24;
	if (yych <= 0xBF) goto yy30;
	goto yy24;
/* *********************************** */
YYCOND_BLOCK_COMMENT:
	if ((limit_ - cursor_) < 4) FILL(4);
	yych = *cursor_;
	if (yych <= 0x7F) {
		if (yych <= '\'') {
			if (yych == '\n') goto yy36;
		} else {
			if (yych <= '(') goto yy38;
			if (yych == ';') goto yy39;
		}
	} else {
		if (yych <= 0xEF) {
			if (yych <= 0xC1) goto yy40;
			if (yych <= 0xDF) goto yy42;
			if (yych <= 0xE0) goto yy43;
			goto yy44;
		} else {
			if (yych <= 0xF0) goto yy45;
			if (yych <= 0xF3) goto yy46;
			if (yych <= 0xF4) goto yy47;
			goto yy40;
		}
	}
yy34:
	++cursor_;
yy35:
#line 761 "src/wast-lexer.cc"
	{ continue; }
#line 450 "src/prebuilt/wast-lexer-gen.cc"
yy36:
	++cursor_;
#line 760 "src/wast-lexer.cc"
	{ NEWLINE; continue; }
#line 455 "src/prebuilt/wast-lexer-gen.cc"
yy38:
	yych = *++cursor_;
	if (yych == ';') goto yy48;
	goto yy35;
yy39:
	yych = *++cursor_;
	if (yych == ')') goto yy50;
	goto yy35;
yy40:
	++cursor_;
yy41:
#line 762 "src/wast-lexer.cc"
	{ MAYBE_MALFORMED_UTF8(" in block comment"); }
#line 469 "src/prebuilt/wast-lexer-gen.cc"
yy42:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy41;
	if (yych <= 0xBF) goto yy34;
	goto yy41;
yy43:
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x9F) goto yy41;
	if (yych <= 0xBF) goto yy52;
	goto yy41;
yy44:
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x7F) goto yy41;
	if (yych <= 0xBF) goto yy52;
	goto yy41;
yy45:
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x8F) goto yy41;
	if (yych <= 0xBF) goto yy54;
	goto yy41;
yy46:
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x7F) goto yy41;
	if (yych <= 0xBF) goto yy54;
	goto yy41;
yy47:
	yych = *(marker_ = ++cursor_);
	if (yych <= 0x7F) goto yy41;
	if (yych <= 0x8F) goto yy54;
	goto yy41;
yy48:
	++cursor_;
#line 755 "src/wast-lexer.cc"
	{ COMMENT_NESTING++; continue; }
#line 504 "src/prebuilt/wast-lexer-gen.cc"
yy50:
	++cursor_;
#line 756 "src/wast-lexer.cc"
	{ if (--COMMENT_NESTING == 0) {
                                    BEGIN(YYCOND_i);
                                  }
                                  continue; }
#line 512 "src/prebuilt/wast-lexer-gen.cc"
yy52:
	yych = *++cursor_;
	if (yych <= 0x7F) goto yy53;
	if (yych <= 0xBF) goto yy34;
yy53:
	cursor_ = marker_;
	goto yy41;
yy54:
	++cursor_;
	if ((yych = *cursor_) <= 0x7F) goto yy53;
	if (yych <= 0xBF) goto yy52;
	goto yy53;
/* *********************************** */
YYCOND_LINE_COMMENT:
	{
		static const unsigned char yybm[] = {
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128,   0, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			128, 128, 128, 128, 128, 128, 128, 128, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
		};
		if ((limit_ - cursor_) < 2) FILL(2);
		yych = *cursor_;
		if (yybm[0+yych] & 128) {
			goto yy57;
		}
		if (yych <= 0xE0) {
			if (yych <= '\n') goto yy60;
			if (yych <= 0xC1) goto yy62;
			if (yych <= 0xDF) goto yy64;
			goto yy65;
		} else {
			if (yych <= 0xF0) {
				if (yych <= 0xEF) goto yy66;
				goto yy67;
			} else {
				if (yych <= 0xF3) goto yy68;
				if (yych <= 0xF4) goto yy69;
				goto yy62;
			}
		}
yy57:
		yyaccept = 0;
		marker_ = ++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yybm[0+yych] & 128) {
			goto yy57;
		}
		if (yych <= 0xEF) {
			if (yych <= 0xC1) goto yy59;
			if (yych <= 0xDF) goto yy70;
			if (yych <= 0xE0) goto yy72;
			goto yy73;
		} else {
			if (yych <= 0xF0) goto yy74;
			if (yych <= 0xF3) goto yy75;
			if (yych <= 0xF4) goto yy76;
		}
yy59:
#line 753 "src/wast-lexer.cc"
		{ continue; }
#line 603 "src/prebuilt/wast-lexer-gen.cc"
yy60:
		++cursor_;
		BEGIN(YYCOND_i);
#line 752 "src/wast-lexer.cc"
		{ NEWLINE; continue; }
#line 609 "src/prebuilt/wast-lexer-gen.cc"
yy62:
		++cursor_;
yy63:
#line 767 "src/wast-lexer.cc"
		{ MAYBE_MALFORMED_UTF8(""); }
#line 615 "src/prebuilt/wast-lexer-gen.cc"
yy64:
		yych = *++cursor_;
		if (yych <= 0x7F) goto yy63;
		if (yych <= 0xBF) goto yy57;
		goto yy63;
yy65:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x9F) goto yy63;
		if (yych <= 0xBF) goto yy70;
		goto yy63;
yy66:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x7F) goto yy63;
		if (yych <= 0xBF) goto yy70;
		goto yy63;
yy67:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x8F) goto yy63;
		if (yych <= 0xBF) goto yy73;
		goto yy63;
yy68:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x7F) goto yy63;
		if (yych <= 0xBF) goto yy73;
		goto yy63;
yy69:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x7F) goto yy63;
		if (yych <= 0x8F) goto yy73;
		goto yy63;
yy70:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy71;
		if (yych <= 0xBF) goto yy57;
yy71:
		cursor_ = marker_;
		if (yyaccept == 0) {
			goto yy59;
		} else {
			goto yy63;
		}
yy72:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x9F) goto yy71;
		if (yych <= 0xBF) goto yy70;
		goto yy71;
yy73:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy71;
		if (yych <= 0xBF) goto yy70;
		goto yy71;
yy74:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x8F) goto yy71;
		if (yych <= 0xBF) goto yy73;
		goto yy71;
yy75:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy71;
		if (yych <= 0xBF) goto yy73;
		goto yy71;
yy76:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy71;
		if (yych <= 0x8F) goto yy73;
		goto yy71;
	}
/* *********************************** */
YYCOND_i:
	{
		static const unsigned char yybm[] = {
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   4,   0,   0,   0,   4,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			 36, 104,   0, 104, 104, 104, 104, 104, 
			 32,  32, 104, 104,  40, 104, 104, 104, 
			248, 248, 248, 248, 248, 248, 248, 248, 
			248, 248, 104,  32, 104, 104, 104, 104, 
			104, 232, 232, 232, 232, 232, 232, 104, 
			104, 104, 104, 104, 104, 104, 104, 104, 
			104, 104, 104, 104, 104, 104, 104, 104, 
			104, 104, 104,  40,  72,  40, 104, 104, 
			104, 232, 232, 232, 232, 232, 232, 104, 
			104, 104, 104, 104, 104, 104, 104, 104, 
			104, 104, 104, 104, 104, 104, 104, 104, 
			104, 104, 104,  40, 104,  40, 104,  32, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
			  0,   0,   0,   0,   0,   0,   0,   0, 
		};
		if ((limit_ - cursor_) < 29) FILL(29);
		yych = *cursor_;
		if (yybm[0+yych] & 4) {
			goto yy81;
		}
		if (yych <= 'e') {
			if (yych <= '+') {
				if (yych <= '#') {
					if (yych <= 0x1F) {
						if (yych <= 0x08) goto yy79;
						if (yych <= '\n') goto yy84;
					} else {
						if (yych == '"') goto yy89;
						goto yy86;
					}
				} else {
					if (yych <= '(') {
						if (yych <= '$') goto yy91;
						if (yych <= '\'') goto yy86;
						goto yy92;
					} else {
						if (yych <= ')') goto yy94;
						if (yych <= '*') goto yy86;
						goto yy96;
					}
				}
			} else {
				if (yych <= ':') {
					if (yych <= '/') {
						if (yych == '-') goto yy96;
						goto yy86;
					} else {
						if (yych <= '0') goto yy97;
						if (yych <= '9') goto yy99;
						goto yy86;
					}
				} else {
					if (yych <= 'a') {
						if (yych <= ';') goto yy101;
						if (yych <= '`') goto yy86;
						goto yy102;
					} else {
						if (yych <= 'b') goto yy103;
						if (yych <= 'c') goto yy104;
						if (yych <= 'd') goto yy105;
						goto yy106;
					}
				}
			}
		} else {
			if (yych <= 's') {
				if (yych <= 'l') {
					if (yych <= 'h') {
						if (yych <= 'f') goto yy107;
						if (yych <= 'g') goto yy108;
						goto yy86;
					} else {
						if (yych <= 'i') goto yy109;
						if (yych <= 'k') goto yy86;
						goto yy110;
					}
				} else {
					if (yych <= 'o') {
						if (yych <= 'm') goto yy111;
						if (yych <= 'n') goto yy112;
						goto yy113;
					} else {
						if (yych <= 'p') goto yy114;
						if (yych <= 'q') goto yy115;
						if (yych <= 'r') goto yy116;
						goto yy117;
					}
				}
			} else {
				if (yych <= 0xC1) {
					if (yych <= 'v') {
						if (yych <= 't') goto yy118;
						if (yych <= 'u') goto yy119;
						goto yy120;
					} else {
						if (yych <= '~') goto yy86;
						if (yych >= 0x80) goto yy121;
					}
				} else {
					if (yych <= 0xEF) {
						if (yych <= 0xDF) goto yy123;
						if (yych <= 0xE0) goto yy124;
						goto yy125;
					} else {
						if (yych <= 0xF0) goto yy126;
						if (yych <= 0xF3) goto yy127;
						if (yych <= 0xF4) goto yy128;
						goto yy121;
					}
				}
			}
		}
yy79:
		++cursor_;
yy80:
#line 766 "src/wast-lexer.cc"
		{ ERROR("unexpected char"); continue; }
#line 839 "src/prebuilt/wast-lexer-gen.cc"
yy81:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yybm[0+yych] & 4) {
			goto yy81;
		}
#line 764 "src/wast-lexer.cc"
		{ continue; }
#line 849 "src/prebuilt/wast-lexer-gen.cc"
yy84:
		++cursor_;
#line 763 "src/wast-lexer.cc"
		{ NEWLINE; continue; }
#line 854 "src/prebuilt/wast-lexer-gen.cc"
yy86:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
yy87:
		if (yybm[0+yych] & 8) {
			goto yy86;
		}
yy88:
#line 765 "src/wast-lexer.cc"
		{ RETURN_TEXT(Reserved); }
#line 866 "src/prebuilt/wast-lexer-gen.cc"
yy89:
		yyaccept = 0;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x1F) goto yy90;
		if (yych <= 0x7F) goto yy130;
		if (yych <= 0xC1) goto yy90;
		if (yych <= 0xF4) goto yy130;
yy90:
		BEGIN(YYCOND_BAD_TEXT);
#line 233 "src/wast-lexer.cc"
		{ continue; }
#line 878 "src/prebuilt/wast-lexer-gen.cc"
yy91:
		yych = *++cursor_;
		if (yych <= '\'') {
			if (yych == '!') goto yy142;
			if (yych <= '"') goto yy88;
			goto yy142;
		} else {
			if (yych <= ':') {
				if (yych <= ')') goto yy88;
				goto yy142;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy142;
				goto yy88;
			}
		}
yy92:
		++cursor_;
		if ((yych = *cursor_) == ';') goto yy144;
#line 224 "src/wast-lexer.cc"
		{ RETURN(Lpar); }
#line 900 "src/prebuilt/wast-lexer-gen.cc"
yy94:
		++cursor_;
#line 225 "src/wast-lexer.cc"
		{ RETURN(Rpar); }
#line 905 "src/prebuilt/wast-lexer-gen.cc"
yy96:
		yych = *++cursor_;
		if (yych <= 'h') {
			if (yych <= '/') goto yy87;
			if (yych <= '0') goto yy146;
			if (yych <= '9') goto yy148;
			goto yy87;
		} else {
			if (yych <= 'i') goto yy150;
			if (yych == 'n') goto yy151;
			goto yy87;
		}
yy97:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 16) {
			goto yy99;
		}
		if (yych <= ';') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych >= '#') goto yy86;
			} else {
				if (yych <= '-') {
					if (yych >= '*') goto yy86;
				} else {
					if (yych <= '.') goto yy152;
					if (yych <= ':') goto yy86;
				}
			}
		} else {
			if (yych <= 'd') {
				if (yych <= 'E') {
					if (yych <= 'D') goto yy86;
					goto yy154;
				} else {
					if (yych == '_') goto yy155;
					goto yy86;
				}
			} else {
				if (yych <= 'w') {
					if (yych <= 'e') goto yy154;
					goto yy86;
				} else {
					if (yych <= 'x') goto yy156;
					if (yych <= '~') goto yy86;
				}
			}
		}
yy98:
#line 226 "src/wast-lexer.cc"
		{ RETURN_LITERAL(Nat, Int); }
#line 957 "src/prebuilt/wast-lexer-gen.cc"
yy99:
		++cursor_;
		if ((limit_ - cursor_) < 3) FILL(3);
		yych = *cursor_;
		if (yybm[0+yych] & 16) {
			goto yy99;
		}
		if (yych <= ':') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy98;
				goto yy86;
			} else {
				if (yych <= ')') goto yy98;
				if (yych == '.') goto yy152;
				goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') goto yy98;
				if (yych == 'E') goto yy154;
				goto yy86;
			} else {
				if (yych <= 'd') {
					if (yych <= '_') goto yy155;
					goto yy86;
				} else {
					if (yych <= 'e') goto yy154;
					if (yych <= '~') goto yy86;
					goto yy98;
				}
			}
		}
yy101:
		yych = *++cursor_;
		if (yych == ';') goto yy157;
		goto yy80;
yy102:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'l') goto yy159;
			if (yych <= 'm') goto yy87;
			goto yy160;
		} else {
			if (yych <= 'r') goto yy87;
			if (yych <= 's') goto yy161;
			if (yych <= 't') goto yy162;
			goto yy87;
		}
yy103:
		yych = *++cursor_;
		if (yych <= 'k') {
			if (yych == 'i') goto yy163;
			goto yy87;
		} else {
			if (yych <= 'l') goto yy164;
			if (yych == 'r') goto yy165;
			goto yy87;
		}
yy104:
		yych = *++cursor_;
		if (yych == 'a') goto yy167;
		if (yych == 'u') goto yy168;
		goto yy87;
yy105:
		yych = *++cursor_;
		if (yych == 'a') goto yy169;
		if (yych == 'r') goto yy170;
		goto yy87;
yy106:
		yych = *++cursor_;
		switch (yych) {
		case 'l':	goto yy171;
		case 'n':	goto yy172;
		case 'v':	goto yy173;
		case 'x':	goto yy174;
		default:	goto yy87;
		}
yy107:
		yych = *++cursor_;
		if (yych <= '5') {
			if (yych == '3') goto yy175;
			goto yy87;
		} else {
			if (yych <= '6') goto yy176;
			if (yych == 'u') goto yy177;
			goto yy87;
		}
yy108:
		yych = *++cursor_;
		if (yych <= 'k') {
			if (yych == 'e') goto yy178;
			goto yy87;
		} else {
			if (yych <= 'l') goto yy179;
			if (yych == 'r') goto yy180;
			goto yy87;
		}
yy109:
		yych = *++cursor_;
		if (yych <= '7') {
			if (yych <= '2') {
				if (yych == '1') goto yy181;
				goto yy87;
			} else {
				if (yych <= '3') goto yy182;
				if (yych == '6') goto yy183;
				goto yy87;
			}
		} else {
			if (yych <= 'f') {
				if (yych <= '8') goto yy184;
				if (yych <= 'e') goto yy87;
				goto yy185;
			} else {
				if (yych <= 'l') goto yy87;
				if (yych <= 'm') goto yy187;
				if (yych <= 'n') goto yy188;
				goto yy87;
			}
		}
yy110:
		yych = *++cursor_;
		if (yych == 'o') goto yy189;
		goto yy87;
yy111:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'e') goto yy190;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy191;
			if (yych == 'u') goto yy192;
			goto yy87;
		}
yy112:
		yych = *++cursor_;
		if (yych == 'a') goto yy193;
		if (yych == 'o') goto yy194;
		goto yy87;
yy113:
		yych = *++cursor_;
		if (yych == 'f') goto yy195;
		goto yy87;
yy114:
		yych = *++cursor_;
		if (yych == 'a') goto yy196;
		goto yy87;
yy115:
		yych = *++cursor_;
		if (yych == 'u') goto yy197;
		goto yy87;
yy116:
		yych = *++cursor_;
		if (yych == 'e') goto yy198;
		goto yy87;
yy117:
		yych = *++cursor_;
		if (yych <= 'g') {
			if (yych == 'e') goto yy199;
			goto yy87;
		} else {
			if (yych <= 'h') goto yy200;
			if (yych == 't') goto yy201;
			goto yy87;
		}
yy118:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy202;
		case 'e':	goto yy203;
		case 'h':	goto yy204;
		case 'r':	goto yy205;
		case 'y':	goto yy206;
		default:	goto yy87;
		}
yy119:
		yych = *++cursor_;
		if (yych == 'n') goto yy207;
		goto yy87;
yy120:
		yych = *++cursor_;
		if (yych == '1') goto yy208;
		if (yych == '8') goto yy209;
		goto yy87;
yy121:
		++cursor_;
yy122:
#line 767 "src/wast-lexer.cc"
		{ MAYBE_MALFORMED_UTF8(""); }
#line 1148 "src/prebuilt/wast-lexer-gen.cc"
yy123:
		yych = *++cursor_;
		if (yych <= 0x7F) goto yy122;
		if (yych <= 0xBF) goto yy79;
		goto yy122;
yy124:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x9F) goto yy122;
		if (yych <= 0xBF) goto yy210;
		goto yy122;
yy125:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x7F) goto yy122;
		if (yych <= 0xBF) goto yy210;
		goto yy122;
yy126:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x8F) goto yy122;
		if (yych <= 0xBF) goto yy211;
		goto yy122;
yy127:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x7F) goto yy122;
		if (yych <= 0xBF) goto yy211;
		goto yy122;
yy128:
		yyaccept = 1;
		yych = *(marker_ = ++cursor_);
		if (yych <= 0x7F) goto yy122;
		if (yych <= 0x8F) goto yy211;
		goto yy122;
yy129:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
yy130:
		if (yybm[0+yych] & 32) {
			goto yy129;
		}
		if (yych <= 0xDF) {
			if (yych <= '"') {
				if (yych >= ' ') goto yy132;
			} else {
				if (yych <= '\\') goto yy134;
				if (yych >= 0xC2) goto yy135;
			}
		} else {
			if (yych <= 0xF0) {
				if (yych <= 0xE0) goto yy136;
				if (yych <= 0xEF) goto yy137;
				goto yy138;
			} else {
				if (yych <= 0xF3) goto yy139;
				if (yych <= 0xF4) goto yy140;
			}
		}
yy131:
		cursor_ = marker_;
		if (yyaccept == 0) {
			goto yy90;
		} else {
			goto yy122;
		}
yy132:
		++cursor_;
#line 232 "src/wast-lexer.cc"
		{ RETURN_TEXT(Text); }
#line 1220 "src/prebuilt/wast-lexer-gen.cc"
yy134:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '[') {
			if (yych <= '\'') {
				if (yych == '"') goto yy129;
				if (yych <= '&') goto yy131;
				goto yy129;
			} else {
				if (yych <= '9') {
					if (yych <= '/') goto yy131;
					goto yy212;
				} else {
					if (yych <= '@') goto yy131;
					if (yych <= 'F') goto yy212;
					goto yy131;
				}
			}
		} else {
			if (yych <= 'n') {
				if (yych <= '`') {
					if (yych <= '\\') goto yy129;
					goto yy131;
				} else {
					if (yych <= 'f') goto yy212;
					if (yych <= 'm') goto yy131;
					goto yy129;
				}
			} else {
				if (yych <= 'r') {
					if (yych <= 'q') goto yy131;
					goto yy129;
				} else {
					if (yych == 't') goto yy129;
					goto yy131;
				}
			}
		}
yy135:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy131;
		if (yych <= 0xBF) goto yy129;
		goto yy131;
yy136:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x9F) goto yy131;
		if (yych <= 0xBF) goto yy135;
		goto yy131;
yy137:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy131;
		if (yych <= 0xBF) goto yy135;
		goto yy131;
yy138:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x8F) goto yy131;
		if (yych <= 0xBF) goto yy137;
		goto yy131;
yy139:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy131;
		if (yych <= 0xBF) goto yy137;
		goto yy131;
yy140:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= 0x7F) goto yy131;
		if (yych <= 0x8F) goto yy137;
		goto yy131;
yy141:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
yy142:
		if (yybm[0+yych] & 64) {
			goto yy141;
		}
		if (yych <= ')') goto yy143;
		if (yych <= ',') goto yy86;
		if (yych <= ';') goto yy143;
		if (yych <= '}') goto yy86;
yy143:
#line 748 "src/wast-lexer.cc"
		{ RETURN_TEXT(Var); }
#line 1317 "src/prebuilt/wast-lexer-gen.cc"
yy144:
		++cursor_;
		BEGIN(YYCOND_BLOCK_COMMENT);
#line 754 "src/wast-lexer.cc"
		{ COMMENT_NESTING = 1; continue; }
#line 1323 "src/prebuilt/wast-lexer-gen.cc"
yy146:
		++cursor_;
		if ((yych = *cursor_) <= ':') {
			if (yych <= ')') {
				if (yych <= '!') {
					if (yych >= '!') goto yy86;
				} else {
					if (yych <= '"') goto yy147;
					if (yych <= '\'') goto yy86;
				}
			} else {
				if (yych <= '.') {
					if (yych <= '-') goto yy86;
					goto yy152;
				} else {
					if (yych <= '/') goto yy86;
					if (yych <= '9') goto yy148;
					goto yy86;
				}
			}
		} else {
			if (yych <= '_') {
				if (yych <= 'D') {
					if (yych >= '<') goto yy86;
				} else {
					if (yych <= 'E') goto yy154;
					if (yych <= '^') goto yy86;
					goto yy213;
				}
			} else {
				if (yych <= 'w') {
					if (yych == 'e') goto yy154;
					goto yy86;
				} else {
					if (yych <= 'x') goto yy214;
					if (yych <= '~') goto yy86;
				}
			}
		}
yy147:
#line 227 "src/wast-lexer.cc"
		{ RETURN_LITERAL(Int, Int); }
#line 1366 "src/prebuilt/wast-lexer-gen.cc"
yy148:
		++cursor_;
		if ((limit_ - cursor_) < 3) FILL(3);
		yych = *cursor_;
		if (yych <= '9') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy147;
				goto yy86;
			} else {
				if (yych <= '-') {
					if (yych <= ')') goto yy147;
					goto yy86;
				} else {
					if (yych <= '.') goto yy152;
					if (yych <= '/') goto yy86;
					goto yy148;
				}
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') {
					if (yych <= ':') goto yy86;
					goto yy147;
				} else {
					if (yych == 'E') goto yy154;
					goto yy86;
				}
			} else {
				if (yych <= 'd') {
					if (yych <= '_') goto yy213;
					goto yy86;
				} else {
					if (yych <= 'e') goto yy154;
					if (yych <= '~') goto yy86;
					goto yy147;
				}
			}
		}
yy150:
		yych = *++cursor_;
		if (yych == 'n') goto yy215;
		goto yy87;
yy151:
		yych = *++cursor_;
		if (yych == 'a') goto yy193;
		goto yy87;
yy152:
		++cursor_;
		if ((yych = *cursor_) <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy153;
				if (yych <= '/') goto yy86;
				goto yy216;
			}
		} else {
			if (yych <= 'E') {
				if (yych == ';') goto yy153;
				if (yych <= 'D') goto yy86;
				goto yy154;
			} else {
				if (yych == 'e') goto yy154;
				if (yych <= '~') goto yy86;
			}
		}
yy153:
#line 228 "src/wast-lexer.cc"
		{ RETURN_LITERAL(Float, Float); }
#line 1438 "src/prebuilt/wast-lexer-gen.cc"
yy154:
		yych = *++cursor_;
		if (yych <= ',') {
			if (yych == '+') goto yy218;
			goto yy87;
		} else {
			if (yych <= '-') goto yy218;
			if (yych <= '/') goto yy87;
			if (yych <= '9') goto yy219;
			goto yy87;
		}
yy155:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yybm[0+yych] & 16) {
			goto yy99;
		}
		if (yych <= '\'') {
			if (yych == '!') goto yy86;
			if (yych <= '"') goto yy88;
			goto yy86;
		} else {
			if (yych <= ':') {
				if (yych <= ')') goto yy88;
				goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy156:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yybm[0+yych] & 128) {
			goto yy221;
		}
		if (yych <= '\'') {
			if (yych == '!') goto yy86;
			if (yych <= '"') goto yy88;
			goto yy86;
		} else {
			if (yych <= ':') {
				if (yych <= ')') goto yy88;
				goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy157:
		++cursor_;
		BEGIN(YYCOND_LINE_COMMENT);
#line 751 "src/wast-lexer.cc"
		{ continue; }
#line 1497 "src/prebuilt/wast-lexer-gen.cc"
yy159:
		yych = *++cursor_;
		if (yych == 'i') goto yy223;
		goto yy87;
yy160:
		yych = *++cursor_;
		if (yych == 'y') goto yy224;
		goto yy87;
yy161:
		yych = *++cursor_;
		if (yych == 's') goto yy225;
		goto yy87;
yy162:
		yych = *++cursor_;
		if (yych == 'o') goto yy226;
		goto yy87;
yy163:
		yych = *++cursor_;
		if (yych == 'n') goto yy227;
		goto yy87;
yy164:
		yych = *++cursor_;
		if (yych == 'o') goto yy228;
		goto yy87;
yy165:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy166;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy229;
				if (yych <= '~') goto yy86;
			}
		}
yy166:
#line 259 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Br); }
#line 1542 "src/prebuilt/wast-lexer-gen.cc"
yy167:
		yych = *++cursor_;
		if (yych == 'l') goto yy230;
		if (yych == 't') goto yy231;
		goto yy87;
yy168:
		yych = *++cursor_;
		if (yych == 'r') goto yy232;
		goto yy87;
yy169:
		yych = *++cursor_;
		if (yych == 't') goto yy233;
		goto yy87;
yy170:
		yych = *++cursor_;
		if (yych == 'o') goto yy234;
		goto yy87;
yy171:
		yych = *++cursor_;
		if (yych == 'e') goto yy235;
		if (yych == 's') goto yy236;
		goto yy87;
yy172:
		yych = *++cursor_;
		if (yych == 'd') goto yy237;
		goto yy87;
yy173:
		yych = *++cursor_;
		if (yych == 'e') goto yy239;
		goto yy87;
yy174:
		yych = *++cursor_;
		if (yych == 'p') goto yy240;
		goto yy87;
yy175:
		yych = *++cursor_;
		if (yych == '2') goto yy241;
		goto yy87;
yy176:
		yych = *++cursor_;
		if (yych == '4') goto yy243;
		goto yy87;
yy177:
		yych = *++cursor_;
		if (yych == 'n') goto yy245;
		goto yy87;
yy178:
		yych = *++cursor_;
		if (yych == 't') goto yy246;
		goto yy87;
yy179:
		yych = *++cursor_;
		if (yych == 'o') goto yy248;
		goto yy87;
yy180:
		yych = *++cursor_;
		if (yych == 'o') goto yy249;
		goto yy87;
yy181:
		yych = *++cursor_;
		if (yych == '6') goto yy250;
		goto yy87;
yy182:
		yych = *++cursor_;
		if (yych == '2') goto yy251;
		goto yy87;
yy183:
		yych = *++cursor_;
		if (yych == '4') goto yy253;
		goto yy87;
yy184:
		yych = *++cursor_;
		if (yych == 'x') goto yy255;
		goto yy87;
yy185:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 255 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(If); }
#line 1624 "src/prebuilt/wast-lexer-gen.cc"
yy187:
		yych = *++cursor_;
		if (yych == 'p') goto yy256;
		goto yy87;
yy188:
		yych = *++cursor_;
		if (yych == 'f') goto yy257;
		if (yych == 'v') goto yy259;
		goto yy87;
yy189:
		yych = *++cursor_;
		if (yych == 'c') goto yy260;
		if (yych == 'o') goto yy261;
		goto yy87;
yy190:
		yych = *++cursor_;
		if (yych == 'm') goto yy262;
		goto yy87;
yy191:
		yych = *++cursor_;
		if (yych == 'd') goto yy263;
		goto yy87;
yy192:
		yych = *++cursor_;
		if (yych == 't') goto yy264;
		goto yy87;
yy193:
		yych = *++cursor_;
		if (yych == 'n') goto yy266;
		goto yy87;
yy194:
		yych = *++cursor_;
		if (yych == 'p') goto yy268;
		goto yy87;
yy195:
		yych = *++cursor_;
		if (yych == 'f') goto yy270;
		goto yy87;
yy196:
		yych = *++cursor_;
		if (yych <= 'q') goto yy87;
		if (yych <= 'r') goto yy271;
		if (yych <= 's') goto yy272;
		goto yy87;
yy197:
		yych = *++cursor_;
		if (yych == 'o') goto yy273;
		goto yy87;
yy198:
		yych = *++cursor_;
		if (yych <= 'g') {
			if (yych <= 'e') goto yy87;
			if (yych <= 'f') goto yy274;
			goto yy275;
		} else {
			if (yych <= 'r') goto yy87;
			if (yych <= 's') goto yy276;
			if (yych <= 't') goto yy277;
			goto yy87;
		}
yy199:
		yych = *++cursor_;
		if (yych == 'l') goto yy278;
		if (yych == 't') goto yy279;
		goto yy87;
yy200:
		yych = *++cursor_;
		if (yych == 'a') goto yy280;
		goto yy87;
yy201:
		yych = *++cursor_;
		if (yych == 'a') goto yy281;
		goto yy87;
yy202:
		yych = *++cursor_;
		if (yych == 'b') goto yy282;
		goto yy87;
yy203:
		yych = *++cursor_;
		if (yych == 'e') goto yy283;
		goto yy87;
yy204:
		yych = *++cursor_;
		if (yych == 'e') goto yy284;
		if (yych == 'r') goto yy285;
		goto yy87;
yy205:
		yych = *++cursor_;
		if (yych == 'y') goto yy286;
		goto yy87;
yy206:
		yych = *++cursor_;
		if (yych == 'p') goto yy288;
		goto yy87;
yy207:
		yych = *++cursor_;
		if (yych == 'r') goto yy289;
		goto yy87;
yy208:
		yych = *++cursor_;
		if (yych == '2') goto yy290;
		goto yy87;
yy209:
		yych = *++cursor_;
		if (yych == 'x') goto yy291;
		goto yy87;
yy210:
		yych = *++cursor_;
		if (yych <= 0x7F) goto yy131;
		if (yych <= 0xBF) goto yy79;
		goto yy131;
yy211:
		yych = *++cursor_;
		if (yych <= 0x7F) goto yy131;
		if (yych <= 0xBF) goto yy210;
		goto yy131;
yy212:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '@') {
			if (yych <= '/') goto yy131;
			if (yych <= '9') goto yy129;
			goto yy131;
		} else {
			if (yych <= 'F') goto yy129;
			if (yych <= '`') goto yy131;
			if (yych <= 'f') goto yy129;
			goto yy131;
		}
yy213:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ')') {
			if (yych <= '!') {
				if (yych <= ' ') goto yy88;
				goto yy86;
			} else {
				if (yych <= '"') goto yy88;
				if (yych <= '\'') goto yy86;
				goto yy88;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy148;
				goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy214:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy88;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy88;
				if (yych <= '/') goto yy86;
				goto yy292;
			}
		} else {
			if (yych <= 'F') {
				if (yych == ';') goto yy88;
				if (yych <= '@') goto yy86;
				goto yy292;
			} else {
				if (yych <= '`') goto yy86;
				if (yych <= 'f') goto yy292;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy215:
		yych = *++cursor_;
		if (yych == 'f') goto yy257;
		goto yy87;
yy216:
		++cursor_;
		if ((limit_ - cursor_) < 2) FILL(2);
		yych = *cursor_;
		if (yych <= ':') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy153;
				goto yy86;
			} else {
				if (yych <= ')') goto yy153;
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy216;
				goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') goto yy153;
				if (yych == 'E') goto yy154;
				goto yy86;
			} else {
				if (yych <= 'd') {
					if (yych <= '_') goto yy294;
					goto yy86;
				} else {
					if (yych <= 'e') goto yy154;
					if (yych <= '~') goto yy86;
					goto yy153;
				}
			}
		}
yy218:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ')') {
			if (yych <= '!') {
				if (yych <= ' ') goto yy88;
				goto yy86;
			} else {
				if (yych <= '"') goto yy88;
				if (yych <= '\'') goto yy86;
				goto yy88;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '/') goto yy86;
				if (yych >= ':') goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy219:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '/') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy153;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy153;
				goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '9') goto yy219;
				if (yych <= ':') goto yy86;
				goto yy153;
			} else {
				if (yych == '_') goto yy218;
				if (yych <= '~') goto yy86;
				goto yy153;
			}
		}
yy221:
		++cursor_;
		if ((limit_ - cursor_) < 3) FILL(3);
		yych = *cursor_;
		if (yybm[0+yych] & 128) {
			goto yy221;
		}
		if (yych <= ':') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy98;
				goto yy86;
			} else {
				if (yych <= ')') goto yy98;
				if (yych == '.') goto yy295;
				goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') goto yy98;
				if (yych == 'P') goto yy297;
				goto yy86;
			} else {
				if (yych <= 'o') {
					if (yych <= '_') goto yy156;
					goto yy86;
				} else {
					if (yych <= 'p') goto yy297;
					if (yych <= '~') goto yy86;
					goto yy98;
				}
			}
		}
yy223:
		yych = *++cursor_;
		if (yych == 'g') goto yy298;
		goto yy87;
yy224:
		yych = *++cursor_;
		if (yych == 'f') goto yy299;
		if (yych == 'r') goto yy300;
		goto yy87;
yy225:
		yych = *++cursor_;
		if (yych == 'e') goto yy301;
		goto yy87;
yy226:
		yych = *++cursor_;
		if (yych == 'm') goto yy302;
		goto yy87;
yy227:
		yych = *++cursor_;
		if (yych == 'a') goto yy303;
		goto yy87;
yy228:
		yych = *++cursor_;
		if (yych == 'c') goto yy304;
		goto yy87;
yy229:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'i') goto yy305;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy306;
			if (yych == 't') goto yy307;
			goto yy87;
		}
yy230:
		yych = *++cursor_;
		if (yych == 'l') goto yy308;
		goto yy87;
yy231:
		yych = *++cursor_;
		if (yych == 'c') goto yy310;
		goto yy87;
yy232:
		yych = *++cursor_;
		if (yych == 'r') goto yy311;
		goto yy87;
yy233:
		yych = *++cursor_;
		if (yych == 'a') goto yy312;
		goto yy87;
yy234:
		yych = *++cursor_;
		if (yych == 'p') goto yy314;
		goto yy87;
yy235:
		yych = *++cursor_;
		if (yych == 'm') goto yy316;
		goto yy87;
yy236:
		yych = *++cursor_;
		if (yych == 'e') goto yy318;
		goto yy87;
yy237:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 265 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(End); }
#line 1990 "src/prebuilt/wast-lexer-gen.cc"
yy239:
		yych = *++cursor_;
		if (yych == 'n') goto yy320;
		goto yy87;
yy240:
		yych = *++cursor_;
		if (yych == 'o') goto yy321;
		goto yy87;
yy241:
		++cursor_;
		if ((yych = *cursor_) <= '-') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '.') goto yy322;
				if (yych <= ':') goto yy86;
			} else {
				if (yych == 'x') goto yy323;
				if (yych <= '~') goto yy86;
			}
		}
#line 247 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, F32); }
#line 2019 "src/prebuilt/wast-lexer-gen.cc"
yy243:
		++cursor_;
		if ((yych = *cursor_) <= '-') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '.') goto yy324;
				if (yych <= ':') goto yy86;
			} else {
				if (yych == 'x') goto yy325;
				if (yych <= '~') goto yy86;
			}
		}
#line 248 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, F64); }
#line 2040 "src/prebuilt/wast-lexer-gen.cc"
yy245:
		yych = *++cursor_;
		if (yych == 'c') goto yy326;
		goto yy87;
yy246:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy247;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy328;
				if (yych <= '~') goto yy86;
			}
		}
yy247:
#line 728 "src/wast-lexer.cc"
		{ RETURN(Get); }
#line 2065 "src/prebuilt/wast-lexer-gen.cc"
yy248:
		yych = *++cursor_;
		if (yych == 'b') goto yy329;
		goto yy87;
yy249:
		yych = *++cursor_;
		if (yych == 'w') goto yy330;
		goto yy87;
yy250:
		yych = *++cursor_;
		if (yych == 'x') goto yy331;
		goto yy87;
yy251:
		++cursor_;
		if ((yych = *cursor_) <= '-') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '.') goto yy332;
				if (yych <= ':') goto yy86;
			} else {
				if (yych == 'x') goto yy333;
				if (yych <= '~') goto yy86;
			}
		}
#line 245 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, I32); }
#line 2098 "src/prebuilt/wast-lexer-gen.cc"
yy253:
		++cursor_;
		if ((yych = *cursor_) <= '-') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '.') goto yy334;
				if (yych <= ':') goto yy86;
			} else {
				if (yych == 'x') goto yy335;
				if (yych <= '~') goto yy86;
			}
		}
#line 246 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, I64); }
#line 2119 "src/prebuilt/wast-lexer-gen.cc"
yy255:
		yych = *++cursor_;
		if (yych == '1') goto yy336;
		goto yy87;
yy256:
		yych = *++cursor_;
		if (yych == 'o') goto yy337;
		goto yy87;
yy257:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 230 "src/wast-lexer.cc"
		{ RETURN_LITERAL(Float, Infinity); }
#line 2135 "src/prebuilt/wast-lexer-gen.cc"
yy259:
		yych = *++cursor_;
		if (yych == 'o') goto yy338;
		goto yy87;
yy260:
		yych = *++cursor_;
		if (yych == 'a') goto yy339;
		goto yy87;
yy261:
		yych = *++cursor_;
		if (yych == 'p') goto yy340;
		goto yy87;
yy262:
		yych = *++cursor_;
		if (yych == 'o') goto yy342;
		goto yy87;
yy263:
		yych = *++cursor_;
		if (yych == 'u') goto yy343;
		goto yy87;
yy264:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 252 "src/wast-lexer.cc"
		{ RETURN(Mut); }
#line 2163 "src/prebuilt/wast-lexer-gen.cc"
yy266:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy267;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '9') goto yy86;
				goto yy344;
			} else {
				if (yych <= ';') goto yy267;
				if (yych <= '~') goto yy86;
			}
		}
yy267:
#line 231 "src/wast-lexer.cc"
		{ RETURN_LITERAL(Float, Nan); }
#line 2185 "src/prebuilt/wast-lexer-gen.cc"
yy268:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 253 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Nop); }
#line 2193 "src/prebuilt/wast-lexer-gen.cc"
yy270:
		yych = *++cursor_;
		if (yych == 's') goto yy345;
		goto yy87;
yy271:
		yych = *++cursor_;
		if (yych == 'a') goto yy346;
		goto yy87;
yy272:
		yych = *++cursor_;
		if (yych == 's') goto yy347;
		goto yy87;
yy273:
		yych = *++cursor_;
		if (yych == 't') goto yy348;
		goto yy87;
yy274:
		yych = *++cursor_;
		if (yych == '.') goto yy349;
		goto yy87;
yy275:
		yych = *++cursor_;
		if (yych == 'i') goto yy350;
		goto yy87;
yy276:
		yych = *++cursor_;
		if (yych == 'u') goto yy351;
		goto yy87;
yy277:
		yych = *++cursor_;
		if (yych == 'h') goto yy352;
		if (yych == 'u') goto yy353;
		goto yy87;
yy278:
		yych = *++cursor_;
		if (yych == 'e') goto yy354;
		goto yy87;
yy279:
		yych = *++cursor_;
		if (yych == '_') goto yy355;
		goto yy87;
yy280:
		yych = *++cursor_;
		if (yych == 'r') goto yy356;
		goto yy87;
yy281:
		yych = *++cursor_;
		if (yych == 'r') goto yy357;
		goto yy87;
yy282:
		yych = *++cursor_;
		if (yych == 'l') goto yy358;
		goto yy87;
yy283:
		yych = *++cursor_;
		if (yych == '_') goto yy359;
		goto yy87;
yy284:
		yych = *++cursor_;
		if (yych == 'n') goto yy360;
		goto yy87;
yy285:
		yych = *++cursor_;
		if (yych == 'o') goto yy362;
		goto yy87;
yy286:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 743 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Try); }
#line 2266 "src/prebuilt/wast-lexer-gen.cc"
yy288:
		yych = *++cursor_;
		if (yych == 'e') goto yy363;
		goto yy87;
yy289:
		yych = *++cursor_;
		if (yych == 'e') goto yy365;
		goto yy87;
yy290:
		yych = *++cursor_;
		if (yych == '8') goto yy366;
		goto yy87;
yy291:
		yych = *++cursor_;
		if (yych == '1') goto yy368;
		goto yy87;
yy292:
		++cursor_;
		if ((limit_ - cursor_) < 3) FILL(3);
		yych = *cursor_;
		if (yych <= ';') {
			if (yych <= ')') {
				if (yych <= '!') {
					if (yych <= ' ') goto yy147;
					goto yy86;
				} else {
					if (yych <= '"') goto yy147;
					if (yych <= '\'') goto yy86;
					goto yy147;
				}
			} else {
				if (yych <= '/') {
					if (yych == '.') goto yy295;
					goto yy86;
				} else {
					if (yych <= '9') goto yy292;
					if (yych <= ':') goto yy86;
					goto yy147;
				}
			}
		} else {
			if (yych <= '_') {
				if (yych <= 'O') {
					if (yych <= '@') goto yy86;
					if (yych <= 'F') goto yy292;
					goto yy86;
				} else {
					if (yych <= 'P') goto yy297;
					if (yych <= '^') goto yy86;
					goto yy214;
				}
			} else {
				if (yych <= 'o') {
					if (yych <= '`') goto yy86;
					if (yych <= 'f') goto yy292;
					goto yy86;
				} else {
					if (yych <= 'p') goto yy297;
					if (yych <= '~') goto yy86;
					goto yy147;
				}
			}
		}
yy294:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ')') {
			if (yych <= '!') {
				if (yych <= ' ') goto yy88;
				goto yy86;
			} else {
				if (yych <= '"') goto yy88;
				if (yych <= '\'') goto yy86;
				goto yy88;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy216;
				goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy295:
		++cursor_;
		if ((yych = *cursor_) <= ';') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych >= '#') goto yy86;
			} else {
				if (yych <= '/') {
					if (yych >= '*') goto yy86;
				} else {
					if (yych <= '9') goto yy369;
					if (yych <= ':') goto yy86;
				}
			}
		} else {
			if (yych <= '`') {
				if (yych <= 'F') {
					if (yych <= '@') goto yy86;
					goto yy369;
				} else {
					if (yych == 'P') goto yy297;
					goto yy86;
				}
			} else {
				if (yych <= 'o') {
					if (yych <= 'f') goto yy369;
					goto yy86;
				} else {
					if (yych <= 'p') goto yy297;
					if (yych <= '~') goto yy86;
				}
			}
		}
yy296:
#line 229 "src/wast-lexer.cc"
		{ RETURN_LITERAL(Float, Hexfloat); }
#line 2390 "src/prebuilt/wast-lexer-gen.cc"
yy297:
		yych = *++cursor_;
		if (yych <= ',') {
			if (yych == '+') goto yy371;
			goto yy87;
		} else {
			if (yych <= '-') goto yy371;
			if (yych <= '/') goto yy87;
			if (yych <= '9') goto yy372;
			goto yy87;
		}
yy298:
		yych = *++cursor_;
		if (yych == 'n') goto yy374;
		goto yy87;
yy299:
		yych = *++cursor_;
		if (yych == 'u') goto yy375;
		goto yy87;
yy300:
		yych = *++cursor_;
		if (yych == 'e') goto yy376;
		goto yy87;
yy301:
		yych = *++cursor_;
		if (yych == 'r') goto yy377;
		goto yy87;
yy302:
		yych = *++cursor_;
		if (yych == 'i') goto yy378;
		goto yy87;
yy303:
		yych = *++cursor_;
		if (yych == 'r') goto yy379;
		goto yy87;
yy304:
		yych = *++cursor_;
		if (yych == 'k') goto yy380;
		goto yy87;
yy305:
		yych = *++cursor_;
		if (yych == 'f') goto yy382;
		goto yy87;
yy306:
		yych = *++cursor_;
		if (yych == 'n') goto yy384;
		goto yy87;
yy307:
		yych = *++cursor_;
		if (yych == 'a') goto yy385;
		goto yy87;
yy308:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy309;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy386;
				if (yych <= '~') goto yy86;
			}
		}
yy309:
#line 262 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Call); }
#line 2462 "src/prebuilt/wast-lexer-gen.cc"
yy310:
		yych = *++cursor_;
		if (yych == 'h') goto yy387;
		goto yy87;
yy311:
		yych = *++cursor_;
		if (yych == 'e') goto yy389;
		goto yy87;
yy312:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy313;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy390;
				goto yy86;
			} else {
				if (yych <= ';') goto yy313;
				if (yych <= '~') goto yy86;
			}
		}
yy313:
#line 720 "src/wast-lexer.cc"
		{ RETURN(Data); }
#line 2492 "src/prebuilt/wast-lexer-gen.cc"
yy314:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 264 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Drop); }
#line 2500 "src/prebuilt/wast-lexer-gen.cc"
yy316:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy317;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy391;
				goto yy86;
			} else {
				if (yych <= ';') goto yy317;
				if (yych <= '~') goto yy86;
			}
		}
yy317:
#line 719 "src/wast-lexer.cc"
		{ RETURN(Elem); }
#line 2522 "src/prebuilt/wast-lexer-gen.cc"
yy318:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 257 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Else); }
#line 2530 "src/prebuilt/wast-lexer-gen.cc"
yy320:
		yych = *++cursor_;
		if (yych == 't') goto yy392;
		goto yy87;
yy321:
		yych = *++cursor_;
		if (yych == 'r') goto yy394;
		goto yy87;
yy322:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy395;
		case 'c':	goto yy396;
		case 'd':	goto yy397;
		case 'e':	goto yy398;
		case 'f':	goto yy399;
		case 'g':	goto yy400;
		case 'l':	goto yy401;
		case 'm':	goto yy402;
		case 'n':	goto yy403;
		case 'r':	goto yy404;
		case 's':	goto yy405;
		case 't':	goto yy406;
		default:	goto yy87;
		}
yy323:
		yych = *++cursor_;
		if (yych == '4') goto yy407;
		goto yy87;
yy324:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy409;
		case 'c':	goto yy410;
		case 'd':	goto yy411;
		case 'e':	goto yy412;
		case 'f':	goto yy413;
		case 'g':	goto yy414;
		case 'l':	goto yy415;
		case 'm':	goto yy416;
		case 'n':	goto yy417;
		case 'p':	goto yy418;
		case 'r':	goto yy419;
		case 's':	goto yy420;
		case 't':	goto yy421;
		default:	goto yy87;
		}
yy325:
		yych = *++cursor_;
		if (yych == '2') goto yy422;
		goto yy87;
yy326:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy327;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'q') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'r') goto yy424;
				if (yych <= '~') goto yy86;
			}
		}
yy327:
#line 708 "src/wast-lexer.cc"
		{ RETURN(Func); }
#line 2602 "src/prebuilt/wast-lexer-gen.cc"
yy328:
		yych = *++cursor_;
		if (yych == 'g') goto yy425;
		if (yych == 'l') goto yy426;
		goto yy87;
yy329:
		yych = *++cursor_;
		if (yych == 'a') goto yy427;
		goto yy87;
yy330:
		yych = *++cursor_;
		if (yych == '_') goto yy428;
		goto yy87;
yy331:
		yych = *++cursor_;
		if (yych == '8') goto yy429;
		goto yy87;
yy332:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy431;
		case 'c':	goto yy432;
		case 'd':	goto yy433;
		case 'e':	goto yy434;
		case 'g':	goto yy435;
		case 'l':	goto yy436;
		case 'm':	goto yy437;
		case 'n':	goto yy438;
		case 'o':	goto yy439;
		case 'p':	goto yy440;
		case 'r':	goto yy441;
		case 's':	goto yy442;
		case 't':	goto yy443;
		case 'w':	goto yy444;
		case 'x':	goto yy445;
		default:	goto yy87;
		}
yy333:
		yych = *++cursor_;
		if (yych == '4') goto yy446;
		goto yy87;
yy334:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy448;
		case 'c':	goto yy449;
		case 'd':	goto yy450;
		case 'e':	goto yy451;
		case 'g':	goto yy452;
		case 'l':	goto yy453;
		case 'm':	goto yy454;
		case 'n':	goto yy455;
		case 'o':	goto yy456;
		case 'p':	goto yy457;
		case 'r':	goto yy458;
		case 's':	goto yy459;
		case 't':	goto yy460;
		case 'x':	goto yy461;
		default:	goto yy87;
		}
yy335:
		yych = *++cursor_;
		if (yych == '2') goto yy462;
		goto yy87;
yy336:
		yych = *++cursor_;
		if (yych == '6') goto yy464;
		goto yy87;
yy337:
		yych = *++cursor_;
		if (yych == 'r') goto yy466;
		goto yy87;
yy338:
		yych = *++cursor_;
		if (yych == 'k') goto yy467;
		goto yy87;
yy339:
		yych = *++cursor_;
		if (yych == 'l') goto yy468;
		goto yy87;
yy340:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 258 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Loop); }
#line 2690 "src/prebuilt/wast-lexer-gen.cc"
yy342:
		yych = *++cursor_;
		if (yych == 'r') goto yy470;
		goto yy87;
yy343:
		yych = *++cursor_;
		if (yych == 'l') goto yy471;
		goto yy87;
yy344:
		yych = *++cursor_;
		if (yych == '0') goto yy472;
		goto yy87;
yy345:
		yych = *++cursor_;
		if (yych == 'e') goto yy473;
		goto yy87;
yy346:
		yych = *++cursor_;
		if (yych == 'm') goto yy474;
		goto yy87;
yy347:
		yych = *++cursor_;
		if (yych == 'i') goto yy476;
		goto yy87;
yy348:
		yych = *++cursor_;
		if (yych == 'e') goto yy477;
		goto yy87;
yy349:
		yych = *++cursor_;
		if (yych == 'i') goto yy479;
		if (yych == 'n') goto yy480;
		goto yy87;
yy350:
		yych = *++cursor_;
		if (yych == 's') goto yy481;
		goto yy87;
yy351:
		yych = *++cursor_;
		if (yych == 'l') goto yy482;
		goto yy87;
yy352:
		yych = *++cursor_;
		if (yych == 'r') goto yy483;
		goto yy87;
yy353:
		yych = *++cursor_;
		if (yych == 'r') goto yy484;
		goto yy87;
yy354:
		yych = *++cursor_;
		if (yych == 'c') goto yy485;
		goto yy87;
yy355:
		yych = *++cursor_;
		if (yych == 'g') goto yy486;
		if (yych == 'l') goto yy487;
		goto yy87;
yy356:
		yych = *++cursor_;
		if (yych == 'e') goto yy488;
		goto yy87;
yy357:
		yych = *++cursor_;
		if (yych == 't') goto yy489;
		goto yy87;
yy358:
		yych = *++cursor_;
		if (yych == 'e') goto yy491;
		goto yy87;
yy359:
		yych = *++cursor_;
		if (yych == 'l') goto yy493;
		goto yy87;
yy360:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 256 "src/wast-lexer.cc"
		{ RETURN(Then); }
#line 2772 "src/prebuilt/wast-lexer-gen.cc"
yy362:
		yych = *++cursor_;
		if (yych == 'w') goto yy494;
		goto yy87;
yy363:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 707 "src/wast-lexer.cc"
		{ RETURN(Type); }
#line 2784 "src/prebuilt/wast-lexer-gen.cc"
yy365:
		yych = *++cursor_;
		if (yych == 'a') goto yy496;
		goto yy87;
yy366:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy367;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy497;
				goto yy86;
			} else {
				if (yych <= ';') goto yy367;
				if (yych <= '~') goto yy86;
			}
		}
yy367:
#line 249 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, V128); }
#line 2810 "src/prebuilt/wast-lexer-gen.cc"
yy368:
		yych = *++cursor_;
		if (yych == '6') goto yy498;
		goto yy87;
yy369:
		++cursor_;
		if ((limit_ - cursor_) < 2) FILL(2);
		yych = *cursor_;
		if (yych <= '@') {
			if (yych <= ')') {
				if (yych <= '!') {
					if (yych <= ' ') goto yy296;
					goto yy86;
				} else {
					if (yych <= '"') goto yy296;
					if (yych <= '\'') goto yy86;
					goto yy296;
				}
			} else {
				if (yych <= '9') {
					if (yych <= '/') goto yy86;
					goto yy369;
				} else {
					if (yych == ';') goto yy296;
					goto yy86;
				}
			}
		} else {
			if (yych <= '_') {
				if (yych <= 'O') {
					if (yych <= 'F') goto yy369;
					goto yy86;
				} else {
					if (yych <= 'P') goto yy297;
					if (yych <= '^') goto yy86;
					goto yy499;
				}
			} else {
				if (yych <= 'o') {
					if (yych <= '`') goto yy86;
					if (yych <= 'f') goto yy369;
					goto yy86;
				} else {
					if (yych <= 'p') goto yy297;
					if (yych <= '~') goto yy86;
					goto yy296;
				}
			}
		}
yy371:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ')') {
			if (yych <= '!') {
				if (yych <= ' ') goto yy88;
				goto yy86;
			} else {
				if (yych <= '"') goto yy88;
				if (yych <= '\'') goto yy86;
				goto yy88;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '/') goto yy86;
				if (yych >= ':') goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy372:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '/') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy296;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy296;
				goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '9') goto yy372;
				if (yych <= ':') goto yy86;
				goto yy296;
			} else {
				if (yych == '_') goto yy371;
				if (yych <= '~') goto yy86;
				goto yy296;
			}
		}
yy374:
		yych = *++cursor_;
		if (yych == '=') goto yy500;
		goto yy87;
yy375:
		yych = *++cursor_;
		if (yych == 'n') goto yy501;
		goto yy87;
yy376:
		yych = *++cursor_;
		if (yych == 'f') goto yy502;
		goto yy87;
yy377:
		yych = *++cursor_;
		if (yych == 't') goto yy504;
		goto yy87;
yy378:
		yych = *++cursor_;
		if (yych == 'c') goto yy505;
		goto yy87;
yy379:
		yych = *++cursor_;
		if (yych == 'y') goto yy506;
		goto yy87;
yy380:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 254 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Block); }
#line 2938 "src/prebuilt/wast-lexer-gen.cc"
yy382:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 260 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(BrIf); }
#line 2946 "src/prebuilt/wast-lexer-gen.cc"
yy384:
		yych = *++cursor_;
		if (yych == '_') goto yy508;
		goto yy87;
yy385:
		yych = *++cursor_;
		if (yych == 'b') goto yy509;
		goto yy87;
yy386:
		yych = *++cursor_;
		if (yych == 'i') goto yy510;
		goto yy87;
yy387:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 744 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Catch); }
#line 2966 "src/prebuilt/wast-lexer-gen.cc"
yy389:
		yych = *++cursor_;
		if (yych == 'n') goto yy511;
		goto yy87;
yy390:
		yych = *++cursor_;
		if (yych == 'd') goto yy512;
		goto yy87;
yy391:
		yych = *++cursor_;
		if (yych == 'd') goto yy513;
		goto yy87;
yy392:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 724 "src/wast-lexer.cc"
		{ RETURN(Event); }
#line 2986 "src/prebuilt/wast-lexer-gen.cc"
yy394:
		yych = *++cursor_;
		if (yych == 't') goto yy514;
		goto yy87;
yy395:
		yych = *++cursor_;
		if (yych == 'b') goto yy516;
		if (yych == 'd') goto yy517;
		goto yy87;
yy396:
		yych = *++cursor_;
		if (yych == 'e') goto yy518;
		if (yych == 'o') goto yy519;
		goto yy87;
yy397:
		yych = *++cursor_;
		if (yych == 'e') goto yy520;
		if (yych == 'i') goto yy521;
		goto yy87;
yy398:
		yych = *++cursor_;
		if (yych == 'q') goto yy522;
		goto yy87;
yy399:
		yych = *++cursor_;
		if (yych == 'l') goto yy524;
		goto yy87;
yy400:
		yych = *++cursor_;
		if (yych == 'e') goto yy525;
		if (yych == 't') goto yy527;
		goto yy87;
yy401:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'e') goto yy529;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy531;
			if (yych == 't') goto yy532;
			goto yy87;
		}
yy402:
		yych = *++cursor_;
		if (yych <= 'h') {
			if (yych == 'a') goto yy534;
			goto yy87;
		} else {
			if (yych <= 'i') goto yy535;
			if (yych == 'u') goto yy536;
			goto yy87;
		}
yy403:
		yych = *++cursor_;
		if (yych == 'e') goto yy537;
		goto yy87;
yy404:
		yych = *++cursor_;
		if (yych == 'e') goto yy539;
		goto yy87;
yy405:
		yych = *++cursor_;
		if (yych <= 's') {
			if (yych == 'q') goto yy540;
			goto yy87;
		} else {
			if (yych <= 't') goto yy541;
			if (yych <= 'u') goto yy542;
			goto yy87;
		}
yy406:
		yych = *++cursor_;
		if (yych == 'r') goto yy543;
		goto yy87;
yy407:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy408;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy544;
				goto yy86;
			} else {
				if (yych <= ';') goto yy408;
				if (yych <= '~') goto yy86;
			}
		}
yy408:
#line 741 "src/wast-lexer.cc"
		{ RETURN(F32X4); }
#line 3082 "src/prebuilt/wast-lexer-gen.cc"
yy409:
		yych = *++cursor_;
		if (yych == 'b') goto yy545;
		if (yych == 'd') goto yy546;
		goto yy87;
yy410:
		yych = *++cursor_;
		if (yych == 'e') goto yy547;
		if (yych == 'o') goto yy548;
		goto yy87;
yy411:
		yych = *++cursor_;
		if (yych == 'i') goto yy549;
		goto yy87;
yy412:
		yych = *++cursor_;
		if (yych == 'q') goto yy550;
		goto yy87;
yy413:
		yych = *++cursor_;
		if (yych == 'l') goto yy552;
		goto yy87;
yy414:
		yych = *++cursor_;
		if (yych == 'e') goto yy553;
		if (yych == 't') goto yy555;
		goto yy87;
yy415:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'e') goto yy557;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy559;
			if (yych == 't') goto yy560;
			goto yy87;
		}
yy416:
		yych = *++cursor_;
		if (yych <= 'h') {
			if (yych == 'a') goto yy562;
			goto yy87;
		} else {
			if (yych <= 'i') goto yy563;
			if (yych == 'u') goto yy564;
			goto yy87;
		}
yy417:
		yych = *++cursor_;
		if (yych == 'e') goto yy565;
		goto yy87;
yy418:
		yych = *++cursor_;
		if (yych == 'r') goto yy567;
		goto yy87;
yy419:
		yych = *++cursor_;
		if (yych == 'e') goto yy568;
		goto yy87;
yy420:
		yych = *++cursor_;
		if (yych <= 's') {
			if (yych == 'q') goto yy569;
			goto yy87;
		} else {
			if (yych <= 't') goto yy570;
			if (yych <= 'u') goto yy571;
			goto yy87;
		}
yy421:
		yych = *++cursor_;
		if (yych == 'r') goto yy572;
		goto yy87;
yy422:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy423;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy573;
				goto yy86;
			} else {
				if (yych <= ';') goto yy423;
				if (yych <= '~') goto yy86;
			}
		}
yy423:
#line 742 "src/wast-lexer.cc"
		{ RETURN(F64X2); }
#line 3177 "src/prebuilt/wast-lexer-gen.cc"
yy424:
		yych = *++cursor_;
		if (yych == 'e') goto yy574;
		goto yy87;
yy425:
		yych = *++cursor_;
		if (yych == 'l') goto yy575;
		goto yy87;
yy426:
		yych = *++cursor_;
		if (yych == 'o') goto yy576;
		goto yy87;
yy427:
		yych = *++cursor_;
		if (yych == 'l') goto yy577;
		goto yy87;
yy428:
		yych = *++cursor_;
		if (yych == 'm') goto yy579;
		goto yy87;
yy429:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy430;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy580;
				goto yy86;
			} else {
				if (yych <= ';') goto yy430;
				if (yych <= '~') goto yy86;
			}
		}
yy430:
#line 738 "src/wast-lexer.cc"
		{ RETURN(I16X8); }
#line 3219 "src/prebuilt/wast-lexer-gen.cc"
yy431:
		yych = *++cursor_;
		if (yych <= 'm') {
			if (yych == 'd') goto yy581;
			goto yy87;
		} else {
			if (yych <= 'n') goto yy582;
			if (yych == 't') goto yy583;
			goto yy87;
		}
yy432:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'l') goto yy584;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy585;
			if (yych == 't') goto yy586;
			goto yy87;
		}
yy433:
		yych = *++cursor_;
		if (yych == 'i') goto yy587;
		goto yy87;
yy434:
		yych = *++cursor_;
		if (yych == 'q') goto yy588;
		if (yych == 'x') goto yy590;
		goto yy87;
yy435:
		yych = *++cursor_;
		if (yych == 'e') goto yy591;
		if (yych == 't') goto yy592;
		goto yy87;
yy436:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'e') goto yy593;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy594;
			if (yych == 't') goto yy595;
			goto yy87;
		}
yy437:
		yych = *++cursor_;
		if (yych == 'u') goto yy596;
		goto yy87;
yy438:
		yych = *++cursor_;
		if (yych == 'e') goto yy597;
		goto yy87;
yy439:
		yych = *++cursor_;
		if (yych == 'r') goto yy599;
		goto yy87;
yy440:
		yych = *++cursor_;
		if (yych == 'o') goto yy601;
		goto yy87;
yy441:
		yych = *++cursor_;
		if (yych == 'e') goto yy602;
		if (yych == 'o') goto yy603;
		goto yy87;
yy442:
		yych = *++cursor_;
		if (yych <= 's') {
			if (yych == 'h') goto yy604;
			goto yy87;
		} else {
			if (yych <= 't') goto yy605;
			if (yych <= 'u') goto yy606;
			goto yy87;
		}
yy443:
		yych = *++cursor_;
		if (yych == 'r') goto yy607;
		goto yy87;
yy444:
		yych = *++cursor_;
		if (yych == 'r') goto yy608;
		goto yy87;
yy445:
		yych = *++cursor_;
		if (yych == 'o') goto yy609;
		goto yy87;
yy446:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy447;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy610;
				goto yy86;
			} else {
				if (yych <= ';') goto yy447;
				if (yych <= '~') goto yy86;
			}
		}
yy447:
#line 739 "src/wast-lexer.cc"
		{ RETURN(I32X4); }
#line 3328 "src/prebuilt/wast-lexer-gen.cc"
yy448:
		yych = *++cursor_;
		if (yych <= 'm') {
			if (yych == 'd') goto yy611;
			goto yy87;
		} else {
			if (yych <= 'n') goto yy612;
			if (yych == 't') goto yy613;
			goto yy87;
		}
yy449:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'l') goto yy614;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy615;
			if (yych == 't') goto yy616;
			goto yy87;
		}
yy450:
		yych = *++cursor_;
		if (yych == 'i') goto yy617;
		goto yy87;
yy451:
		yych = *++cursor_;
		if (yych == 'q') goto yy618;
		if (yych == 'x') goto yy620;
		goto yy87;
yy452:
		yych = *++cursor_;
		if (yych == 'e') goto yy621;
		if (yych == 't') goto yy622;
		goto yy87;
yy453:
		yych = *++cursor_;
		if (yych <= 'n') {
			if (yych == 'e') goto yy623;
			goto yy87;
		} else {
			if (yych <= 'o') goto yy624;
			if (yych == 't') goto yy625;
			goto yy87;
		}
yy454:
		yych = *++cursor_;
		if (yych == 'u') goto yy626;
		goto yy87;
yy455:
		yych = *++cursor_;
		if (yych == 'e') goto yy627;
		goto yy87;
yy456:
		yych = *++cursor_;
		if (yych == 'r') goto yy629;
		goto yy87;
yy457:
		yych = *++cursor_;
		if (yych == 'o') goto yy631;
		goto yy87;
yy458:
		yych = *++cursor_;
		if (yych == 'e') goto yy632;
		if (yych == 'o') goto yy633;
		goto yy87;
yy459:
		yych = *++cursor_;
		if (yych <= 's') {
			if (yych == 'h') goto yy634;
			goto yy87;
		} else {
			if (yych <= 't') goto yy635;
			if (yych <= 'u') goto yy636;
			goto yy87;
		}
yy460:
		yych = *++cursor_;
		if (yych == 'r') goto yy637;
		goto yy87;
yy461:
		yych = *++cursor_;
		if (yych == 'o') goto yy638;
		goto yy87;
yy462:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy463;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy639;
				goto yy86;
			} else {
				if (yych <= ';') goto yy463;
				if (yych <= '~') goto yy86;
			}
		}
yy463:
#line 740 "src/wast-lexer.cc"
		{ RETURN(I64X2); }
#line 3433 "src/prebuilt/wast-lexer-gen.cc"
yy464:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy465;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy640;
				goto yy86;
			} else {
				if (yych <= ';') goto yy465;
				if (yych <= '~') goto yy86;
			}
		}
yy465:
#line 737 "src/wast-lexer.cc"
		{ RETURN(I8X16); }
#line 3455 "src/prebuilt/wast-lexer-gen.cc"
yy466:
		yych = *++cursor_;
		if (yych == 't') goto yy641;
		goto yy87;
yy467:
		yych = *++cursor_;
		if (yych == 'e') goto yy643;
		goto yy87;
yy468:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy469;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy645;
				goto yy86;
			} else {
				if (yych <= ';') goto yy469;
				if (yych <= '~') goto yy86;
			}
		}
yy469:
#line 711 "src/wast-lexer.cc"
		{ RETURN(Local); }
#line 3485 "src/prebuilt/wast-lexer-gen.cc"
yy470:
		yych = *++cursor_;
		if (yych == 'y') goto yy646;
		goto yy87;
yy471:
		yych = *++cursor_;
		if (yych == 'e') goto yy648;
		goto yy87;
yy472:
		yych = *++cursor_;
		if (yych == 'x') goto yy650;
		goto yy87;
yy473:
		yych = *++cursor_;
		if (yych == 't') goto yy651;
		goto yy87;
yy474:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 709 "src/wast-lexer.cc"
		{ RETURN(Param); }
#line 3509 "src/prebuilt/wast-lexer-gen.cc"
yy476:
		yych = *++cursor_;
		if (yych == 'v') goto yy653;
		goto yy87;
yy477:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 715 "src/wast-lexer.cc"
		{ RETURN(Quote); }
#line 3521 "src/prebuilt/wast-lexer-gen.cc"
yy479:
		yych = *++cursor_;
		if (yych == 's') goto yy654;
		goto yy87;
yy480:
		yych = *++cursor_;
		if (yych == 'u') goto yy655;
		goto yy87;
yy481:
		yych = *++cursor_;
		if (yych == 't') goto yy656;
		goto yy87;
yy482:
		yych = *++cursor_;
		if (yych == 't') goto yy657;
		goto yy87;
yy483:
		yych = *++cursor_;
		if (yych == 'o') goto yy659;
		goto yy87;
yy484:
		yych = *++cursor_;
		if (yych == 'n') goto yy660;
		goto yy87;
yy485:
		yych = *++cursor_;
		if (yych == 't') goto yy662;
		goto yy87;
yy486:
		yych = *++cursor_;
		if (yych == 'l') goto yy664;
		goto yy87;
yy487:
		yych = *++cursor_;
		if (yych == 'o') goto yy665;
		goto yy87;
yy488:
		yych = *++cursor_;
		if (yych == 'd') goto yy666;
		goto yy87;
yy489:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 718 "src/wast-lexer.cc"
		{ RETURN(Start); }
#line 3569 "src/prebuilt/wast-lexer-gen.cc"
yy491:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy492;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy668;
				goto yy86;
			} else {
				if (yych <= ';') goto yy492;
				if (yych <= '~') goto yy86;
			}
		}
yy492:
#line 716 "src/wast-lexer.cc"
		{ RETURN(Table); }
#line 3591 "src/prebuilt/wast-lexer-gen.cc"
yy493:
		yych = *++cursor_;
		if (yych == 'o') goto yy669;
		goto yy87;
yy494:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 745 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Throw); }
#line 3603 "src/prebuilt/wast-lexer-gen.cc"
yy496:
		yych = *++cursor_;
		if (yych == 'c') goto yy670;
		goto yy87;
yy497:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy671;
		case 'b':	goto yy672;
		case 'c':	goto yy673;
		case 'l':	goto yy674;
		case 'n':	goto yy675;
		case 'o':	goto yy676;
		case 's':	goto yy677;
		case 'x':	goto yy678;
		default:	goto yy87;
		}
yy498:
		yych = *++cursor_;
		if (yych == '.') goto yy679;
		goto yy87;
yy499:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy88;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy88;
				if (yych <= '/') goto yy86;
				goto yy369;
			}
		} else {
			if (yych <= 'F') {
				if (yych == ';') goto yy88;
				if (yych <= '@') goto yy86;
				goto yy369;
			} else {
				if (yych <= '`') goto yy86;
				if (yych <= 'f') goto yy369;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy500:
		yych = *++cursor_;
		if (yych <= '/') goto yy87;
		if (yych <= '0') goto yy680;
		if (yych <= '9') goto yy682;
		goto yy87;
yy501:
		yych = *++cursor_;
		if (yych == 'c') goto yy684;
		goto yy87;
yy502:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 251 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, Anyref); }
#line 3668 "src/prebuilt/wast-lexer-gen.cc"
yy504:
		yych = *++cursor_;
		if (yych == '_') goto yy686;
		goto yy87;
yy505:
		yych = *++cursor_;
		if (yych == '.') goto yy687;
		goto yy87;
yy506:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 714 "src/wast-lexer.cc"
		{ RETURN(Bin); }
#line 3684 "src/prebuilt/wast-lexer-gen.cc"
yy508:
		yych = *++cursor_;
		if (yych == 'e') goto yy688;
		goto yy87;
yy509:
		yych = *++cursor_;
		if (yych == 'l') goto yy689;
		goto yy87;
yy510:
		yych = *++cursor_;
		if (yych == 'n') goto yy690;
		goto yy87;
yy511:
		yych = *++cursor_;
		if (yych == 't') goto yy691;
		goto yy87;
yy512:
		yych = *++cursor_;
		if (yych == 'r') goto yy692;
		goto yy87;
yy513:
		yych = *++cursor_;
		if (yych == 'r') goto yy693;
		goto yy87;
yy514:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 723 "src/wast-lexer.cc"
		{ RETURN(Export); }
#line 3716 "src/prebuilt/wast-lexer-gen.cc"
yy516:
		yych = *++cursor_;
		if (yych == 's') goto yy694;
		goto yy87;
yy517:
		yych = *++cursor_;
		if (yych == 'd') goto yy696;
		goto yy87;
yy518:
		yych = *++cursor_;
		if (yych == 'i') goto yy698;
		goto yy87;
yy519:
		yych = *++cursor_;
		if (yych == 'n') goto yy699;
		if (yych == 'p') goto yy700;
		goto yy87;
yy520:
		yych = *++cursor_;
		if (yych == 'm') goto yy701;
		goto yy87;
yy521:
		yych = *++cursor_;
		if (yych == 'v') goto yy702;
		goto yy87;
yy522:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 392 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32Eq); }
#line 3749 "src/prebuilt/wast-lexer-gen.cc"
yy524:
		yych = *++cursor_;
		if (yych == 'o') goto yy704;
		goto yy87;
yy525:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 402 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32Ge); }
#line 3761 "src/prebuilt/wast-lexer-gen.cc"
yy527:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 400 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32Gt); }
#line 3769 "src/prebuilt/wast-lexer-gen.cc"
yy529:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 398 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32Le); }
#line 3777 "src/prebuilt/wast-lexer-gen.cc"
yy531:
		yych = *++cursor_;
		if (yych == 'a') goto yy705;
		goto yy87;
yy532:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 396 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32Lt); }
#line 3789 "src/prebuilt/wast-lexer-gen.cc"
yy534:
		yych = *++cursor_;
		if (yych == 'x') goto yy706;
		goto yy87;
yy535:
		yych = *++cursor_;
		if (yych == 'n') goto yy708;
		goto yy87;
yy536:
		yych = *++cursor_;
		if (yych == 'l') goto yy710;
		goto yy87;
yy537:
		++cursor_;
		if ((yych = *cursor_) <= ':') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= 'a') {
				if (yych <= ';') goto yy538;
				if (yych <= '`') goto yy86;
				goto yy712;
			} else {
				if (yych == 'g') goto yy713;
				if (yych <= '~') goto yy86;
			}
		}
yy538:
#line 394 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32Ne); }
#line 3824 "src/prebuilt/wast-lexer-gen.cc"
yy539:
		yych = *++cursor_;
		if (yych == 'i') goto yy715;
		goto yy87;
yy540:
		yych = *++cursor_;
		if (yych == 'r') goto yy716;
		goto yy87;
yy541:
		yych = *++cursor_;
		if (yych == 'o') goto yy717;
		goto yy87;
yy542:
		yych = *++cursor_;
		if (yych == 'b') goto yy718;
		goto yy87;
yy543:
		yych = *++cursor_;
		if (yych == 'u') goto yy720;
		goto yy87;
yy544:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy721;
		case 'c':	goto yy722;
		case 'd':	goto yy723;
		case 'e':	goto yy724;
		case 'g':	goto yy725;
		case 'l':	goto yy726;
		case 'm':	goto yy727;
		case 'n':	goto yy728;
		case 'r':	goto yy729;
		case 's':	goto yy730;
		default:	goto yy87;
		}
yy545:
		yych = *++cursor_;
		if (yych == 's') goto yy731;
		goto yy87;
yy546:
		yych = *++cursor_;
		if (yych == 'd') goto yy733;
		goto yy87;
yy547:
		yych = *++cursor_;
		if (yych == 'i') goto yy735;
		goto yy87;
yy548:
		yych = *++cursor_;
		if (yych == 'n') goto yy736;
		if (yych == 'p') goto yy737;
		goto yy87;
yy549:
		yych = *++cursor_;
		if (yych == 'v') goto yy738;
		goto yy87;
yy550:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 393 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64Eq); }
#line 3888 "src/prebuilt/wast-lexer-gen.cc"
yy552:
		yych = *++cursor_;
		if (yych == 'o') goto yy740;
		goto yy87;
yy553:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 403 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64Ge); }
#line 3900 "src/prebuilt/wast-lexer-gen.cc"
yy555:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 401 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64Gt); }
#line 3908 "src/prebuilt/wast-lexer-gen.cc"
yy557:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 399 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64Le); }
#line 3916 "src/prebuilt/wast-lexer-gen.cc"
yy559:
		yych = *++cursor_;
		if (yych == 'a') goto yy741;
		goto yy87;
yy560:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 397 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64Lt); }
#line 3928 "src/prebuilt/wast-lexer-gen.cc"
yy562:
		yych = *++cursor_;
		if (yych == 'x') goto yy742;
		goto yy87;
yy563:
		yych = *++cursor_;
		if (yych == 'n') goto yy744;
		goto yy87;
yy564:
		yych = *++cursor_;
		if (yych == 'l') goto yy746;
		goto yy87;
yy565:
		++cursor_;
		if ((yych = *cursor_) <= ':') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= 'a') {
				if (yych <= ';') goto yy566;
				if (yych <= '`') goto yy86;
				goto yy748;
			} else {
				if (yych == 'g') goto yy749;
				if (yych <= '~') goto yy86;
			}
		}
yy566:
#line 395 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64Ne); }
#line 3963 "src/prebuilt/wast-lexer-gen.cc"
yy567:
		yych = *++cursor_;
		if (yych == 'o') goto yy751;
		goto yy87;
yy568:
		yych = *++cursor_;
		if (yych == 'i') goto yy752;
		goto yy87;
yy569:
		yych = *++cursor_;
		if (yych == 'r') goto yy753;
		goto yy87;
yy570:
		yych = *++cursor_;
		if (yych == 'o') goto yy754;
		goto yy87;
yy571:
		yych = *++cursor_;
		if (yych == 'b') goto yy755;
		goto yy87;
yy572:
		yych = *++cursor_;
		if (yych == 'u') goto yy757;
		goto yy87;
yy573:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy758;
		case 'c':	goto yy759;
		case 'd':	goto yy760;
		case 'e':	goto yy761;
		case 'g':	goto yy762;
		case 'l':	goto yy763;
		case 'm':	goto yy764;
		case 'n':	goto yy765;
		case 'r':	goto yy766;
		case 's':	goto yy767;
		default:	goto yy87;
		}
yy574:
		yych = *++cursor_;
		if (yych == 'f') goto yy768;
		goto yy87;
yy575:
		yych = *++cursor_;
		if (yych == 'o') goto yy770;
		goto yy87;
yy576:
		yych = *++cursor_;
		if (yych == 'c') goto yy771;
		goto yy87;
yy577:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy578;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy772;
				goto yy86;
			} else {
				if (yych <= ';') goto yy578;
				if (yych <= '~') goto yy86;
			}
		}
yy578:
#line 712 "src/wast-lexer.cc"
		{ RETURN(Global); }
#line 4036 "src/prebuilt/wast-lexer-gen.cc"
yy579:
		yych = *++cursor_;
		if (yych == 'e') goto yy773;
		goto yy87;
yy580:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy774;
		case 'e':	goto yy775;
		case 'g':	goto yy776;
		case 'l':	goto yy777;
		case 'm':	goto yy778;
		case 'n':	goto yy779;
		case 'r':	goto yy780;
		case 's':	goto yy781;
		default:	goto yy87;
		}
yy581:
		yych = *++cursor_;
		if (yych == 'd') goto yy782;
		goto yy87;
yy582:
		yych = *++cursor_;
		if (yych == 'd') goto yy784;
		goto yy87;
yy583:
		yych = *++cursor_;
		if (yych == 'o') goto yy786;
		goto yy87;
yy584:
		yych = *++cursor_;
		if (yych == 'z') goto yy787;
		goto yy87;
yy585:
		yych = *++cursor_;
		if (yych == 'n') goto yy789;
		goto yy87;
yy586:
		yych = *++cursor_;
		if (yych == 'z') goto yy790;
		goto yy87;
yy587:
		yych = *++cursor_;
		if (yych == 'v') goto yy792;
		goto yy87;
yy588:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy589;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'y') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'z') goto yy793;
				if (yych <= '~') goto yy86;
			}
		}
yy589:
#line 372 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32Eq); }
#line 4102 "src/prebuilt/wast-lexer-gen.cc"
yy590:
		yych = *++cursor_;
		if (yych == 't') goto yy795;
		goto yy87;
yy591:
		yych = *++cursor_;
		if (yych == '_') goto yy796;
		goto yy87;
yy592:
		yych = *++cursor_;
		if (yych == '_') goto yy797;
		goto yy87;
yy593:
		yych = *++cursor_;
		if (yych == '_') goto yy798;
		goto yy87;
yy594:
		yych = *++cursor_;
		if (yych == 'a') goto yy799;
		goto yy87;
yy595:
		yych = *++cursor_;
		if (yych == '_') goto yy800;
		goto yy87;
yy596:
		yych = *++cursor_;
		if (yych == 'l') goto yy801;
		goto yy87;
yy597:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 374 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32Ne); }
#line 4138 "src/prebuilt/wast-lexer-gen.cc"
yy599:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 344 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Or); }
#line 4146 "src/prebuilt/wast-lexer-gen.cc"
yy601:
		yych = *++cursor_;
		if (yych == 'p') goto yy803;
		goto yy87;
yy602:
		yych = *++cursor_;
		if (yych == 'i') goto yy804;
		if (yych == 'm') goto yy805;
		goto yy87;
yy603:
		yych = *++cursor_;
		if (yych == 't') goto yy806;
		goto yy87;
yy604:
		yych = *++cursor_;
		if (yych == 'l') goto yy807;
		if (yych == 'r') goto yy809;
		goto yy87;
yy605:
		yych = *++cursor_;
		if (yych == 'o') goto yy810;
		goto yy87;
yy606:
		yych = *++cursor_;
		if (yych == 'b') goto yy811;
		goto yy87;
yy607:
		yych = *++cursor_;
		if (yych == 'u') goto yy813;
		goto yy87;
yy608:
		yych = *++cursor_;
		if (yych == 'a') goto yy814;
		goto yy87;
yy609:
		yych = *++cursor_;
		if (yych == 'r') goto yy815;
		goto yy87;
yy610:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy817;
		case 'e':	goto yy818;
		case 'g':	goto yy819;
		case 'l':	goto yy820;
		case 'm':	goto yy821;
		case 'n':	goto yy822;
		case 'r':	goto yy823;
		case 's':	goto yy824;
		case 't':	goto yy825;
		default:	goto yy87;
		}
yy611:
		yych = *++cursor_;
		if (yych == 'd') goto yy826;
		goto yy87;
yy612:
		yych = *++cursor_;
		if (yych == 'd') goto yy828;
		goto yy87;
yy613:
		yych = *++cursor_;
		if (yych == 'o') goto yy830;
		goto yy87;
yy614:
		yych = *++cursor_;
		if (yych == 'z') goto yy831;
		goto yy87;
yy615:
		yych = *++cursor_;
		if (yych == 'n') goto yy833;
		goto yy87;
yy616:
		yych = *++cursor_;
		if (yych == 'z') goto yy834;
		goto yy87;
yy617:
		yych = *++cursor_;
		if (yych == 'v') goto yy836;
		goto yy87;
yy618:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy619;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'y') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'z') goto yy837;
				if (yych <= '~') goto yy86;
			}
		}
yy619:
#line 373 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64Eq); }
#line 4247 "src/prebuilt/wast-lexer-gen.cc"
yy620:
		yych = *++cursor_;
		if (yych == 't') goto yy839;
		goto yy87;
yy621:
		yych = *++cursor_;
		if (yych == '_') goto yy840;
		goto yy87;
yy622:
		yych = *++cursor_;
		if (yych == '_') goto yy841;
		goto yy87;
yy623:
		yych = *++cursor_;
		if (yych == '_') goto yy842;
		goto yy87;
yy624:
		yych = *++cursor_;
		if (yych == 'a') goto yy843;
		goto yy87;
yy625:
		yych = *++cursor_;
		if (yych == '_') goto yy844;
		goto yy87;
yy626:
		yych = *++cursor_;
		if (yych == 'l') goto yy845;
		goto yy87;
yy627:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 375 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64Ne); }
#line 4283 "src/prebuilt/wast-lexer-gen.cc"
yy629:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 345 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Or); }
#line 4291 "src/prebuilt/wast-lexer-gen.cc"
yy631:
		yych = *++cursor_;
		if (yych == 'p') goto yy847;
		goto yy87;
yy632:
		yych = *++cursor_;
		if (yych == 'i') goto yy848;
		if (yych == 'm') goto yy849;
		goto yy87;
yy633:
		yych = *++cursor_;
		if (yych == 't') goto yy850;
		goto yy87;
yy634:
		yych = *++cursor_;
		if (yych == 'l') goto yy851;
		if (yych == 'r') goto yy853;
		goto yy87;
yy635:
		yych = *++cursor_;
		if (yych == 'o') goto yy854;
		goto yy87;
yy636:
		yych = *++cursor_;
		if (yych == 'b') goto yy855;
		goto yy87;
yy637:
		yych = *++cursor_;
		if (yych == 'u') goto yy857;
		goto yy87;
yy638:
		yych = *++cursor_;
		if (yych == 'r') goto yy858;
		goto yy87;
yy639:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy860;
		case 'e':	goto yy861;
		case 'n':	goto yy862;
		case 'r':	goto yy863;
		case 's':	goto yy864;
		case 't':	goto yy865;
		default:	goto yy87;
		}
yy640:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy866;
		case 'e':	goto yy867;
		case 'g':	goto yy868;
		case 'l':	goto yy869;
		case 'm':	goto yy870;
		case 'n':	goto yy871;
		case 'r':	goto yy872;
		case 's':	goto yy873;
		default:	goto yy87;
		}
yy641:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 722 "src/wast-lexer.cc"
		{ RETURN(Import); }
#line 4357 "src/prebuilt/wast-lexer-gen.cc"
yy643:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 727 "src/wast-lexer.cc"
		{ RETURN(Invoke); }
#line 4365 "src/prebuilt/wast-lexer-gen.cc"
yy645:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'g') goto yy874;
			goto yy87;
		} else {
			if (yych <= 's') goto yy875;
			if (yych <= 't') goto yy876;
			goto yy87;
		}
yy646:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy647;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= ':') {
				if (yych == '.') goto yy877;
				goto yy86;
			} else {
				if (yych <= ';') goto yy647;
				if (yych <= '~') goto yy86;
			}
		}
yy647:
#line 717 "src/wast-lexer.cc"
		{ RETURN(Memory); }
#line 4397 "src/prebuilt/wast-lexer-gen.cc"
yy648:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 713 "src/wast-lexer.cc"
		{ RETURN(Module); }
#line 4405 "src/prebuilt/wast-lexer-gen.cc"
yy650:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy88;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy88;
				if (yych <= '/') goto yy86;
				goto yy878;
			}
		} else {
			if (yych <= 'F') {
				if (yych == ';') goto yy88;
				if (yych <= '@') goto yy86;
				goto yy878;
			} else {
				if (yych <= '`') goto yy86;
				if (yych <= 'f') goto yy878;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy651:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy652;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '<') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '=') goto yy880;
				if (yych <= '~') goto yy86;
			}
		}
yy652:
#line 721 "src/wast-lexer.cc"
		{ RETURN(Offset); }
#line 4452 "src/prebuilt/wast-lexer-gen.cc"
yy653:
		yych = *++cursor_;
		if (yych == 'e') goto yy881;
		goto yy87;
yy654:
		yych = *++cursor_;
		if (yych == '_') goto yy883;
		goto yy87;
yy655:
		yych = *++cursor_;
		if (yych == 'l') goto yy884;
		goto yy87;
yy656:
		yych = *++cursor_;
		if (yych == 'e') goto yy885;
		goto yy87;
yy657:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 710 "src/wast-lexer.cc"
		{ RETURN(Result); }
#line 4476 "src/prebuilt/wast-lexer-gen.cc"
yy659:
		yych = *++cursor_;
		if (yych == 'w') goto yy886;
		goto yy87;
yy660:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy661;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy888;
				if (yych <= '~') goto yy86;
			}
		}
yy661:
#line 266 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Return); }
#line 4501 "src/prebuilt/wast-lexer-gen.cc"
yy662:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 437 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Select); }
#line 4509 "src/prebuilt/wast-lexer-gen.cc"
yy664:
		yych = *++cursor_;
		if (yych == 'o') goto yy889;
		goto yy87;
yy665:
		yych = *++cursor_;
		if (yych == 'c') goto yy890;
		goto yy87;
yy666:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 749 "src/wast-lexer.cc"
		{ RETURN(Shared); }
#line 4525 "src/prebuilt/wast-lexer-gen.cc"
yy668:
		yych = *++cursor_;
		switch (yych) {
		case 'c':	goto yy891;
		case 'g':	goto yy892;
		case 'i':	goto yy893;
		case 's':	goto yy894;
		default:	goto yy87;
		}
yy669:
		yych = *++cursor_;
		if (yych == 'c') goto yy895;
		goto yy87;
yy670:
		yych = *++cursor_;
		if (yych == 'h') goto yy896;
		goto yy87;
yy671:
		yych = *++cursor_;
		if (yych == 'n') goto yy897;
		goto yy87;
yy672:
		yych = *++cursor_;
		if (yych == 'i') goto yy898;
		goto yy87;
yy673:
		yych = *++cursor_;
		if (yych == 'o') goto yy899;
		goto yy87;
yy674:
		yych = *++cursor_;
		if (yych == 'o') goto yy900;
		goto yy87;
yy675:
		yych = *++cursor_;
		if (yych == 'o') goto yy901;
		goto yy87;
yy676:
		yych = *++cursor_;
		if (yych == 'r') goto yy902;
		goto yy87;
yy677:
		yych = *++cursor_;
		if (yych == 't') goto yy904;
		goto yy87;
yy678:
		yych = *++cursor_;
		if (yych == 'o') goto yy905;
		goto yy87;
yy679:
		yych = *++cursor_;
		if (yych == 's') goto yy906;
		goto yy87;
yy680:
		++cursor_;
		if ((yych = *cursor_) <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy681;
				if (yych <= '/') goto yy86;
				goto yy682;
			}
		} else {
			if (yych <= '_') {
				if (yych == ';') goto yy681;
				if (yych <= '^') goto yy86;
				goto yy907;
			} else {
				if (yych == 'x') goto yy908;
				if (yych <= '~') goto yy86;
			}
		}
yy681:
#line 296 "src/wast-lexer.cc"
		{ RETURN_TEXT_AT(AlignEqNat, 6); }
#line 4603 "src/prebuilt/wast-lexer-gen.cc"
yy682:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '/') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy681;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy681;
				goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '9') goto yy682;
				if (yych <= ':') goto yy86;
				goto yy681;
			} else {
				if (yych == '_') goto yy907;
				if (yych <= '~') goto yy86;
				goto yy681;
			}
		}
yy684:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 667 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, Funcref); }
#line 4635 "src/prebuilt/wast-lexer-gen.cc"
yy686:
		yych = *++cursor_;
		switch (yych) {
		case 'e':	goto yy909;
		case 'i':	goto yy910;
		case 'm':	goto yy911;
		case 'r':	goto yy912;
		case 't':	goto yy913;
		case 'u':	goto yy914;
		default:	goto yy87;
		}
yy687:
		yych = *++cursor_;
		if (yych == 'n') goto yy915;
		goto yy87;
yy688:
		yych = *++cursor_;
		if (yych == 'x') goto yy916;
		goto yy87;
yy689:
		yych = *++cursor_;
		if (yych == 'e') goto yy917;
		goto yy87;
yy690:
		yych = *++cursor_;
		if (yych == 'd') goto yy919;
		goto yy87;
yy691:
		yych = *++cursor_;
		if (yych == '_') goto yy920;
		goto yy87;
yy692:
		yych = *++cursor_;
		if (yych == 'o') goto yy921;
		goto yy87;
yy693:
		yych = *++cursor_;
		if (yych == 'o') goto yy922;
		goto yy87;
yy694:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 311 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Abs); }
#line 4682 "src/prebuilt/wast-lexer-gen.cc"
yy696:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 358 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Add); }
#line 4690 "src/prebuilt/wast-lexer-gen.cc"
yy698:
		yych = *++cursor_;
		if (yych == 'l') goto yy923;
		goto yy87;
yy699:
		yych = *++cursor_;
		if (yych == 's') goto yy925;
		if (yych == 'v') goto yy926;
		goto yy87;
yy700:
		yych = *++cursor_;
		if (yych == 'y') goto yy927;
		goto yy87;
yy701:
		yych = *++cursor_;
		if (yych == 'o') goto yy928;
		goto yy87;
yy702:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 364 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Div); }
#line 4715 "src/prebuilt/wast-lexer-gen.cc"
yy704:
		yych = *++cursor_;
		if (yych == 'o') goto yy929;
		goto yy87;
yy705:
		yych = *++cursor_;
		if (yych == 'd') goto yy930;
		goto yy87;
yy706:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 368 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Max); }
#line 4731 "src/prebuilt/wast-lexer-gen.cc"
yy708:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 366 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Min); }
#line 4739 "src/prebuilt/wast-lexer-gen.cc"
yy710:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 362 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Mul); }
#line 4747 "src/prebuilt/wast-lexer-gen.cc"
yy712:
		yych = *++cursor_;
		if (yych == 'r') goto yy932;
		goto yy87;
yy713:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 309 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Neg); }
#line 4759 "src/prebuilt/wast-lexer-gen.cc"
yy715:
		yych = *++cursor_;
		if (yych == 'n') goto yy933;
		goto yy87;
yy716:
		yych = *++cursor_;
		if (yych == 't') goto yy934;
		goto yy87;
yy717:
		yych = *++cursor_;
		if (yych == 'r') goto yy936;
		goto yy87;
yy718:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 360 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Sub); }
#line 4779 "src/prebuilt/wast-lexer-gen.cc"
yy720:
		yych = *++cursor_;
		if (yych == 'n') goto yy937;
		goto yy87;
yy721:
		yych = *++cursor_;
		if (yych == 'b') goto yy938;
		if (yych == 'd') goto yy939;
		goto yy87;
yy722:
		yych = *++cursor_;
		if (yych == 'o') goto yy940;
		goto yy87;
yy723:
		yych = *++cursor_;
		if (yych == 'i') goto yy941;
		goto yy87;
yy724:
		yych = *++cursor_;
		if (yych == 'q') goto yy942;
		if (yych == 'x') goto yy944;
		goto yy87;
yy725:
		yych = *++cursor_;
		if (yych == 'e') goto yy945;
		if (yych == 't') goto yy947;
		goto yy87;
yy726:
		yych = *++cursor_;
		if (yych == 'e') goto yy949;
		if (yych == 't') goto yy951;
		goto yy87;
yy727:
		yych = *++cursor_;
		if (yych <= 'h') {
			if (yych == 'a') goto yy953;
			goto yy87;
		} else {
			if (yych <= 'i') goto yy954;
			if (yych == 'u') goto yy955;
			goto yy87;
		}
yy728:
		yych = *++cursor_;
		if (yych == 'e') goto yy956;
		goto yy87;
yy729:
		yych = *++cursor_;
		if (yych == 'e') goto yy958;
		goto yy87;
yy730:
		yych = *++cursor_;
		if (yych <= 'q') {
			if (yych <= 'o') goto yy87;
			if (yych <= 'p') goto yy959;
			goto yy960;
		} else {
			if (yych == 'u') goto yy961;
			goto yy87;
		}
yy731:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 312 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Abs); }
#line 4847 "src/prebuilt/wast-lexer-gen.cc"
yy733:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 359 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Add); }
#line 4855 "src/prebuilt/wast-lexer-gen.cc"
yy735:
		yych = *++cursor_;
		if (yych == 'l') goto yy962;
		goto yy87;
yy736:
		yych = *++cursor_;
		if (yych == 's') goto yy964;
		if (yych == 'v') goto yy965;
		goto yy87;
yy737:
		yych = *++cursor_;
		if (yych == 'y') goto yy966;
		goto yy87;
yy738:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 365 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Div); }
#line 4876 "src/prebuilt/wast-lexer-gen.cc"
yy740:
		yych = *++cursor_;
		if (yych == 'o') goto yy967;
		goto yy87;
yy741:
		yych = *++cursor_;
		if (yych == 'd') goto yy968;
		goto yy87;
yy742:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 369 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Max); }
#line 4892 "src/prebuilt/wast-lexer-gen.cc"
yy744:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 367 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Min); }
#line 4900 "src/prebuilt/wast-lexer-gen.cc"
yy746:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 363 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Mul); }
#line 4908 "src/prebuilt/wast-lexer-gen.cc"
yy748:
		yych = *++cursor_;
		if (yych == 'r') goto yy970;
		goto yy87;
yy749:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 310 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Neg); }
#line 4920 "src/prebuilt/wast-lexer-gen.cc"
yy751:
		yych = *++cursor_;
		if (yych == 'm') goto yy971;
		goto yy87;
yy752:
		yych = *++cursor_;
		if (yych == 'n') goto yy972;
		goto yy87;
yy753:
		yych = *++cursor_;
		if (yych == 't') goto yy973;
		goto yy87;
yy754:
		yych = *++cursor_;
		if (yych == 'r') goto yy975;
		goto yy87;
yy755:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 361 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Sub); }
#line 4944 "src/prebuilt/wast-lexer-gen.cc"
yy757:
		yych = *++cursor_;
		if (yych == 'n') goto yy976;
		goto yy87;
yy758:
		yych = *++cursor_;
		if (yych == 'b') goto yy977;
		if (yych == 'd') goto yy978;
		goto yy87;
yy759:
		yych = *++cursor_;
		if (yych == 'o') goto yy979;
		goto yy87;
yy760:
		yych = *++cursor_;
		if (yych == 'i') goto yy980;
		goto yy87;
yy761:
		yych = *++cursor_;
		if (yych == 'q') goto yy981;
		if (yych == 'x') goto yy983;
		goto yy87;
yy762:
		yych = *++cursor_;
		if (yych == 'e') goto yy984;
		if (yych == 't') goto yy986;
		goto yy87;
yy763:
		yych = *++cursor_;
		if (yych == 'e') goto yy988;
		if (yych == 't') goto yy990;
		goto yy87;
yy764:
		yych = *++cursor_;
		if (yych <= 'h') {
			if (yych == 'a') goto yy992;
			goto yy87;
		} else {
			if (yych <= 'i') goto yy993;
			if (yych == 'u') goto yy994;
			goto yy87;
		}
yy765:
		yych = *++cursor_;
		if (yych == 'e') goto yy995;
		goto yy87;
yy766:
		yych = *++cursor_;
		if (yych == 'e') goto yy997;
		goto yy87;
yy767:
		yych = *++cursor_;
		if (yych <= 'q') {
			if (yych <= 'o') goto yy87;
			if (yych <= 'p') goto yy998;
			goto yy999;
		} else {
			if (yych == 'u') goto yy1000;
			goto yy87;
		}
yy768:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 250 "src/wast-lexer.cc"
		{ RETURN_TYPE(ValueType, Funcref); }
#line 5012 "src/prebuilt/wast-lexer-gen.cc"
yy770:
		yych = *++cursor_;
		if (yych == 'b') goto yy1001;
		goto yy87;
yy771:
		yych = *++cursor_;
		if (yych == 'a') goto yy1002;
		goto yy87;
yy772:
		yych = *++cursor_;
		if (yych == 'g') goto yy1003;
		if (yych == 's') goto yy1004;
		goto yy87;
yy773:
		yych = *++cursor_;
		if (yych == 'm') goto yy1005;
		goto yy87;
yy774:
		yych = *++cursor_;
		if (yych <= 'k') {
			if (yych == 'd') goto yy1006;
			goto yy87;
		} else {
			if (yych <= 'l') goto yy1007;
			if (yych == 'n') goto yy1008;
			goto yy87;
		}
yy775:
		yych = *++cursor_;
		if (yych == 'q') goto yy1009;
		if (yych == 'x') goto yy1011;
		goto yy87;
yy776:
		yych = *++cursor_;
		if (yych == 'e') goto yy1012;
		if (yych == 't') goto yy1013;
		goto yy87;
yy777:
		yych = *++cursor_;
		if (yych == 'e') goto yy1014;
		if (yych == 't') goto yy1015;
		goto yy87;
yy778:
		yych = *++cursor_;
		if (yych == 'u') goto yy1016;
		goto yy87;
yy779:
		yych = *++cursor_;
		if (yych == 'e') goto yy1017;
		goto yy87;
yy780:
		yych = *++cursor_;
		if (yych == 'e') goto yy1019;
		goto yy87;
yy781:
		yych = *++cursor_;
		if (yych <= 'o') {
			if (yych == 'h') goto yy1020;
			goto yy87;
		} else {
			if (yych <= 'p') goto yy1021;
			if (yych == 'u') goto yy1022;
			goto yy87;
		}
yy782:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 328 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Add); }
#line 5084 "src/prebuilt/wast-lexer-gen.cc"
yy784:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 342 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32And); }
#line 5092 "src/prebuilt/wast-lexer-gen.cc"
yy786:
		yych = *++cursor_;
		if (yych == 'm') goto yy1023;
		goto yy87;
yy787:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 303 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32Clz); }
#line 5104 "src/prebuilt/wast-lexer-gen.cc"
yy789:
		yych = *++cursor_;
		if (yych == 's') goto yy1024;
		goto yy87;
yy790:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 305 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32Ctz); }
#line 5116 "src/prebuilt/wast-lexer-gen.cc"
yy792:
		yych = *++cursor_;
		if (yych == '_') goto yy1025;
		goto yy87;
yy793:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 301 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32Eqz); }
#line 5128 "src/prebuilt/wast-lexer-gen.cc"
yy795:
		yych = *++cursor_;
		if (yych == 'e') goto yy1026;
		goto yy87;
yy796:
		yych = *++cursor_;
		if (yych == 's') goto yy1027;
		if (yych == 'u') goto yy1029;
		goto yy87;
yy797:
		yych = *++cursor_;
		if (yych == 's') goto yy1031;
		if (yych == 'u') goto yy1033;
		goto yy87;
yy798:
		yych = *++cursor_;
		if (yych == 's') goto yy1035;
		if (yych == 'u') goto yy1037;
		goto yy87;
yy799:
		yych = *++cursor_;
		if (yych == 'd') goto yy1039;
		goto yy87;
yy800:
		yych = *++cursor_;
		if (yych == 's') goto yy1041;
		if (yych == 'u') goto yy1043;
		goto yy87;
yy801:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 332 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Mul); }
#line 5164 "src/prebuilt/wast-lexer-gen.cc"
yy803:
		yych = *++cursor_;
		if (yych == 'c') goto yy1045;
		goto yy87;
yy804:
		yych = *++cursor_;
		if (yych == 'n') goto yy1046;
		goto yy87;
yy805:
		yych = *++cursor_;
		if (yych == '_') goto yy1047;
		goto yy87;
yy806:
		yych = *++cursor_;
		if (yych == 'l') goto yy1048;
		if (yych == 'r') goto yy1050;
		goto yy87;
yy807:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 348 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Shl); }
#line 5189 "src/prebuilt/wast-lexer-gen.cc"
yy809:
		yych = *++cursor_;
		if (yych == '_') goto yy1052;
		goto yy87;
yy810:
		yych = *++cursor_;
		if (yych == 'r') goto yy1053;
		goto yy87;
yy811:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 330 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Sub); }
#line 5205 "src/prebuilt/wast-lexer-gen.cc"
yy813:
		yych = *++cursor_;
		if (yych == 'n') goto yy1054;
		goto yy87;
yy814:
		yych = *++cursor_;
		if (yych == 'p') goto yy1055;
		goto yy87;
yy815:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 346 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Xor); }
#line 5221 "src/prebuilt/wast-lexer-gen.cc"
yy817:
		yych = *++cursor_;
		if (yych <= 'k') {
			if (yych == 'd') goto yy1056;
			goto yy87;
		} else {
			if (yych <= 'l') goto yy1057;
			if (yych == 'n') goto yy1058;
			goto yy87;
		}
yy818:
		yych = *++cursor_;
		if (yych == 'q') goto yy1059;
		if (yych == 'x') goto yy1061;
		goto yy87;
yy819:
		yych = *++cursor_;
		if (yych == 'e') goto yy1062;
		if (yych == 't') goto yy1063;
		goto yy87;
yy820:
		yych = *++cursor_;
		if (yych == 'e') goto yy1064;
		if (yych == 't') goto yy1065;
		goto yy87;
yy821:
		yych = *++cursor_;
		if (yych == 'u') goto yy1066;
		goto yy87;
yy822:
		yych = *++cursor_;
		if (yych == 'e') goto yy1067;
		goto yy87;
yy823:
		yych = *++cursor_;
		if (yych == 'e') goto yy1069;
		goto yy87;
yy824:
		yych = *++cursor_;
		if (yych <= 'o') {
			if (yych == 'h') goto yy1070;
			goto yy87;
		} else {
			if (yych <= 'p') goto yy1071;
			if (yych == 'u') goto yy1072;
			goto yy87;
		}
yy825:
		yych = *++cursor_;
		if (yych == 'r') goto yy1073;
		goto yy87;
yy826:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 329 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Add); }
#line 5280 "src/prebuilt/wast-lexer-gen.cc"
yy828:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 343 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64And); }
#line 5288 "src/prebuilt/wast-lexer-gen.cc"
yy830:
		yych = *++cursor_;
		if (yych == 'm') goto yy1074;
		goto yy87;
yy831:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 304 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64Clz); }
#line 5300 "src/prebuilt/wast-lexer-gen.cc"
yy833:
		yych = *++cursor_;
		if (yych == 's') goto yy1075;
		goto yy87;
yy834:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 306 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64Ctz); }
#line 5312 "src/prebuilt/wast-lexer-gen.cc"
yy836:
		yych = *++cursor_;
		if (yych == '_') goto yy1076;
		goto yy87;
yy837:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 302 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64Eqz); }
#line 5324 "src/prebuilt/wast-lexer-gen.cc"
yy839:
		yych = *++cursor_;
		if (yych == 'e') goto yy1077;
		goto yy87;
yy840:
		yych = *++cursor_;
		if (yych == 's') goto yy1078;
		if (yych == 'u') goto yy1080;
		goto yy87;
yy841:
		yych = *++cursor_;
		if (yych == 's') goto yy1082;
		if (yych == 'u') goto yy1084;
		goto yy87;
yy842:
		yych = *++cursor_;
		if (yych == 's') goto yy1086;
		if (yych == 'u') goto yy1088;
		goto yy87;
yy843:
		yych = *++cursor_;
		if (yych == 'd') goto yy1090;
		goto yy87;
yy844:
		yych = *++cursor_;
		if (yych == 's') goto yy1092;
		if (yych == 'u') goto yy1094;
		goto yy87;
yy845:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 333 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Mul); }
#line 5360 "src/prebuilt/wast-lexer-gen.cc"
yy847:
		yych = *++cursor_;
		if (yych == 'c') goto yy1096;
		goto yy87;
yy848:
		yych = *++cursor_;
		if (yych == 'n') goto yy1097;
		goto yy87;
yy849:
		yych = *++cursor_;
		if (yych == '_') goto yy1098;
		goto yy87;
yy850:
		yych = *++cursor_;
		if (yych == 'l') goto yy1099;
		if (yych == 'r') goto yy1101;
		goto yy87;
yy851:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 349 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Shl); }
#line 5385 "src/prebuilt/wast-lexer-gen.cc"
yy853:
		yych = *++cursor_;
		if (yych == '_') goto yy1103;
		goto yy87;
yy854:
		yych = *++cursor_;
		if (yych == 'r') goto yy1104;
		goto yy87;
yy855:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 331 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Sub); }
#line 5401 "src/prebuilt/wast-lexer-gen.cc"
yy857:
		yych = *++cursor_;
		if (yych == 'n') goto yy1105;
		goto yy87;
yy858:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 347 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Xor); }
#line 5413 "src/prebuilt/wast-lexer-gen.cc"
yy860:
		yych = *++cursor_;
		if (yych <= 'k') {
			if (yych == 'd') goto yy1106;
			goto yy87;
		} else {
			if (yych <= 'l') goto yy1107;
			if (yych == 'n') goto yy1108;
			goto yy87;
		}
yy861:
		yych = *++cursor_;
		if (yych == 'x') goto yy1109;
		goto yy87;
yy862:
		yych = *++cursor_;
		if (yych == 'e') goto yy1110;
		goto yy87;
yy863:
		yych = *++cursor_;
		if (yych == 'e') goto yy1111;
		goto yy87;
yy864:
		yych = *++cursor_;
		if (yych <= 'o') {
			if (yych == 'h') goto yy1112;
			goto yy87;
		} else {
			if (yych <= 'p') goto yy1113;
			if (yych == 'u') goto yy1114;
			goto yy87;
		}
yy865:
		yych = *++cursor_;
		if (yych == 'r') goto yy1115;
		goto yy87;
yy866:
		yych = *++cursor_;
		if (yych <= 'k') {
			if (yych == 'd') goto yy1116;
			goto yy87;
		} else {
			if (yych <= 'l') goto yy1117;
			if (yych == 'n') goto yy1118;
			goto yy87;
		}
yy867:
		yych = *++cursor_;
		if (yych == 'q') goto yy1119;
		if (yych == 'x') goto yy1121;
		goto yy87;
yy868:
		yych = *++cursor_;
		if (yych == 'e') goto yy1122;
		if (yych == 't') goto yy1123;
		goto yy87;
yy869:
		yych = *++cursor_;
		if (yych == 'e') goto yy1124;
		if (yych == 't') goto yy1125;
		goto yy87;
yy870:
		yych = *++cursor_;
		if (yych == 'u') goto yy1126;
		goto yy87;
yy871:
		yych = *++cursor_;
		if (yych == 'e') goto yy1127;
		goto yy87;
yy872:
		yych = *++cursor_;
		if (yych == 'e') goto yy1129;
		goto yy87;
yy873:
		yych = *++cursor_;
		if (yych <= 'o') {
			if (yych == 'h') goto yy1130;
			goto yy87;
		} else {
			if (yych <= 'p') goto yy1131;
			if (yych == 'u') goto yy1132;
			goto yy87;
		}
yy874:
		yych = *++cursor_;
		if (yych == 'e') goto yy1133;
		goto yy87;
yy875:
		yych = *++cursor_;
		if (yych == 'e') goto yy1134;
		goto yy87;
yy876:
		yych = *++cursor_;
		if (yych == 'e') goto yy1135;
		goto yy87;
yy877:
		yych = *++cursor_;
		switch (yych) {
		case 'c':	goto yy1136;
		case 'f':	goto yy1137;
		case 'g':	goto yy1138;
		case 'i':	goto yy1139;
		case 's':	goto yy1140;
		default:	goto yy87;
		}
yy878:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ':') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy267;
				goto yy86;
			} else {
				if (yych <= ')') goto yy267;
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy878;
				goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') goto yy267;
				if (yych <= '@') goto yy86;
				if (yych <= 'F') goto yy878;
				goto yy86;
			} else {
				if (yych <= '`') {
					if (yych <= '_') goto yy650;
					goto yy86;
				} else {
					if (yych <= 'f') goto yy878;
					if (yych <= '~') goto yy86;
					goto yy267;
				}
			}
		}
yy880:
		yych = *++cursor_;
		if (yych <= '/') goto yy87;
		if (yych <= '0') goto yy1141;
		if (yych <= '9') goto yy1143;
		goto yy87;
yy881:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 725 "src/wast-lexer.cc"
		{ RETURN(Passive); }
#line 5564 "src/prebuilt/wast-lexer-gen.cc"
yy883:
		yych = *++cursor_;
		if (yych == 'n') goto yy1145;
		goto yy87;
yy884:
		yych = *++cursor_;
		if (yych == 'l') goto yy1146;
		goto yy87;
yy885:
		yych = *++cursor_;
		if (yych == 'r') goto yy1148;
		goto yy87;
yy886:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 746 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Rethrow); }
#line 5584 "src/prebuilt/wast-lexer-gen.cc"
yy888:
		yych = *++cursor_;
		if (yych == 'c') goto yy1150;
		goto yy87;
yy889:
		yych = *++cursor_;
		if (yych == 'b') goto yy1151;
		goto yy87;
yy890:
		yych = *++cursor_;
		if (yych == 'a') goto yy1152;
		goto yy87;
yy891:
		yych = *++cursor_;
		if (yych == 'o') goto yy1153;
		goto yy87;
yy892:
		yych = *++cursor_;
		if (yych == 'e') goto yy1154;
		if (yych == 'r') goto yy1155;
		goto yy87;
yy893:
		yych = *++cursor_;
		if (yych == 'n') goto yy1156;
		goto yy87;
yy894:
		yych = *++cursor_;
		if (yych == 'e') goto yy1157;
		if (yych == 'i') goto yy1158;
		goto yy87;
yy895:
		yych = *++cursor_;
		if (yych == 'a') goto yy1159;
		goto yy87;
yy896:
		yych = *++cursor_;
		if (yych == 'a') goto yy1160;
		goto yy87;
yy897:
		yych = *++cursor_;
		if (yych == 'd') goto yy1161;
		goto yy87;
yy898:
		yych = *++cursor_;
		if (yych == 't') goto yy1163;
		goto yy87;
yy899:
		yych = *++cursor_;
		if (yych == 'n') goto yy1164;
		goto yy87;
yy900:
		yych = *++cursor_;
		if (yych == 'a') goto yy1165;
		goto yy87;
yy901:
		yych = *++cursor_;
		if (yych == 't') goto yy1166;
		goto yy87;
yy902:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 583 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, V128Or); }
#line 5650 "src/prebuilt/wast-lexer-gen.cc"
yy904:
		yych = *++cursor_;
		if (yych == 'o') goto yy1168;
		goto yy87;
yy905:
		yych = *++cursor_;
		if (yych == 'r') goto yy1169;
		goto yy87;
yy906:
		yych = *++cursor_;
		if (yych == 'h') goto yy1171;
		goto yy87;
yy907:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ')') {
			if (yych <= '!') {
				if (yych <= ' ') goto yy88;
				goto yy86;
			} else {
				if (yych <= '"') goto yy88;
				if (yych <= '\'') goto yy86;
				goto yy88;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy682;
				goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy908:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy88;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy88;
				if (yych <= '/') goto yy86;
				goto yy1172;
			}
		} else {
			if (yych <= 'F') {
				if (yych == ';') goto yy88;
				if (yych <= '@') goto yy86;
				goto yy1172;
			} else {
				if (yych <= '`') goto yy86;
				if (yych <= 'f') goto yy1172;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy909:
		yych = *++cursor_;
		if (yych == 'x') goto yy1174;
		goto yy87;
yy910:
		yych = *++cursor_;
		if (yych == 'n') goto yy1175;
		goto yy87;
yy911:
		yych = *++cursor_;
		if (yych == 'a') goto yy1176;
		goto yy87;
yy912:
		yych = *++cursor_;
		if (yych == 'e') goto yy1177;
		goto yy87;
yy913:
		yych = *++cursor_;
		if (yych == 'r') goto yy1178;
		goto yy87;
yy914:
		yych = *++cursor_;
		if (yych == 'n') goto yy1179;
		goto yy87;
yy915:
		yych = *++cursor_;
		if (yych == 'o') goto yy1180;
		goto yy87;
yy916:
		yych = *++cursor_;
		if (yych == 'n') goto yy1181;
		goto yy87;
yy917:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 261 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(BrTable); }
#line 5752 "src/prebuilt/wast-lexer-gen.cc"
yy919:
		yych = *++cursor_;
		if (yych == 'i') goto yy1183;
		goto yy87;
yy920:
		yych = *++cursor_;
		if (yych == 'm') goto yy1184;
		goto yy87;
yy921:
		yych = *++cursor_;
		if (yych == 'p') goto yy1185;
		goto yy87;
yy922:
		yych = *++cursor_;
		if (yych == 'p') goto yy1187;
		goto yy87;
yy923:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 315 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Ceil); }
#line 5776 "src/prebuilt/wast-lexer-gen.cc"
yy925:
		yych = *++cursor_;
		if (yych == 't') goto yy1189;
		goto yy87;
yy926:
		yych = *++cursor_;
		if (yych == 'e') goto yy1191;
		goto yy87;
yy927:
		yych = *++cursor_;
		if (yych == 's') goto yy1192;
		goto yy87;
yy928:
		yych = *++cursor_;
		if (yych == 't') goto yy1193;
		goto yy87;
yy929:
		yych = *++cursor_;
		if (yych == 'r') goto yy1194;
		goto yy87;
yy930:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 274 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, F32Load); }
#line 5804 "src/prebuilt/wast-lexer-gen.cc"
yy932:
		yych = *++cursor_;
		if (yych == 'e') goto yy1196;
		goto yy87;
yy933:
		yych = *++cursor_;
		if (yych == 't') goto yy1197;
		goto yy87;
yy934:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 313 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Sqrt); }
#line 5820 "src/prebuilt/wast-lexer-gen.cc"
yy936:
		yych = *++cursor_;
		if (yych == 'e') goto yy1198;
		goto yy87;
yy937:
		yych = *++cursor_;
		if (yych == 'c') goto yy1200;
		goto yy87;
yy938:
		yych = *++cursor_;
		if (yych == 's') goto yy1202;
		goto yy87;
yy939:
		yych = *++cursor_;
		if (yych == 'd') goto yy1204;
		goto yy87;
yy940:
		yych = *++cursor_;
		if (yych == 'n') goto yy1206;
		goto yy87;
yy941:
		yych = *++cursor_;
		if (yych == 'v') goto yy1207;
		goto yy87;
yy942:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 598 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32X4Eq); }
#line 5852 "src/prebuilt/wast-lexer-gen.cc"
yy944:
		yych = *++cursor_;
		if (yych == 't') goto yy1209;
		goto yy87;
yy945:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 635 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32X4Ge); }
#line 5864 "src/prebuilt/wast-lexer-gen.cc"
yy947:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 627 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32X4Gt); }
#line 5872 "src/prebuilt/wast-lexer-gen.cc"
yy949:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 619 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32X4Le); }
#line 5880 "src/prebuilt/wast-lexer-gen.cc"
yy951:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 611 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32X4Lt); }
#line 5888 "src/prebuilt/wast-lexer-gen.cc"
yy953:
		yych = *++cursor_;
		if (yych == 'x') goto yy1210;
		goto yy87;
yy954:
		yych = *++cursor_;
		if (yych == 'n') goto yy1212;
		goto yy87;
yy955:
		yych = *++cursor_;
		if (yych == 'l') goto yy1214;
		goto yy87;
yy956:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy957;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'f') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'g') goto yy1216;
				if (yych <= '~') goto yy86;
			}
		}
yy957:
#line 603 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F32X4Ne); }
#line 5921 "src/prebuilt/wast-lexer-gen.cc"
yy958:
		yych = *++cursor_;
		if (yych == 'p') goto yy1218;
		goto yy87;
yy959:
		yych = *++cursor_;
		if (yych == 'l') goto yy1219;
		goto yy87;
yy960:
		yych = *++cursor_;
		if (yych == 'r') goto yy1220;
		goto yy87;
yy961:
		yych = *++cursor_;
		if (yych == 'b') goto yy1221;
		goto yy87;
yy962:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 316 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Ceil); }
#line 5945 "src/prebuilt/wast-lexer-gen.cc"
yy964:
		yych = *++cursor_;
		if (yych == 't') goto yy1223;
		goto yy87;
yy965:
		yych = *++cursor_;
		if (yych == 'e') goto yy1225;
		goto yy87;
yy966:
		yych = *++cursor_;
		if (yych == 's') goto yy1226;
		goto yy87;
yy967:
		yych = *++cursor_;
		if (yych == 'r') goto yy1227;
		goto yy87;
yy968:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 275 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, F64Load); }
#line 5969 "src/prebuilt/wast-lexer-gen.cc"
yy970:
		yych = *++cursor_;
		if (yych == 'e') goto yy1229;
		goto yy87;
yy971:
		yych = *++cursor_;
		if (yych == 'o') goto yy1230;
		goto yy87;
yy972:
		yych = *++cursor_;
		if (yych == 't') goto yy1231;
		goto yy87;
yy973:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 314 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Sqrt); }
#line 5989 "src/prebuilt/wast-lexer-gen.cc"
yy975:
		yych = *++cursor_;
		if (yych == 'e') goto yy1232;
		goto yy87;
yy976:
		yych = *++cursor_;
		if (yych == 'c') goto yy1234;
		goto yy87;
yy977:
		yych = *++cursor_;
		if (yych == 's') goto yy1236;
		goto yy87;
yy978:
		yych = *++cursor_;
		if (yych == 'd') goto yy1238;
		goto yy87;
yy979:
		yych = *++cursor_;
		if (yych == 'n') goto yy1240;
		goto yy87;
yy980:
		yych = *++cursor_;
		if (yych == 'v') goto yy1241;
		goto yy87;
yy981:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 599 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64X2Eq); }
#line 6021 "src/prebuilt/wast-lexer-gen.cc"
yy983:
		yych = *++cursor_;
		if (yych == 't') goto yy1243;
		goto yy87;
yy984:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 636 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64X2Ge); }
#line 6033 "src/prebuilt/wast-lexer-gen.cc"
yy986:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 628 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64X2Gt); }
#line 6041 "src/prebuilt/wast-lexer-gen.cc"
yy988:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 620 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64X2Le); }
#line 6049 "src/prebuilt/wast-lexer-gen.cc"
yy990:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 612 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64X2Lt); }
#line 6057 "src/prebuilt/wast-lexer-gen.cc"
yy992:
		yych = *++cursor_;
		if (yych == 'x') goto yy1244;
		goto yy87;
yy993:
		yych = *++cursor_;
		if (yych == 'n') goto yy1246;
		goto yy87;
yy994:
		yych = *++cursor_;
		if (yych == 'l') goto yy1248;
		goto yy87;
yy995:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy996;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'f') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'g') goto yy1250;
				if (yych <= '~') goto yy86;
			}
		}
yy996:
#line 604 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, F64X2Ne); }
#line 6090 "src/prebuilt/wast-lexer-gen.cc"
yy997:
		yych = *++cursor_;
		if (yych == 'p') goto yy1252;
		goto yy87;
yy998:
		yych = *++cursor_;
		if (yych == 'l') goto yy1253;
		goto yy87;
yy999:
		yych = *++cursor_;
		if (yych == 'r') goto yy1254;
		goto yy87;
yy1000:
		yych = *++cursor_;
		if (yych == 'b') goto yy1255;
		goto yy87;
yy1001:
		yych = *++cursor_;
		if (yych == 'a') goto yy1257;
		goto yy87;
yy1002:
		yych = *++cursor_;
		if (yych == 'l') goto yy1258;
		goto yy87;
yy1003:
		yych = *++cursor_;
		if (yych == 'e') goto yy1260;
		goto yy87;
yy1004:
		yych = *++cursor_;
		if (yych == 'e') goto yy1261;
		goto yy87;
yy1005:
		yych = *++cursor_;
		if (yych == 'o') goto yy1262;
		goto yy87;
yy1006:
		yych = *++cursor_;
		if (yych == 'd') goto yy1263;
		goto yy87;
yy1007:
		yych = *++cursor_;
		if (yych == 'l') goto yy1265;
		goto yy87;
yy1008:
		yych = *++cursor_;
		if (yych == 'y') goto yy1266;
		goto yy87;
yy1009:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 596 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8Eq); }
#line 6146 "src/prebuilt/wast-lexer-gen.cc"
yy1011:
		yych = *++cursor_;
		if (yych == 't') goto yy1267;
		goto yy87;
yy1012:
		yych = *++cursor_;
		if (yych == '_') goto yy1268;
		goto yy87;
yy1013:
		yych = *++cursor_;
		if (yych == '_') goto yy1269;
		goto yy87;
yy1014:
		yych = *++cursor_;
		if (yych == '_') goto yy1270;
		goto yy87;
yy1015:
		yych = *++cursor_;
		if (yych == '_') goto yy1271;
		goto yy87;
yy1016:
		yych = *++cursor_;
		if (yych == 'l') goto yy1272;
		goto yy87;
yy1017:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1018;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'f') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'g') goto yy1274;
				if (yych <= '~') goto yy86;
			}
		}
yy1018:
#line 601 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8Ne); }
#line 6191 "src/prebuilt/wast-lexer-gen.cc"
yy1019:
		yych = *++cursor_;
		if (yych == 'p') goto yy1276;
		goto yy87;
yy1020:
		yych = *++cursor_;
		if (yych == 'l') goto yy1277;
		if (yych == 'r') goto yy1279;
		goto yy87;
yy1021:
		yych = *++cursor_;
		if (yych == 'l') goto yy1280;
		goto yy87;
yy1022:
		yych = *++cursor_;
		if (yych == 'b') goto yy1281;
		goto yy87;
yy1023:
		yych = *++cursor_;
		if (yych == 'i') goto yy1283;
		goto yy87;
yy1024:
		yych = *++cursor_;
		if (yych == 't') goto yy1284;
		goto yy87;
yy1025:
		yych = *++cursor_;
		if (yych == 's') goto yy1286;
		if (yych == 'u') goto yy1288;
		goto yy87;
yy1026:
		yych = *++cursor_;
		if (yych == 'n') goto yy1290;
		goto yy87;
yy1027:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 388 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32GeS); }
#line 6233 "src/prebuilt/wast-lexer-gen.cc"
yy1029:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 390 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32GeU); }
#line 6241 "src/prebuilt/wast-lexer-gen.cc"
yy1031:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 384 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32GtS); }
#line 6249 "src/prebuilt/wast-lexer-gen.cc"
yy1033:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 386 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32GtU); }
#line 6257 "src/prebuilt/wast-lexer-gen.cc"
yy1035:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 380 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32LeS); }
#line 6265 "src/prebuilt/wast-lexer-gen.cc"
yy1037:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 382 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32LeU); }
#line 6273 "src/prebuilt/wast-lexer-gen.cc"
yy1039:
		++cursor_;
		if ((yych = *cursor_) <= '0') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= '8') {
				if (yych <= '1') goto yy1291;
				if (yych <= '7') goto yy86;
				goto yy1292;
			} else {
				if (yych == ';') goto yy1040;
				if (yych <= '~') goto yy86;
			}
		}
yy1040:
#line 272 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I32Load); }
#line 6296 "src/prebuilt/wast-lexer-gen.cc"
yy1041:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 376 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32LtS); }
#line 6304 "src/prebuilt/wast-lexer-gen.cc"
yy1043:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 378 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32LtU); }
#line 6312 "src/prebuilt/wast-lexer-gen.cc"
yy1045:
		yych = *++cursor_;
		if (yych == 'n') goto yy1293;
		goto yy87;
yy1046:
		yych = *++cursor_;
		if (yych == 't') goto yy1294;
		goto yy87;
yy1047:
		yych = *++cursor_;
		if (yych == 's') goto yy1295;
		if (yych == 'u') goto yy1297;
		goto yy87;
yy1048:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 354 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Rotl); }
#line 6333 "src/prebuilt/wast-lexer-gen.cc"
yy1050:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 356 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32Rotr); }
#line 6341 "src/prebuilt/wast-lexer-gen.cc"
yy1052:
		yych = *++cursor_;
		if (yych == 's') goto yy1299;
		if (yych == 'u') goto yy1301;
		goto yy87;
yy1053:
		yych = *++cursor_;
		if (yych == 'e') goto yy1303;
		goto yy87;
yy1054:
		yych = *++cursor_;
		if (yych == 'c') goto yy1305;
		goto yy87;
yy1055:
		yych = *++cursor_;
		if (yych == '/') goto yy1306;
		if (yych == '_') goto yy1307;
		goto yy87;
yy1056:
		yych = *++cursor_;
		if (yych == 'd') goto yy1308;
		goto yy87;
yy1057:
		yych = *++cursor_;
		if (yych == 'l') goto yy1310;
		goto yy87;
yy1058:
		yych = *++cursor_;
		if (yych == 'y') goto yy1311;
		goto yy87;
yy1059:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 597 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4Eq); }
#line 6379 "src/prebuilt/wast-lexer-gen.cc"
yy1061:
		yych = *++cursor_;
		if (yych == 't') goto yy1312;
		goto yy87;
yy1062:
		yych = *++cursor_;
		if (yych == '_') goto yy1313;
		goto yy87;
yy1063:
		yych = *++cursor_;
		if (yych == '_') goto yy1314;
		goto yy87;
yy1064:
		yych = *++cursor_;
		if (yych == '_') goto yy1315;
		goto yy87;
yy1065:
		yych = *++cursor_;
		if (yych == '_') goto yy1316;
		goto yy87;
yy1066:
		yych = *++cursor_;
		if (yych == 'l') goto yy1317;
		goto yy87;
yy1067:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1068;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'f') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'g') goto yy1319;
				if (yych <= '~') goto yy86;
			}
		}
yy1068:
#line 602 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4Ne); }
#line 6424 "src/prebuilt/wast-lexer-gen.cc"
yy1069:
		yych = *++cursor_;
		if (yych == 'p') goto yy1321;
		goto yy87;
yy1070:
		yych = *++cursor_;
		if (yych == 'l') goto yy1322;
		if (yych == 'r') goto yy1324;
		goto yy87;
yy1071:
		yych = *++cursor_;
		if (yych == 'l') goto yy1325;
		goto yy87;
yy1072:
		yych = *++cursor_;
		if (yych == 'b') goto yy1326;
		goto yy87;
yy1073:
		yych = *++cursor_;
		if (yych == 'u') goto yy1328;
		goto yy87;
yy1074:
		yych = *++cursor_;
		if (yych == 'i') goto yy1329;
		goto yy87;
yy1075:
		yych = *++cursor_;
		if (yych == 't') goto yy1330;
		goto yy87;
yy1076:
		yych = *++cursor_;
		if (yych == 's') goto yy1332;
		if (yych == 'u') goto yy1334;
		goto yy87;
yy1077:
		yych = *++cursor_;
		if (yych == 'n') goto yy1336;
		goto yy87;
yy1078:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 389 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64GeS); }
#line 6470 "src/prebuilt/wast-lexer-gen.cc"
yy1080:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 391 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64GeU); }
#line 6478 "src/prebuilt/wast-lexer-gen.cc"
yy1082:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 385 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64GtS); }
#line 6486 "src/prebuilt/wast-lexer-gen.cc"
yy1084:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 387 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64GtU); }
#line 6494 "src/prebuilt/wast-lexer-gen.cc"
yy1086:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 381 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64LeS); }
#line 6502 "src/prebuilt/wast-lexer-gen.cc"
yy1088:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 383 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64LeU); }
#line 6510 "src/prebuilt/wast-lexer-gen.cc"
yy1090:
		++cursor_;
		if ((yych = *cursor_) <= '1') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy1091;
				if (yych <= '0') goto yy86;
				goto yy1337;
			}
		} else {
			if (yych <= '8') {
				if (yych == '3') goto yy1338;
				if (yych <= '7') goto yy86;
				goto yy1339;
			} else {
				if (yych == ';') goto yy1091;
				if (yych <= '~') goto yy86;
			}
		}
yy1091:
#line 273 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load); }
#line 6535 "src/prebuilt/wast-lexer-gen.cc"
yy1092:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 377 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64LtS); }
#line 6543 "src/prebuilt/wast-lexer-gen.cc"
yy1094:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 379 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I64LtU); }
#line 6551 "src/prebuilt/wast-lexer-gen.cc"
yy1096:
		yych = *++cursor_;
		if (yych == 'n') goto yy1340;
		goto yy87;
yy1097:
		yych = *++cursor_;
		if (yych == 't') goto yy1341;
		goto yy87;
yy1098:
		yych = *++cursor_;
		if (yych == 's') goto yy1342;
		if (yych == 'u') goto yy1344;
		goto yy87;
yy1099:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 355 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Rotl); }
#line 6572 "src/prebuilt/wast-lexer-gen.cc"
yy1101:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 357 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64Rotr); }
#line 6580 "src/prebuilt/wast-lexer-gen.cc"
yy1103:
		yych = *++cursor_;
		if (yych == 's') goto yy1346;
		if (yych == 'u') goto yy1348;
		goto yy87;
yy1104:
		yych = *++cursor_;
		if (yych == 'e') goto yy1350;
		goto yy87;
yy1105:
		yych = *++cursor_;
		if (yych == 'c') goto yy1352;
		goto yy87;
yy1106:
		yych = *++cursor_;
		if (yych == 'd') goto yy1353;
		goto yy87;
yy1107:
		yych = *++cursor_;
		if (yych == 'l') goto yy1355;
		goto yy87;
yy1108:
		yych = *++cursor_;
		if (yych == 'y') goto yy1356;
		goto yy87;
yy1109:
		yych = *++cursor_;
		if (yych == 't') goto yy1357;
		goto yy87;
yy1110:
		yych = *++cursor_;
		if (yych == 'g') goto yy1358;
		goto yy87;
yy1111:
		yych = *++cursor_;
		if (yych == 'p') goto yy1360;
		goto yy87;
yy1112:
		yych = *++cursor_;
		if (yych == 'l') goto yy1361;
		if (yych == 'r') goto yy1363;
		goto yy87;
yy1113:
		yych = *++cursor_;
		if (yych == 'l') goto yy1364;
		goto yy87;
yy1114:
		yych = *++cursor_;
		if (yych == 'b') goto yy1365;
		goto yy87;
yy1115:
		yych = *++cursor_;
		if (yych == 'u') goto yy1367;
		goto yy87;
yy1116:
		yych = *++cursor_;
		if (yych == 'd') goto yy1368;
		goto yy87;
yy1117:
		yych = *++cursor_;
		if (yych == 'l') goto yy1370;
		goto yy87;
yy1118:
		yych = *++cursor_;
		if (yych == 'y') goto yy1371;
		goto yy87;
yy1119:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 595 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16Eq); }
#line 6654 "src/prebuilt/wast-lexer-gen.cc"
yy1121:
		yych = *++cursor_;
		if (yych == 't') goto yy1372;
		goto yy87;
yy1122:
		yych = *++cursor_;
		if (yych == '_') goto yy1373;
		goto yy87;
yy1123:
		yych = *++cursor_;
		if (yych == '_') goto yy1374;
		goto yy87;
yy1124:
		yych = *++cursor_;
		if (yych == '_') goto yy1375;
		goto yy87;
yy1125:
		yych = *++cursor_;
		if (yych == '_') goto yy1376;
		goto yy87;
yy1126:
		yych = *++cursor_;
		if (yych == 'l') goto yy1377;
		goto yy87;
yy1127:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1128;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= 'f') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= 'g') goto yy1379;
				if (yych <= '~') goto yy86;
			}
		}
yy1128:
#line 600 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16Ne); }
#line 6699 "src/prebuilt/wast-lexer-gen.cc"
yy1129:
		yych = *++cursor_;
		if (yych == 'p') goto yy1381;
		goto yy87;
yy1130:
		yych = *++cursor_;
		if (yych == 'l') goto yy1382;
		if (yych == 'r') goto yy1384;
		goto yy87;
yy1131:
		yych = *++cursor_;
		if (yych == 'l') goto yy1385;
		goto yy87;
yy1132:
		yych = *++cursor_;
		if (yych == 'b') goto yy1386;
		goto yy87;
yy1133:
		yych = *++cursor_;
		if (yych == 't') goto yy1388;
		goto yy87;
yy1134:
		yych = *++cursor_;
		if (yych == 't') goto yy1390;
		goto yy87;
yy1135:
		yych = *++cursor_;
		if (yych == 'e') goto yy1392;
		goto yy87;
yy1136:
		yych = *++cursor_;
		if (yych == 'o') goto yy1394;
		goto yy87;
yy1137:
		yych = *++cursor_;
		if (yych == 'i') goto yy1395;
		goto yy87;
yy1138:
		yych = *++cursor_;
		if (yych == 'r') goto yy1396;
		goto yy87;
yy1139:
		yych = *++cursor_;
		if (yych == 'n') goto yy1397;
		goto yy87;
yy1140:
		yych = *++cursor_;
		if (yych == 'i') goto yy1398;
		goto yy87;
yy1141:
		++cursor_;
		if ((yych = *cursor_) <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy1142;
				if (yych <= '/') goto yy86;
				goto yy1143;
			}
		} else {
			if (yych <= '_') {
				if (yych == ';') goto yy1142;
				if (yych <= '^') goto yy86;
				goto yy1399;
			} else {
				if (yych == 'x') goto yy1400;
				if (yych <= '~') goto yy86;
			}
		}
yy1142:
#line 295 "src/wast-lexer.cc"
		{ RETURN_TEXT_AT(OffsetEqNat, 7); }
#line 6773 "src/prebuilt/wast-lexer-gen.cc"
yy1143:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '/') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy1142;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy1142;
				goto yy86;
			}
		} else {
			if (yych <= ';') {
				if (yych <= '9') goto yy1143;
				if (yych <= ':') goto yy86;
				goto yy1142;
			} else {
				if (yych == '_') goto yy1399;
				if (yych <= '~') goto yy86;
				goto yy1142;
			}
		}
yy1145:
		yych = *++cursor_;
		if (yych == 'u') goto yy1401;
		goto yy87;
yy1146:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 454 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(RefNull); }
#line 6809 "src/prebuilt/wast-lexer-gen.cc"
yy1148:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 726 "src/wast-lexer.cc"
		{ RETURN(Register); }
#line 6817 "src/prebuilt/wast-lexer-gen.cc"
yy1150:
		yych = *++cursor_;
		if (yych == 'a') goto yy1402;
		goto yy87;
yy1151:
		yych = *++cursor_;
		if (yych == 'a') goto yy1403;
		goto yy87;
yy1152:
		yych = *++cursor_;
		if (yych == 'l') goto yy1404;
		goto yy87;
yy1153:
		yych = *++cursor_;
		if (yych == 'p') goto yy1406;
		goto yy87;
yy1154:
		yych = *++cursor_;
		if (yych == 't') goto yy1407;
		goto yy87;
yy1155:
		yych = *++cursor_;
		if (yych == 'o') goto yy1409;
		goto yy87;
yy1156:
		yych = *++cursor_;
		if (yych == 'i') goto yy1410;
		goto yy87;
yy1157:
		yych = *++cursor_;
		if (yych == 't') goto yy1411;
		goto yy87;
yy1158:
		yych = *++cursor_;
		if (yych == 'z') goto yy1413;
		goto yy87;
yy1159:
		yych = *++cursor_;
		if (yych == 'l') goto yy1414;
		goto yy87;
yy1160:
		yych = *++cursor_;
		if (yych == 'b') goto yy1416;
		goto yy87;
yy1161:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 582 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, V128And); }
#line 6869 "src/prebuilt/wast-lexer-gen.cc"
yy1163:
		yych = *++cursor_;
		if (yych == 's') goto yy1417;
		goto yy87;
yy1164:
		yych = *++cursor_;
		if (yych == 's') goto yy1418;
		goto yy87;
yy1165:
		yych = *++cursor_;
		if (yych == 'd') goto yy1419;
		goto yy87;
yy1166:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 585 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, V128Not); }
#line 6889 "src/prebuilt/wast-lexer-gen.cc"
yy1168:
		yych = *++cursor_;
		if (yych == 'r') goto yy1421;
		goto yy87;
yy1169:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 584 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, V128Xor); }
#line 6901 "src/prebuilt/wast-lexer-gen.cc"
yy1171:
		yych = *++cursor_;
		if (yych == 'u') goto yy1422;
		goto yy87;
yy1172:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ':') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy681;
				goto yy86;
			} else {
				if (yych <= ')') goto yy681;
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy1172;
				goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') goto yy681;
				if (yych <= '@') goto yy86;
				if (yych <= 'F') goto yy1172;
				goto yy86;
			} else {
				if (yych <= '`') {
					if (yych <= '_') goto yy908;
					goto yy86;
				} else {
					if (yych <= 'f') goto yy1172;
					if (yych <= '~') goto yy86;
					goto yy681;
				}
			}
		}
yy1174:
		yych = *++cursor_;
		if (yych == 'h') goto yy1423;
		goto yy87;
yy1175:
		yych = *++cursor_;
		if (yych == 'v') goto yy1424;
		goto yy87;
yy1176:
		yych = *++cursor_;
		if (yych == 'l') goto yy1425;
		goto yy87;
yy1177:
		yych = *++cursor_;
		if (yych == 't') goto yy1426;
		goto yy87;
yy1178:
		yych = *++cursor_;
		if (yych == 'a') goto yy1427;
		goto yy87;
yy1179:
		yych = *++cursor_;
		if (yych == 'l') goto yy1428;
		goto yy87;
yy1180:
		yych = *++cursor_;
		if (yych == 't') goto yy1429;
		goto yy87;
yy1181:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 747 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(BrOnExn); }
#line 6973 "src/prebuilt/wast-lexer-gen.cc"
yy1183:
		yych = *++cursor_;
		if (yych == 'r') goto yy1430;
		goto yy87;
yy1184:
		yych = *++cursor_;
		if (yych == 'e') goto yy1431;
		goto yy87;
yy1185:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 442 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(DataDrop); }
#line 6989 "src/prebuilt/wast-lexer-gen.cc"
yy1187:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 448 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(ElemDrop); }
#line 6997 "src/prebuilt/wast-lexer-gen.cc"
yy1189:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 299 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Const, F32Const); }
#line 7005 "src/prebuilt/wast-lexer-gen.cc"
yy1191:
		yych = *++cursor_;
		if (yych == 'r') goto yy1432;
		goto yy87;
yy1192:
		yych = *++cursor_;
		if (yych == 'i') goto yy1433;
		goto yy87;
yy1193:
		yych = *++cursor_;
		if (yych == 'e') goto yy1434;
		goto yy87;
yy1194:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 317 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Floor); }
#line 7025 "src/prebuilt/wast-lexer-gen.cc"
yy1196:
		yych = *++cursor_;
		if (yych == 's') goto yy1435;
		goto yy87;
yy1197:
		yych = *++cursor_;
		if (yych == 'e') goto yy1436;
		goto yy87;
yy1198:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 278 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, F32Store); }
#line 7041 "src/prebuilt/wast-lexer-gen.cc"
yy1200:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 319 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Trunc); }
#line 7049 "src/prebuilt/wast-lexer-gen.cc"
yy1202:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 639 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32X4Abs); }
#line 7057 "src/prebuilt/wast-lexer-gen.cc"
yy1204:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 645 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32X4Add); }
#line 7065 "src/prebuilt/wast-lexer-gen.cc"
yy1206:
		yych = *++cursor_;
		if (yych == 'v') goto yy1437;
		goto yy87;
yy1207:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 649 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32X4Div); }
#line 7077 "src/prebuilt/wast-lexer-gen.cc"
yy1209:
		yych = *++cursor_;
		if (yych == 'r') goto yy1438;
		goto yy87;
yy1210:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 643 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32X4Max); }
#line 7089 "src/prebuilt/wast-lexer-gen.cc"
yy1212:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 641 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32X4Min); }
#line 7097 "src/prebuilt/wast-lexer-gen.cc"
yy1214:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 651 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32X4Mul); }
#line 7105 "src/prebuilt/wast-lexer-gen.cc"
yy1216:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 637 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32X4Neg); }
#line 7113 "src/prebuilt/wast-lexer-gen.cc"
yy1218:
		yych = *++cursor_;
		if (yych == 'l') goto yy1439;
		goto yy87;
yy1219:
		yych = *++cursor_;
		if (yych == 'a') goto yy1440;
		goto yy87;
yy1220:
		yych = *++cursor_;
		if (yych == 't') goto yy1441;
		goto yy87;
yy1221:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 647 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32X4Sub); }
#line 7133 "src/prebuilt/wast-lexer-gen.cc"
yy1223:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 300 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Const, F64Const); }
#line 7141 "src/prebuilt/wast-lexer-gen.cc"
yy1225:
		yych = *++cursor_;
		if (yych == 'r') goto yy1443;
		goto yy87;
yy1226:
		yych = *++cursor_;
		if (yych == 'i') goto yy1444;
		goto yy87;
yy1227:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 318 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Floor); }
#line 7157 "src/prebuilt/wast-lexer-gen.cc"
yy1229:
		yych = *++cursor_;
		if (yych == 's') goto yy1445;
		goto yy87;
yy1230:
		yych = *++cursor_;
		if (yych == 't') goto yy1446;
		goto yy87;
yy1231:
		yych = *++cursor_;
		if (yych == 'e') goto yy1447;
		goto yy87;
yy1232:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 279 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, F64Store); }
#line 7177 "src/prebuilt/wast-lexer-gen.cc"
yy1234:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 320 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Trunc); }
#line 7185 "src/prebuilt/wast-lexer-gen.cc"
yy1236:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 640 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64X2Abs); }
#line 7193 "src/prebuilt/wast-lexer-gen.cc"
yy1238:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 646 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64X2Add); }
#line 7201 "src/prebuilt/wast-lexer-gen.cc"
yy1240:
		yych = *++cursor_;
		if (yych == 'v') goto yy1448;
		goto yy87;
yy1241:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 650 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64X2Div); }
#line 7213 "src/prebuilt/wast-lexer-gen.cc"
yy1243:
		yych = *++cursor_;
		if (yych == 'r') goto yy1449;
		goto yy87;
yy1244:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 644 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64X2Max); }
#line 7225 "src/prebuilt/wast-lexer-gen.cc"
yy1246:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 642 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64X2Min); }
#line 7233 "src/prebuilt/wast-lexer-gen.cc"
yy1248:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 652 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64X2Mul); }
#line 7241 "src/prebuilt/wast-lexer-gen.cc"
yy1250:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 638 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64X2Neg); }
#line 7249 "src/prebuilt/wast-lexer-gen.cc"
yy1252:
		yych = *++cursor_;
		if (yych == 'l') goto yy1450;
		goto yy87;
yy1253:
		yych = *++cursor_;
		if (yych == 'a') goto yy1451;
		goto yy87;
yy1254:
		yych = *++cursor_;
		if (yych == 't') goto yy1452;
		goto yy87;
yy1255:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 648 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64X2Sub); }
#line 7269 "src/prebuilt/wast-lexer-gen.cc"
yy1257:
		yych = *++cursor_;
		if (yych == 'l') goto yy1454;
		goto yy87;
yy1258:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 668 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(LocalGet); }
#line 7281 "src/prebuilt/wast-lexer-gen.cc"
yy1260:
		yych = *++cursor_;
		if (yych == 't') goto yy1456;
		goto yy87;
yy1261:
		yych = *++cursor_;
		if (yych == 't') goto yy1458;
		goto yy87;
yy1262:
		yych = *++cursor_;
		if (yych == 'r') goto yy1460;
		goto yy87;
yy1263:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1264;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy1461;
				if (yych <= '~') goto yy86;
			}
		}
yy1264:
#line 548 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8Add); }
#line 7314 "src/prebuilt/wast-lexer-gen.cc"
yy1265:
		yych = *++cursor_;
		if (yych == '_') goto yy1462;
		goto yy87;
yy1266:
		yych = *++cursor_;
		if (yych == '_') goto yy1463;
		goto yy87;
yy1267:
		yych = *++cursor_;
		if (yych == 'r') goto yy1464;
		goto yy87;
yy1268:
		yych = *++cursor_;
		if (yych == 's') goto yy1465;
		if (yych == 'u') goto yy1467;
		goto yy87;
yy1269:
		yych = *++cursor_;
		if (yych == 's') goto yy1469;
		if (yych == 'u') goto yy1471;
		goto yy87;
yy1270:
		yych = *++cursor_;
		if (yych == 's') goto yy1473;
		if (yych == 'u') goto yy1475;
		goto yy87;
yy1271:
		yych = *++cursor_;
		if (yych == 's') goto yy1477;
		if (yych == 'u') goto yy1479;
		goto yy87;
yy1272:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 556 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8Mul); }
#line 7354 "src/prebuilt/wast-lexer-gen.cc"
yy1274:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 559 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I16X8Neg); }
#line 7362 "src/prebuilt/wast-lexer-gen.cc"
yy1276:
		yych = *++cursor_;
		if (yych == 'l') goto yy1481;
		goto yy87;
yy1277:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 571 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8Shl); }
#line 7374 "src/prebuilt/wast-lexer-gen.cc"
yy1279:
		yych = *++cursor_;
		if (yych == '_') goto yy1482;
		goto yy87;
yy1280:
		yych = *++cursor_;
		if (yych == 'a') goto yy1483;
		goto yy87;
yy1281:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1282;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy1484;
				if (yych <= '~') goto yy86;
			}
		}
yy1282:
#line 552 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8Sub); }
#line 7403 "src/prebuilt/wast-lexer-gen.cc"
yy1283:
		yych = *++cursor_;
		if (yych == 'c') goto yy1485;
		goto yy87;
yy1284:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 297 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Const, I32Const); }
#line 7415 "src/prebuilt/wast-lexer-gen.cc"
yy1286:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 334 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32DivS); }
#line 7423 "src/prebuilt/wast-lexer-gen.cc"
yy1288:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 336 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32DivU); }
#line 7431 "src/prebuilt/wast-lexer-gen.cc"
yy1290:
		yych = *++cursor_;
		if (yych == 'd') goto yy1486;
		goto yy87;
yy1291:
		yych = *++cursor_;
		if (yych == '6') goto yy1487;
		goto yy87;
yy1292:
		yych = *++cursor_;
		if (yych == '_') goto yy1488;
		goto yy87;
yy1293:
		yych = *++cursor_;
		if (yych == 't') goto yy1489;
		goto yy87;
yy1294:
		yych = *++cursor_;
		if (yych == 'e') goto yy1491;
		goto yy87;
yy1295:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 338 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32RemS); }
#line 7459 "src/prebuilt/wast-lexer-gen.cc"
yy1297:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 340 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32RemU); }
#line 7467 "src/prebuilt/wast-lexer-gen.cc"
yy1299:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 350 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32ShrS); }
#line 7475 "src/prebuilt/wast-lexer-gen.cc"
yy1301:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 352 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32ShrU); }
#line 7483 "src/prebuilt/wast-lexer-gen.cc"
yy1303:
		++cursor_;
		if ((yych = *cursor_) <= '0') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= '8') {
				if (yych <= '1') goto yy1492;
				if (yych <= '7') goto yy86;
				goto yy1493;
			} else {
				if (yych == ';') goto yy1304;
				if (yych <= '~') goto yy86;
			}
		}
yy1304:
#line 276 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I32Store); }
#line 7506 "src/prebuilt/wast-lexer-gen.cc"
yy1305:
		yych = *++cursor_;
		if (yych == '_') goto yy1495;
		goto yy87;
yy1306:
		yych = *++cursor_;
		if (yych == 'i') goto yy1496;
		goto yy87;
yy1307:
		yych = *++cursor_;
		if (yych == 'i') goto yy1497;
		goto yy87;
yy1308:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 549 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32X4Add); }
#line 7526 "src/prebuilt/wast-lexer-gen.cc"
yy1310:
		yych = *++cursor_;
		if (yych == '_') goto yy1498;
		goto yy87;
yy1311:
		yych = *++cursor_;
		if (yych == '_') goto yy1499;
		goto yy87;
yy1312:
		yych = *++cursor_;
		if (yych == 'r') goto yy1500;
		goto yy87;
yy1313:
		yych = *++cursor_;
		if (yych == 's') goto yy1501;
		if (yych == 'u') goto yy1503;
		goto yy87;
yy1314:
		yych = *++cursor_;
		if (yych == 's') goto yy1505;
		if (yych == 'u') goto yy1507;
		goto yy87;
yy1315:
		yych = *++cursor_;
		if (yych == 's') goto yy1509;
		if (yych == 'u') goto yy1511;
		goto yy87;
yy1316:
		yych = *++cursor_;
		if (yych == 's') goto yy1513;
		if (yych == 'u') goto yy1515;
		goto yy87;
yy1317:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 557 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32X4Mul); }
#line 7566 "src/prebuilt/wast-lexer-gen.cc"
yy1319:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 560 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32X4Neg); }
#line 7574 "src/prebuilt/wast-lexer-gen.cc"
yy1321:
		yych = *++cursor_;
		if (yych == 'l') goto yy1517;
		goto yy87;
yy1322:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 572 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32X4Shl); }
#line 7586 "src/prebuilt/wast-lexer-gen.cc"
yy1324:
		yych = *++cursor_;
		if (yych == '_') goto yy1518;
		goto yy87;
yy1325:
		yych = *++cursor_;
		if (yych == 'a') goto yy1519;
		goto yy87;
yy1326:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 553 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32X4Sub); }
#line 7602 "src/prebuilt/wast-lexer-gen.cc"
yy1328:
		yych = *++cursor_;
		if (yych == 'n') goto yy1520;
		goto yy87;
yy1329:
		yych = *++cursor_;
		if (yych == 'c') goto yy1521;
		goto yy87;
yy1330:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 298 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Const, I64Const); }
#line 7618 "src/prebuilt/wast-lexer-gen.cc"
yy1332:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 335 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64DivS); }
#line 7626 "src/prebuilt/wast-lexer-gen.cc"
yy1334:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 337 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64DivU); }
#line 7634 "src/prebuilt/wast-lexer-gen.cc"
yy1336:
		yych = *++cursor_;
		if (yych == 'd') goto yy1522;
		goto yy87;
yy1337:
		yych = *++cursor_;
		if (yych == '6') goto yy1523;
		goto yy87;
yy1338:
		yych = *++cursor_;
		if (yych == '2') goto yy1524;
		goto yy87;
yy1339:
		yych = *++cursor_;
		if (yych == '_') goto yy1525;
		goto yy87;
yy1340:
		yych = *++cursor_;
		if (yych == 't') goto yy1526;
		goto yy87;
yy1341:
		yych = *++cursor_;
		if (yych == 'e') goto yy1528;
		goto yy87;
yy1342:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 339 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64RemS); }
#line 7666 "src/prebuilt/wast-lexer-gen.cc"
yy1344:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 341 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64RemU); }
#line 7674 "src/prebuilt/wast-lexer-gen.cc"
yy1346:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 351 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64ShrS); }
#line 7682 "src/prebuilt/wast-lexer-gen.cc"
yy1348:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 353 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64ShrU); }
#line 7690 "src/prebuilt/wast-lexer-gen.cc"
yy1350:
		++cursor_;
		if ((yych = *cursor_) <= '1') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy1351;
				if (yych <= '0') goto yy86;
				goto yy1529;
			}
		} else {
			if (yych <= '8') {
				if (yych == '3') goto yy1530;
				if (yych <= '7') goto yy86;
				goto yy1531;
			} else {
				if (yych == ';') goto yy1351;
				if (yych <= '~') goto yy86;
			}
		}
yy1351:
#line 277 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I64Store); }
#line 7715 "src/prebuilt/wast-lexer-gen.cc"
yy1352:
		yych = *++cursor_;
		if (yych == '_') goto yy1533;
		goto yy87;
yy1353:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 550 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64X2Add); }
#line 7727 "src/prebuilt/wast-lexer-gen.cc"
yy1355:
		yych = *++cursor_;
		if (yych == '_') goto yy1534;
		goto yy87;
yy1356:
		yych = *++cursor_;
		if (yych == '_') goto yy1535;
		goto yy87;
yy1357:
		yych = *++cursor_;
		if (yych == 'r') goto yy1536;
		goto yy87;
yy1358:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 561 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64X2Neg); }
#line 7747 "src/prebuilt/wast-lexer-gen.cc"
yy1360:
		yych = *++cursor_;
		if (yych == 'l') goto yy1537;
		goto yy87;
yy1361:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 573 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64X2Shl); }
#line 7759 "src/prebuilt/wast-lexer-gen.cc"
yy1363:
		yych = *++cursor_;
		if (yych == '_') goto yy1538;
		goto yy87;
yy1364:
		yych = *++cursor_;
		if (yych == 'a') goto yy1539;
		goto yy87;
yy1365:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 554 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64X2Sub); }
#line 7775 "src/prebuilt/wast-lexer-gen.cc"
yy1367:
		yych = *++cursor_;
		if (yych == 'n') goto yy1540;
		goto yy87;
yy1368:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1369;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy1541;
				if (yych <= '~') goto yy86;
			}
		}
yy1369:
#line 547 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16Add); }
#line 7800 "src/prebuilt/wast-lexer-gen.cc"
yy1370:
		yych = *++cursor_;
		if (yych == '_') goto yy1542;
		goto yy87;
yy1371:
		yych = *++cursor_;
		if (yych == '_') goto yy1543;
		goto yy87;
yy1372:
		yych = *++cursor_;
		if (yych == 'r') goto yy1544;
		goto yy87;
yy1373:
		yych = *++cursor_;
		if (yych == 's') goto yy1545;
		if (yych == 'u') goto yy1547;
		goto yy87;
yy1374:
		yych = *++cursor_;
		if (yych == 's') goto yy1549;
		if (yych == 'u') goto yy1551;
		goto yy87;
yy1375:
		yych = *++cursor_;
		if (yych == 's') goto yy1553;
		if (yych == 'u') goto yy1555;
		goto yy87;
yy1376:
		yych = *++cursor_;
		if (yych == 's') goto yy1557;
		if (yych == 'u') goto yy1559;
		goto yy87;
yy1377:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 555 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16Mul); }
#line 7840 "src/prebuilt/wast-lexer-gen.cc"
yy1379:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 558 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I8X16Neg); }
#line 7848 "src/prebuilt/wast-lexer-gen.cc"
yy1381:
		yych = *++cursor_;
		if (yych == 'l') goto yy1561;
		goto yy87;
yy1382:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 570 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16Shl); }
#line 7860 "src/prebuilt/wast-lexer-gen.cc"
yy1384:
		yych = *++cursor_;
		if (yych == '_') goto yy1562;
		goto yy87;
yy1385:
		yych = *++cursor_;
		if (yych == 'a') goto yy1563;
		goto yy87;
yy1386:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1387;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy1564;
				if (yych <= '~') goto yy86;
			}
		}
yy1387:
#line 551 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16Sub); }
#line 7889 "src/prebuilt/wast-lexer-gen.cc"
yy1388:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 267 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(LocalGet); }
#line 7897 "src/prebuilt/wast-lexer-gen.cc"
yy1390:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 268 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(LocalSet); }
#line 7905 "src/prebuilt/wast-lexer-gen.cc"
yy1392:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 269 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(LocalTee); }
#line 7913 "src/prebuilt/wast-lexer-gen.cc"
yy1394:
		yych = *++cursor_;
		if (yych == 'p') goto yy1565;
		goto yy87;
yy1395:
		yych = *++cursor_;
		if (yych == 'l') goto yy1566;
		goto yy87;
yy1396:
		yych = *++cursor_;
		if (yych == 'o') goto yy1567;
		goto yy87;
yy1397:
		yych = *++cursor_;
		if (yych == 'i') goto yy1568;
		goto yy87;
yy1398:
		yych = *++cursor_;
		if (yych == 'z') goto yy1569;
		goto yy87;
yy1399:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ')') {
			if (yych <= '!') {
				if (yych <= ' ') goto yy88;
				goto yy86;
			} else {
				if (yych <= '"') goto yy88;
				if (yych <= '\'') goto yy86;
				goto yy88;
			}
		} else {
			if (yych <= ':') {
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy1143;
				goto yy86;
			} else {
				if (yych <= ';') goto yy88;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy1400:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= '9') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
				goto yy88;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy88;
				if (yych <= '/') goto yy86;
				goto yy1570;
			}
		} else {
			if (yych <= 'F') {
				if (yych == ';') goto yy88;
				if (yych <= '@') goto yy86;
				goto yy1570;
			} else {
				if (yych <= '`') goto yy86;
				if (yych <= 'f') goto yy1570;
				if (yych <= '~') goto yy86;
				goto yy88;
			}
		}
yy1401:
		yych = *++cursor_;
		if (yych == 'l') goto yy1572;
		goto yy87;
yy1402:
		yych = *++cursor_;
		if (yych == 'l') goto yy1573;
		goto yy87;
yy1403:
		yych = *++cursor_;
		if (yych == 'l') goto yy1574;
		goto yy87;
yy1404:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 669 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(LocalSet); }
#line 8003 "src/prebuilt/wast-lexer-gen.cc"
yy1406:
		yych = *++cursor_;
		if (yych == 'y') goto yy1576;
		goto yy87;
yy1407:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 450 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(TableGet); }
#line 8015 "src/prebuilt/wast-lexer-gen.cc"
yy1409:
		yych = *++cursor_;
		if (yych == 'w') goto yy1578;
		goto yy87;
yy1410:
		yych = *++cursor_;
		if (yych == 't') goto yy1580;
		goto yy87;
yy1411:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 451 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(TableSet); }
#line 8031 "src/prebuilt/wast-lexer-gen.cc"
yy1413:
		yych = *++cursor_;
		if (yych == 'e') goto yy1582;
		goto yy87;
yy1414:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 670 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(LocalTee); }
#line 8043 "src/prebuilt/wast-lexer-gen.cc"
yy1416:
		yych = *++cursor_;
		if (yych == 'l') goto yy1584;
		goto yy87;
yy1417:
		yych = *++cursor_;
		if (yych == 'e') goto yy1585;
		goto yy87;
yy1418:
		yych = *++cursor_;
		if (yych == 't') goto yy1586;
		goto yy87;
yy1419:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 524 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load,  V128Load); }
#line 8063 "src/prebuilt/wast-lexer-gen.cc"
yy1421:
		yych = *++cursor_;
		if (yych == 'e') goto yy1588;
		goto yy87;
yy1422:
		yych = *++cursor_;
		if (yych == 'f') goto yy1590;
		goto yy87;
yy1423:
		yych = *++cursor_;
		if (yych == 'a') goto yy1591;
		goto yy87;
yy1424:
		yych = *++cursor_;
		if (yych == 'a') goto yy1592;
		goto yy87;
yy1425:
		yych = *++cursor_;
		if (yych == 'f') goto yy1593;
		goto yy87;
yy1426:
		yych = *++cursor_;
		if (yych == 'u') goto yy1594;
		goto yy87;
yy1427:
		yych = *++cursor_;
		if (yych == 'p') goto yy1595;
		goto yy87;
yy1428:
		yych = *++cursor_;
		if (yych == 'i') goto yy1597;
		goto yy87;
yy1429:
		yych = *++cursor_;
		if (yych == 'i') goto yy1598;
		goto yy87;
yy1430:
		yych = *++cursor_;
		if (yych == 'e') goto yy1599;
		goto yy87;
yy1431:
		yych = *++cursor_;
		if (yych == 'm') goto yy1600;
		goto yy87;
yy1432:
		yych = *++cursor_;
		if (yych == 't') goto yy1601;
		goto yy87;
yy1433:
		yych = *++cursor_;
		if (yych == 'g') goto yy1602;
		goto yy87;
yy1434:
		yych = *++cursor_;
		if (yych == '/') goto yy1603;
		if (yych == '_') goto yy1604;
		goto yy87;
yy1435:
		yych = *++cursor_;
		if (yych == 't') goto yy1605;
		goto yy87;
yy1436:
		yych = *++cursor_;
		if (yych == 'r') goto yy1607;
		goto yy87;
yy1437:
		yych = *++cursor_;
		if (yych == 'e') goto yy1608;
		goto yy87;
yy1438:
		yych = *++cursor_;
		if (yych == 'a') goto yy1609;
		goto yy87;
yy1439:
		yych = *++cursor_;
		if (yych == 'a') goto yy1610;
		goto yy87;
yy1440:
		yych = *++cursor_;
		if (yych == 't') goto yy1611;
		goto yy87;
yy1441:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 653 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32X4Sqrt); }
#line 8152 "src/prebuilt/wast-lexer-gen.cc"
yy1443:
		yych = *++cursor_;
		if (yych == 't') goto yy1613;
		goto yy87;
yy1444:
		yych = *++cursor_;
		if (yych == 'g') goto yy1614;
		goto yy87;
yy1445:
		yych = *++cursor_;
		if (yych == 't') goto yy1615;
		goto yy87;
yy1446:
		yych = *++cursor_;
		if (yych == 'e') goto yy1617;
		goto yy87;
yy1447:
		yych = *++cursor_;
		if (yych == 'r') goto yy1618;
		goto yy87;
yy1448:
		yych = *++cursor_;
		if (yych == 'e') goto yy1619;
		goto yy87;
yy1449:
		yych = *++cursor_;
		if (yych == 'a') goto yy1620;
		goto yy87;
yy1450:
		yych = *++cursor_;
		if (yych == 'a') goto yy1621;
		goto yy87;
yy1451:
		yych = *++cursor_;
		if (yych == 't') goto yy1622;
		goto yy87;
yy1452:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 654 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64X2Sqrt); }
#line 8196 "src/prebuilt/wast-lexer-gen.cc"
yy1454:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 671 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(GlobalGet); }
#line 8204 "src/prebuilt/wast-lexer-gen.cc"
yy1456:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 270 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(GlobalGet); }
#line 8212 "src/prebuilt/wast-lexer-gen.cc"
yy1458:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 271 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(GlobalSet); }
#line 8220 "src/prebuilt/wast-lexer-gen.cc"
yy1460:
		yych = *++cursor_;
		if (yych == 'y') goto yy1624;
		goto yy87;
yy1461:
		yych = *++cursor_;
		if (yych == 's') goto yy1626;
		goto yy87;
yy1462:
		yych = *++cursor_;
		if (yych == 't') goto yy1627;
		goto yy87;
yy1463:
		yych = *++cursor_;
		if (yych == 't') goto yy1628;
		goto yy87;
yy1464:
		yych = *++cursor_;
		if (yych == 'a') goto yy1629;
		goto yy87;
yy1465:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 631 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8GeS); }
#line 8248 "src/prebuilt/wast-lexer-gen.cc"
yy1467:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 632 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8GeU); }
#line 8256 "src/prebuilt/wast-lexer-gen.cc"
yy1469:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 623 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8GtS); }
#line 8264 "src/prebuilt/wast-lexer-gen.cc"
yy1471:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 624 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8GtU); }
#line 8272 "src/prebuilt/wast-lexer-gen.cc"
yy1473:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 615 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8LeS); }
#line 8280 "src/prebuilt/wast-lexer-gen.cc"
yy1475:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 616 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8LeU); }
#line 8288 "src/prebuilt/wast-lexer-gen.cc"
yy1477:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 607 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8LtS); }
#line 8296 "src/prebuilt/wast-lexer-gen.cc"
yy1479:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 608 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I16X8LtU); }
#line 8304 "src/prebuilt/wast-lexer-gen.cc"
yy1481:
		yych = *++cursor_;
		if (yych == 'a') goto yy1630;
		goto yy87;
yy1482:
		yych = *++cursor_;
		if (yych == 's') goto yy1631;
		if (yych == 'u') goto yy1633;
		goto yy87;
yy1483:
		yych = *++cursor_;
		if (yych == 't') goto yy1635;
		goto yy87;
yy1484:
		yych = *++cursor_;
		if (yych == 's') goto yy1637;
		goto yy87;
yy1485:
		yych = *++cursor_;
		if (yych == '.') goto yy1638;
		goto yy87;
yy1486:
		yych = *++cursor_;
		if (yych == '1') goto yy1639;
		if (yych == '8') goto yy1640;
		goto yy87;
yy1487:
		yych = *++cursor_;
		if (yych == '_') goto yy1641;
		goto yy87;
yy1488:
		yych = *++cursor_;
		if (yych == 's') goto yy1642;
		if (yych == 'u') goto yy1644;
		goto yy87;
yy1489:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 307 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32Popcnt); }
#line 8347 "src/prebuilt/wast-lexer-gen.cc"
yy1491:
		yych = *++cursor_;
		if (yych == 'r') goto yy1646;
		goto yy87;
yy1492:
		yych = *++cursor_;
		if (yych == '6') goto yy1647;
		goto yy87;
yy1493:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 290 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I32Store8); }
#line 8363 "src/prebuilt/wast-lexer-gen.cc"
yy1495:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'f') goto yy1649;
			goto yy87;
		} else {
			if (yych <= 's') goto yy1650;
			if (yych == 'u') goto yy1651;
			goto yy87;
		}
yy1496:
		yych = *++cursor_;
		if (yych == '6') goto yy1652;
		goto yy87;
yy1497:
		yych = *++cursor_;
		if (yych == '6') goto yy1653;
		goto yy87;
yy1498:
		yych = *++cursor_;
		if (yych == 't') goto yy1654;
		goto yy87;
yy1499:
		yych = *++cursor_;
		if (yych == 't') goto yy1655;
		goto yy87;
yy1500:
		yych = *++cursor_;
		if (yych == 'a') goto yy1656;
		goto yy87;
yy1501:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 633 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4GeS); }
#line 8401 "src/prebuilt/wast-lexer-gen.cc"
yy1503:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 634 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4GeU); }
#line 8409 "src/prebuilt/wast-lexer-gen.cc"
yy1505:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 625 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4GtS); }
#line 8417 "src/prebuilt/wast-lexer-gen.cc"
yy1507:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 626 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4GtU); }
#line 8425 "src/prebuilt/wast-lexer-gen.cc"
yy1509:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 617 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4LeS); }
#line 8433 "src/prebuilt/wast-lexer-gen.cc"
yy1511:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 618 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4LeU); }
#line 8441 "src/prebuilt/wast-lexer-gen.cc"
yy1513:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 609 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4LtS); }
#line 8449 "src/prebuilt/wast-lexer-gen.cc"
yy1515:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 610 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I32X4LtU); }
#line 8457 "src/prebuilt/wast-lexer-gen.cc"
yy1517:
		yych = *++cursor_;
		if (yych == 'a') goto yy1657;
		goto yy87;
yy1518:
		yych = *++cursor_;
		if (yych == 's') goto yy1658;
		if (yych == 'u') goto yy1660;
		goto yy87;
yy1519:
		yych = *++cursor_;
		if (yych == 't') goto yy1662;
		goto yy87;
yy1520:
		yych = *++cursor_;
		if (yych == 'c') goto yy1664;
		goto yy87;
yy1521:
		yych = *++cursor_;
		if (yych == '.') goto yy1665;
		goto yy87;
yy1522:
		yych = *++cursor_;
		if (yych <= '3') {
			if (yych == '1') goto yy1666;
			if (yych <= '2') goto yy87;
			goto yy1667;
		} else {
			if (yych <= '8') {
				if (yych <= '7') goto yy87;
				goto yy1668;
			} else {
				if (yych == '_') goto yy1669;
				goto yy87;
			}
		}
yy1523:
		yych = *++cursor_;
		if (yych == '_') goto yy1670;
		goto yy87;
yy1524:
		yych = *++cursor_;
		if (yych == '_') goto yy1671;
		goto yy87;
yy1525:
		yych = *++cursor_;
		if (yych == 's') goto yy1672;
		if (yych == 'u') goto yy1674;
		goto yy87;
yy1526:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 308 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64Popcnt); }
#line 8514 "src/prebuilt/wast-lexer-gen.cc"
yy1528:
		yych = *++cursor_;
		if (yych == 'r') goto yy1676;
		goto yy87;
yy1529:
		yych = *++cursor_;
		if (yych == '6') goto yy1677;
		goto yy87;
yy1530:
		yych = *++cursor_;
		if (yych == '2') goto yy1679;
		goto yy87;
yy1531:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 291 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I64Store8); }
#line 8534 "src/prebuilt/wast-lexer-gen.cc"
yy1533:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'f') goto yy1681;
			goto yy87;
		} else {
			if (yych <= 's') goto yy1682;
			if (yych == 'u') goto yy1683;
			goto yy87;
		}
yy1534:
		yych = *++cursor_;
		if (yych == 't') goto yy1684;
		goto yy87;
yy1535:
		yych = *++cursor_;
		if (yych == 't') goto yy1685;
		goto yy87;
yy1536:
		yych = *++cursor_;
		if (yych == 'a') goto yy1686;
		goto yy87;
yy1537:
		yych = *++cursor_;
		if (yych == 'a') goto yy1687;
		goto yy87;
yy1538:
		yych = *++cursor_;
		if (yych == 's') goto yy1688;
		if (yych == 'u') goto yy1690;
		goto yy87;
yy1539:
		yych = *++cursor_;
		if (yych == 't') goto yy1692;
		goto yy87;
yy1540:
		yych = *++cursor_;
		if (yych == 'c') goto yy1694;
		goto yy87;
yy1541:
		yych = *++cursor_;
		if (yych == 's') goto yy1695;
		goto yy87;
yy1542:
		yych = *++cursor_;
		if (yych == 't') goto yy1696;
		goto yy87;
yy1543:
		yych = *++cursor_;
		if (yych == 't') goto yy1697;
		goto yy87;
yy1544:
		yych = *++cursor_;
		if (yych == 'a') goto yy1698;
		goto yy87;
yy1545:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 629 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16GeS); }
#line 8597 "src/prebuilt/wast-lexer-gen.cc"
yy1547:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 630 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16GeU); }
#line 8605 "src/prebuilt/wast-lexer-gen.cc"
yy1549:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 621 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16GtS); }
#line 8613 "src/prebuilt/wast-lexer-gen.cc"
yy1551:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 622 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16GtU); }
#line 8621 "src/prebuilt/wast-lexer-gen.cc"
yy1553:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 613 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16LeS); }
#line 8629 "src/prebuilt/wast-lexer-gen.cc"
yy1555:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 614 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16LeU); }
#line 8637 "src/prebuilt/wast-lexer-gen.cc"
yy1557:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 605 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16LtS); }
#line 8645 "src/prebuilt/wast-lexer-gen.cc"
yy1559:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 606 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Compare, I8X16LtU); }
#line 8653 "src/prebuilt/wast-lexer-gen.cc"
yy1561:
		yych = *++cursor_;
		if (yych == 'a') goto yy1699;
		goto yy87;
yy1562:
		yych = *++cursor_;
		if (yych == 's') goto yy1700;
		if (yych == 'u') goto yy1702;
		goto yy87;
yy1563:
		yych = *++cursor_;
		if (yych == 't') goto yy1704;
		goto yy87;
yy1564:
		yych = *++cursor_;
		if (yych == 's') goto yy1706;
		goto yy87;
yy1565:
		yych = *++cursor_;
		if (yych == 'y') goto yy1707;
		goto yy87;
yy1566:
		yych = *++cursor_;
		if (yych == 'l') goto yy1709;
		goto yy87;
yy1567:
		yych = *++cursor_;
		if (yych == 'w') goto yy1711;
		goto yy87;
yy1568:
		yych = *++cursor_;
		if (yych == 't') goto yy1713;
		goto yy87;
yy1569:
		yych = *++cursor_;
		if (yych == 'e') goto yy1715;
		goto yy87;
yy1570:
		++cursor_;
		if (limit_ <= cursor_) FILL(1);
		yych = *cursor_;
		if (yych <= ':') {
			if (yych <= '\'') {
				if (yych == '!') goto yy86;
				if (yych <= '"') goto yy1142;
				goto yy86;
			} else {
				if (yych <= ')') goto yy1142;
				if (yych <= '/') goto yy86;
				if (yych <= '9') goto yy1570;
				goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych <= ';') goto yy1142;
				if (yych <= '@') goto yy86;
				if (yych <= 'F') goto yy1570;
				goto yy86;
			} else {
				if (yych <= '`') {
					if (yych <= '_') goto yy1400;
					goto yy86;
				} else {
					if (yych <= 'f') goto yy1570;
					if (yych <= '~') goto yy86;
					goto yy1142;
				}
			}
		}
yy1572:
		yych = *++cursor_;
		if (yych == 'l') goto yy1717;
		goto yy87;
yy1573:
		yych = *++cursor_;
		if (yych == 'l') goto yy1719;
		goto yy87;
yy1574:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 672 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(GlobalSet); }
#line 8738 "src/prebuilt/wast-lexer-gen.cc"
yy1576:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 449 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(TableCopy); }
#line 8746 "src/prebuilt/wast-lexer-gen.cc"
yy1578:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 452 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(TableGrow); }
#line 8754 "src/prebuilt/wast-lexer-gen.cc"
yy1580:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 447 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(TableInit); }
#line 8762 "src/prebuilt/wast-lexer-gen.cc"
yy1582:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 453 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(TableSize); }
#line 8770 "src/prebuilt/wast-lexer-gen.cc"
yy1584:
		yych = *++cursor_;
		if (yych == 'e') goto yy1721;
		goto yy87;
yy1585:
		yych = *++cursor_;
		if (yych == 'l') goto yy1723;
		goto yy87;
yy1586:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 523 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Const, V128Const); }
#line 8786 "src/prebuilt/wast-lexer-gen.cc"
yy1588:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 525 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, V128Store); }
#line 8794 "src/prebuilt/wast-lexer-gen.cc"
yy1590:
		yych = *++cursor_;
		if (yych == 'f') goto yy1724;
		goto yy87;
yy1591:
		yych = *++cursor_;
		if (yych == 'u') goto yy1725;
		goto yy87;
yy1592:
		yych = *++cursor_;
		if (yych == 'l') goto yy1726;
		goto yy87;
yy1593:
		yych = *++cursor_;
		if (yych == 'o') goto yy1727;
		goto yy87;
yy1594:
		yych = *++cursor_;
		if (yych == 'r') goto yy1728;
		goto yy87;
yy1595:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 735 "src/wast-lexer.cc"
		{ RETURN(AssertTrap); }
#line 8822 "src/prebuilt/wast-lexer-gen.cc"
yy1597:
		yych = *++cursor_;
		if (yych == 'n') goto yy1729;
		goto yy87;
yy1598:
		yych = *++cursor_;
		if (yych == 'f') goto yy1730;
		goto yy87;
yy1599:
		yych = *++cursor_;
		if (yych == 'c') goto yy1731;
		goto yy87;
yy1600:
		yych = *++cursor_;
		if (yych == 'o') goto yy1732;
		goto yy87;
yy1601:
		yych = *++cursor_;
		if (yych == '_') goto yy1733;
		goto yy87;
yy1602:
		yych = *++cursor_;
		if (yych == 'n') goto yy1734;
		goto yy87;
yy1603:
		yych = *++cursor_;
		if (yych == 'f') goto yy1736;
		goto yy87;
yy1604:
		yych = *++cursor_;
		if (yych == 'f') goto yy1737;
		goto yy87;
yy1605:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 321 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32Nearest); }
#line 8862 "src/prebuilt/wast-lexer-gen.cc"
yy1607:
		yych = *++cursor_;
		if (yych == 'p') goto yy1738;
		goto yy87;
yy1608:
		yych = *++cursor_;
		if (yych == 'r') goto yy1739;
		goto yy87;
yy1609:
		yych = *++cursor_;
		if (yych == 'c') goto yy1740;
		goto yy87;
yy1610:
		yych = *++cursor_;
		if (yych == 'c') goto yy1741;
		goto yy87;
yy1611:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 530 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32X4Splat); }
#line 8886 "src/prebuilt/wast-lexer-gen.cc"
yy1613:
		yych = *++cursor_;
		if (yych == '_') goto yy1742;
		goto yy87;
yy1614:
		yych = *++cursor_;
		if (yych == 'n') goto yy1743;
		goto yy87;
yy1615:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 322 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64Nearest); }
#line 8902 "src/prebuilt/wast-lexer-gen.cc"
yy1617:
		yych = *++cursor_;
		if (yych == '/') goto yy1745;
		if (yych == '_') goto yy1746;
		goto yy87;
yy1618:
		yych = *++cursor_;
		if (yych == 'p') goto yy1747;
		goto yy87;
yy1619:
		yych = *++cursor_;
		if (yych == 'r') goto yy1748;
		goto yy87;
yy1620:
		yych = *++cursor_;
		if (yych == 'c') goto yy1749;
		goto yy87;
yy1621:
		yych = *++cursor_;
		if (yych == 'c') goto yy1750;
		goto yy87;
yy1622:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 531 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64X2Splat); }
#line 8931 "src/prebuilt/wast-lexer-gen.cc"
yy1624:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 446 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemoryGrow); }
#line 8939 "src/prebuilt/wast-lexer-gen.cc"
yy1626:
		yych = *++cursor_;
		if (yych == 'a') goto yy1751;
		goto yy87;
yy1627:
		yych = *++cursor_;
		if (yych == 'r') goto yy1752;
		goto yy87;
yy1628:
		yych = *++cursor_;
		if (yych == 'r') goto yy1753;
		goto yy87;
yy1629:
		yych = *++cursor_;
		if (yych == 'c') goto yy1754;
		goto yy87;
yy1630:
		yych = *++cursor_;
		if (yych == 'c') goto yy1755;
		goto yy87;
yy1631:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 576 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8ShrS); }
#line 8967 "src/prebuilt/wast-lexer-gen.cc"
yy1633:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 577 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8ShrU); }
#line 8975 "src/prebuilt/wast-lexer-gen.cc"
yy1635:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 527 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I16X8Splat); }
#line 8983 "src/prebuilt/wast-lexer-gen.cc"
yy1637:
		yych = *++cursor_;
		if (yych == 'a') goto yy1756;
		goto yy87;
yy1638:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'l') goto yy1757;
			if (yych <= 'q') goto yy87;
			goto yy1758;
		} else {
			if (yych <= 's') goto yy1759;
			if (yych == 'w') goto yy1760;
			goto yy87;
		}
yy1639:
		yych = *++cursor_;
		if (yych == '6') goto yy1761;
		goto yy87;
yy1640:
		yych = *++cursor_;
		if (yych == '_') goto yy1762;
		goto yy87;
yy1641:
		yych = *++cursor_;
		if (yych == 's') goto yy1763;
		if (yych == 'u') goto yy1765;
		goto yy87;
yy1642:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 280 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I32Load8S); }
#line 9019 "src/prebuilt/wast-lexer-gen.cc"
yy1644:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 282 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I32Load8U); }
#line 9027 "src/prebuilt/wast-lexer-gen.cc"
yy1646:
		yych = *++cursor_;
		if (yych == 'p') goto yy1767;
		goto yy87;
yy1647:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 292 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I32Store16); }
#line 9039 "src/prebuilt/wast-lexer-gen.cc"
yy1649:
		yych = *++cursor_;
		if (yych == '3') goto yy1768;
		if (yych == '6') goto yy1769;
		goto yy87;
yy1650:
		yych = *++cursor_;
		if (yych <= '9') {
			if (yych == '/') goto yy1770;
			goto yy87;
		} else {
			if (yych <= ':') goto yy1771;
			if (yych == 'a') goto yy1772;
			goto yy87;
		}
yy1651:
		yych = *++cursor_;
		if (yych == '/') goto yy1773;
		if (yych == ':') goto yy1774;
		goto yy87;
yy1652:
		yych = *++cursor_;
		if (yych == '4') goto yy1775;
		goto yy87;
yy1653:
		yych = *++cursor_;
		if (yych == '4') goto yy1777;
		goto yy87;
yy1654:
		yych = *++cursor_;
		if (yych == 'r') goto yy1779;
		goto yy87;
yy1655:
		yych = *++cursor_;
		if (yych == 'r') goto yy1780;
		goto yy87;
yy1656:
		yych = *++cursor_;
		if (yych == 'c') goto yy1781;
		goto yy87;
yy1657:
		yych = *++cursor_;
		if (yych == 'c') goto yy1782;
		goto yy87;
yy1658:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 578 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32X4ShrS); }
#line 9091 "src/prebuilt/wast-lexer-gen.cc"
yy1660:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 579 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I32X4ShrU); }
#line 9099 "src/prebuilt/wast-lexer-gen.cc"
yy1662:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 528 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32X4Splat); }
#line 9107 "src/prebuilt/wast-lexer-gen.cc"
yy1664:
		yych = *++cursor_;
		if (yych == '_') goto yy1783;
		goto yy87;
yy1665:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'l') goto yy1784;
			if (yych <= 'q') goto yy87;
			goto yy1785;
		} else {
			if (yych <= 's') goto yy1786;
			if (yych == 'w') goto yy1787;
			goto yy87;
		}
yy1666:
		yych = *++cursor_;
		if (yych == '6') goto yy1788;
		goto yy87;
yy1667:
		yych = *++cursor_;
		if (yych == '2') goto yy1789;
		goto yy87;
yy1668:
		yych = *++cursor_;
		if (yych == '_') goto yy1790;
		goto yy87;
yy1669:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'i') goto yy1791;
			goto yy87;
		} else {
			if (yych <= 's') goto yy1792;
			if (yych == 'u') goto yy1793;
			goto yy87;
		}
yy1670:
		yych = *++cursor_;
		if (yych == 's') goto yy1794;
		if (yych == 'u') goto yy1796;
		goto yy87;
yy1671:
		yych = *++cursor_;
		if (yych == 's') goto yy1798;
		if (yych == 'u') goto yy1800;
		goto yy87;
yy1672:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 281 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load8S); }
#line 9162 "src/prebuilt/wast-lexer-gen.cc"
yy1674:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 283 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load8U); }
#line 9170 "src/prebuilt/wast-lexer-gen.cc"
yy1676:
		yych = *++cursor_;
		if (yych == 'p') goto yy1802;
		goto yy87;
yy1677:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 293 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I64Store16); }
#line 9182 "src/prebuilt/wast-lexer-gen.cc"
yy1679:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 294 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Store, I64Store32); }
#line 9190 "src/prebuilt/wast-lexer-gen.cc"
yy1681:
		yych = *++cursor_;
		if (yych == '3') goto yy1803;
		if (yych == '6') goto yy1804;
		goto yy87;
yy1682:
		yych = *++cursor_;
		if (yych <= '9') {
			if (yych == '/') goto yy1805;
			goto yy87;
		} else {
			if (yych <= ':') goto yy1806;
			if (yych == 'a') goto yy1807;
			goto yy87;
		}
yy1683:
		yych = *++cursor_;
		if (yych == '/') goto yy1808;
		if (yych == ':') goto yy1809;
		goto yy87;
yy1684:
		yych = *++cursor_;
		if (yych == 'r') goto yy1810;
		goto yy87;
yy1685:
		yych = *++cursor_;
		if (yych == 'r') goto yy1811;
		goto yy87;
yy1686:
		yych = *++cursor_;
		if (yych == 'c') goto yy1812;
		goto yy87;
yy1687:
		yych = *++cursor_;
		if (yych == 'c') goto yy1813;
		goto yy87;
yy1688:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 580 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64X2ShrS); }
#line 9234 "src/prebuilt/wast-lexer-gen.cc"
yy1690:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 581 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I64X2ShrU); }
#line 9242 "src/prebuilt/wast-lexer-gen.cc"
yy1692:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 529 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64X2Splat); }
#line 9250 "src/prebuilt/wast-lexer-gen.cc"
yy1694:
		yych = *++cursor_;
		if (yych == '_') goto yy1814;
		goto yy87;
yy1695:
		yych = *++cursor_;
		if (yych == 'a') goto yy1815;
		goto yy87;
yy1696:
		yych = *++cursor_;
		if (yych == 'r') goto yy1816;
		goto yy87;
yy1697:
		yych = *++cursor_;
		if (yych == 'r') goto yy1817;
		goto yy87;
yy1698:
		yych = *++cursor_;
		if (yych == 'c') goto yy1818;
		goto yy87;
yy1699:
		yych = *++cursor_;
		if (yych == 'c') goto yy1819;
		goto yy87;
yy1700:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 574 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16ShrS); }
#line 9282 "src/prebuilt/wast-lexer-gen.cc"
yy1702:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 575 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16ShrU); }
#line 9290 "src/prebuilt/wast-lexer-gen.cc"
yy1704:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 526 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I8X16Splat); }
#line 9298 "src/prebuilt/wast-lexer-gen.cc"
yy1706:
		yych = *++cursor_;
		if (yych == 'a') goto yy1820;
		goto yy87;
yy1707:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 443 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemoryCopy); }
#line 9310 "src/prebuilt/wast-lexer-gen.cc"
yy1709:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 444 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemoryFill); }
#line 9318 "src/prebuilt/wast-lexer-gen.cc"
yy1711:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 440 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemoryGrow); }
#line 9326 "src/prebuilt/wast-lexer-gen.cc"
yy1713:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 441 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemoryInit); }
#line 9334 "src/prebuilt/wast-lexer-gen.cc"
yy1715:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 439 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemorySize); }
#line 9342 "src/prebuilt/wast-lexer-gen.cc"
yy1717:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 455 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(RefIsNull); }
#line 9350 "src/prebuilt/wast-lexer-gen.cc"
yy1719:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1720;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy1821;
				if (yych <= '~') goto yy86;
			}
		}
yy1720:
#line 663 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(ReturnCall); }
#line 9371 "src/prebuilt/wast-lexer-gen.cc"
yy1721:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 438 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(Unreachable); }
#line 9379 "src/prebuilt/wast-lexer-gen.cc"
yy1723:
		yych = *++cursor_;
		if (yych == 'e') goto yy1822;
		goto yy87;
yy1724:
		yych = *++cursor_;
		if (yych == 'l') goto yy1823;
		goto yy87;
yy1725:
		yych = *++cursor_;
		if (yych == 's') goto yy1824;
		goto yy87;
yy1726:
		yych = *++cursor_;
		if (yych == 'i') goto yy1825;
		goto yy87;
yy1727:
		yych = *++cursor_;
		if (yych == 'r') goto yy1826;
		goto yy87;
yy1728:
		yych = *++cursor_;
		if (yych == 'n') goto yy1827;
		goto yy87;
yy1729:
		yych = *++cursor_;
		if (yych == 'k') goto yy1829;
		goto yy87;
yy1730:
		yych = *++cursor_;
		if (yych == 'y') goto yy1830;
		goto yy87;
yy1731:
		yych = *++cursor_;
		if (yych == 't') goto yy1832;
		goto yy87;
yy1732:
		yych = *++cursor_;
		if (yych == 'r') goto yy1834;
		goto yy87;
yy1733:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'i') goto yy1835;
			goto yy87;
		} else {
			if (yych <= 's') goto yy1836;
			if (yych == 'u') goto yy1837;
			goto yy87;
		}
yy1734:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 370 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F32Copysign); }
#line 9437 "src/prebuilt/wast-lexer-gen.cc"
yy1736:
		yych = *++cursor_;
		if (yych == '6') goto yy1838;
		goto yy87;
yy1737:
		yych = *++cursor_;
		if (yych == '6') goto yy1839;
		goto yy87;
yy1738:
		yych = *++cursor_;
		if (yych == 'r') goto yy1840;
		goto yy87;
yy1739:
		yych = *++cursor_;
		if (yych == 't') goto yy1841;
		goto yy87;
yy1740:
		yych = *++cursor_;
		if (yych == 't') goto yy1842;
		goto yy87;
yy1741:
		yych = *++cursor_;
		if (yych == 'e') goto yy1843;
		goto yy87;
yy1742:
		yych = *++cursor_;
		if (yych <= 'r') {
			if (yych == 'i') goto yy1844;
			goto yy87;
		} else {
			if (yych <= 's') goto yy1845;
			if (yych == 'u') goto yy1846;
			goto yy87;
		}
yy1743:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 371 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, F64Copysign); }
#line 9479 "src/prebuilt/wast-lexer-gen.cc"
yy1745:
		yych = *++cursor_;
		if (yych == 'f') goto yy1847;
		goto yy87;
yy1746:
		yych = *++cursor_;
		if (yych == 'f') goto yy1848;
		goto yy87;
yy1747:
		yych = *++cursor_;
		if (yych == 'r') goto yy1849;
		goto yy87;
yy1748:
		yych = *++cursor_;
		if (yych == 't') goto yy1850;
		goto yy87;
yy1749:
		yych = *++cursor_;
		if (yych == 't') goto yy1851;
		goto yy87;
yy1750:
		yych = *++cursor_;
		if (yych == 'e') goto yy1852;
		goto yy87;
yy1751:
		yych = *++cursor_;
		if (yych == 't') goto yy1853;
		goto yy87;
yy1752:
		yych = *++cursor_;
		if (yych == 'u') goto yy1854;
		goto yy87;
yy1753:
		yych = *++cursor_;
		if (yych == 'u') goto yy1855;
		goto yy87;
yy1754:
		yych = *++cursor_;
		if (yych == 't') goto yy1856;
		goto yy87;
yy1755:
		yych = *++cursor_;
		if (yych == 'e') goto yy1857;
		goto yy87;
yy1756:
		yych = *++cursor_;
		if (yych == 't') goto yy1858;
		goto yy87;
yy1757:
		yych = *++cursor_;
		if (yych == 'o') goto yy1859;
		goto yy87;
yy1758:
		yych = *++cursor_;
		if (yych == 'm') goto yy1860;
		goto yy87;
yy1759:
		yych = *++cursor_;
		if (yych == 't') goto yy1861;
		goto yy87;
yy1760:
		yych = *++cursor_;
		if (yych == 'a') goto yy1862;
		goto yy87;
yy1761:
		yych = *++cursor_;
		if (yych == '_') goto yy1863;
		goto yy87;
yy1762:
		yych = *++cursor_;
		if (yych == 's') goto yy1864;
		goto yy87;
yy1763:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 284 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I32Load16S); }
#line 9559 "src/prebuilt/wast-lexer-gen.cc"
yy1765:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 286 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I32Load16U); }
#line 9567 "src/prebuilt/wast-lexer-gen.cc"
yy1767:
		yych = *++cursor_;
		if (yych == 'r') goto yy1866;
		goto yy87;
yy1768:
		yych = *++cursor_;
		if (yych == '2') goto yy1867;
		goto yy87;
yy1769:
		yych = *++cursor_;
		if (yych == '4') goto yy1868;
		goto yy87;
yy1770:
		yych = *++cursor_;
		if (yych == 'f') goto yy1869;
		goto yy87;
yy1771:
		yych = *++cursor_;
		if (yych == 's') goto yy1870;
		goto yy87;
yy1772:
		yych = *++cursor_;
		if (yych == 't') goto yy1871;
		goto yy87;
yy1773:
		yych = *++cursor_;
		if (yych == 'f') goto yy1872;
		goto yy87;
yy1774:
		yych = *++cursor_;
		if (yych == 's') goto yy1873;
		goto yy87;
yy1775:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 675 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32WrapI64); }
#line 9607 "src/prebuilt/wast-lexer-gen.cc"
yy1777:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 406 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32WrapI64); }
#line 9615 "src/prebuilt/wast-lexer-gen.cc"
yy1779:
		yych = *++cursor_;
		if (yych == 'u') goto yy1874;
		goto yy87;
yy1780:
		yych = *++cursor_;
		if (yych == 'u') goto yy1875;
		goto yy87;
yy1781:
		yych = *++cursor_;
		if (yych == 't') goto yy1876;
		goto yy87;
yy1782:
		yych = *++cursor_;
		if (yych == 'e') goto yy1877;
		goto yy87;
yy1783:
		yych = *++cursor_;
		if (yych == 's') goto yy1878;
		goto yy87;
yy1784:
		yych = *++cursor_;
		if (yych == 'o') goto yy1879;
		goto yy87;
yy1785:
		yych = *++cursor_;
		if (yych == 'm') goto yy1880;
		goto yy87;
yy1786:
		yych = *++cursor_;
		if (yych == 't') goto yy1881;
		goto yy87;
yy1787:
		yych = *++cursor_;
		if (yych == 'a') goto yy1882;
		goto yy87;
yy1788:
		yych = *++cursor_;
		if (yych == '_') goto yy1883;
		goto yy87;
yy1789:
		yych = *++cursor_;
		if (yych == '_') goto yy1884;
		goto yy87;
yy1790:
		yych = *++cursor_;
		if (yych == 's') goto yy1885;
		goto yy87;
yy1791:
		yych = *++cursor_;
		if (yych == '3') goto yy1887;
		goto yy87;
yy1792:
		yych = *++cursor_;
		if (yych == '/') goto yy1888;
		goto yy87;
yy1793:
		yych = *++cursor_;
		if (yych == '/') goto yy1889;
		goto yy87;
yy1794:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 285 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load16S); }
#line 9683 "src/prebuilt/wast-lexer-gen.cc"
yy1796:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 287 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load16U); }
#line 9691 "src/prebuilt/wast-lexer-gen.cc"
yy1798:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 288 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load32S); }
#line 9699 "src/prebuilt/wast-lexer-gen.cc"
yy1800:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 289 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Load, I64Load32U); }
#line 9707 "src/prebuilt/wast-lexer-gen.cc"
yy1802:
		yych = *++cursor_;
		if (yych == 'r') goto yy1890;
		goto yy87;
yy1803:
		yych = *++cursor_;
		if (yych == '2') goto yy1891;
		goto yy87;
yy1804:
		yych = *++cursor_;
		if (yych == '4') goto yy1892;
		goto yy87;
yy1805:
		yych = *++cursor_;
		if (yych == 'f') goto yy1893;
		goto yy87;
yy1806:
		yych = *++cursor_;
		if (yych == 's') goto yy1894;
		goto yy87;
yy1807:
		yych = *++cursor_;
		if (yych == 't') goto yy1895;
		goto yy87;
yy1808:
		yych = *++cursor_;
		if (yych == 'f') goto yy1896;
		goto yy87;
yy1809:
		yych = *++cursor_;
		if (yych == 's') goto yy1897;
		goto yy87;
yy1810:
		yych = *++cursor_;
		if (yych == 'u') goto yy1898;
		goto yy87;
yy1811:
		yych = *++cursor_;
		if (yych == 'u') goto yy1899;
		goto yy87;
yy1812:
		yych = *++cursor_;
		if (yych == 't') goto yy1900;
		goto yy87;
yy1813:
		yych = *++cursor_;
		if (yych == 'e') goto yy1901;
		goto yy87;
yy1814:
		yych = *++cursor_;
		if (yych == 's') goto yy1902;
		goto yy87;
yy1815:
		yych = *++cursor_;
		if (yych == 't') goto yy1903;
		goto yy87;
yy1816:
		yych = *++cursor_;
		if (yych == 'u') goto yy1904;
		goto yy87;
yy1817:
		yych = *++cursor_;
		if (yych == 'u') goto yy1905;
		goto yy87;
yy1818:
		yych = *++cursor_;
		if (yych == 't') goto yy1906;
		goto yy87;
yy1819:
		yych = *++cursor_;
		if (yych == 'e') goto yy1907;
		goto yy87;
yy1820:
		yych = *++cursor_;
		if (yych == 't') goto yy1908;
		goto yy87;
yy1821:
		yych = *++cursor_;
		if (yych == 'i') goto yy1909;
		goto yy87;
yy1822:
		yych = *++cursor_;
		if (yych == 'c') goto yy1910;
		goto yy87;
yy1823:
		yych = *++cursor_;
		if (yych == 'e') goto yy1911;
		goto yy87;
yy1824:
		yych = *++cursor_;
		if (yych == 't') goto yy1913;
		goto yy87;
yy1825:
		yych = *++cursor_;
		if (yych == 'd') goto yy1914;
		goto yy87;
yy1826:
		yych = *++cursor_;
		if (yych == 'm') goto yy1916;
		goto yy87;
yy1827:
		++cursor_;
		if ((yych = *cursor_) <= ')') {
			if (yych <= '!') {
				if (yych >= '!') goto yy86;
			} else {
				if (yych <= '"') goto yy1828;
				if (yych <= '\'') goto yy86;
			}
		} else {
			if (yych <= '^') {
				if (yych != ';') goto yy86;
			} else {
				if (yych <= '_') goto yy1917;
				if (yych <= '~') goto yy86;
			}
		}
yy1828:
#line 732 "src/wast-lexer.cc"
		{ RETURN(AssertReturn); }
#line 9828 "src/prebuilt/wast-lexer-gen.cc"
yy1829:
		yych = *++cursor_;
		if (yych == 'a') goto yy1918;
		goto yy87;
yy1830:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 459 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(AtomicNotify); }
#line 9840 "src/prebuilt/wast-lexer-gen.cc"
yy1832:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 263 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(CallIndirect); }
#line 9848 "src/prebuilt/wast-lexer-gen.cc"
yy1834:
		yych = *++cursor_;
		if (yych == 'y') goto yy1919;
		goto yy87;
yy1835:
		yych = *++cursor_;
		if (yych == '3') goto yy1921;
		if (yych == '6') goto yy1922;
		goto yy87;
yy1836:
		yych = *++cursor_;
		if (yych == '/') goto yy1923;
		goto yy87;
yy1837:
		yych = *++cursor_;
		if (yych == '/') goto yy1924;
		goto yy87;
yy1838:
		yych = *++cursor_;
		if (yych == '4') goto yy1925;
		goto yy87;
yy1839:
		yych = *++cursor_;
		if (yych == '4') goto yy1927;
		goto yy87;
yy1840:
		yych = *++cursor_;
		if (yych == 'e') goto yy1929;
		goto yy87;
yy1841:
		yych = *++cursor_;
		if (yych == '_') goto yy1930;
		goto yy87;
yy1842:
		yych = *++cursor_;
		if (yych == '_') goto yy1931;
		goto yy87;
yy1843:
		yych = *++cursor_;
		if (yych == '_') goto yy1932;
		goto yy87;
yy1844:
		yych = *++cursor_;
		if (yych == '3') goto yy1933;
		if (yych == '6') goto yy1934;
		goto yy87;
yy1845:
		yych = *++cursor_;
		if (yych == '/') goto yy1935;
		goto yy87;
yy1846:
		yych = *++cursor_;
		if (yych == '/') goto yy1936;
		goto yy87;
yy1847:
		yych = *++cursor_;
		if (yych == '3') goto yy1937;
		goto yy87;
yy1848:
		yych = *++cursor_;
		if (yych == '3') goto yy1938;
		goto yy87;
yy1849:
		yych = *++cursor_;
		if (yych == 'e') goto yy1939;
		goto yy87;
yy1850:
		yych = *++cursor_;
		if (yych == '_') goto yy1940;
		goto yy87;
yy1851:
		yych = *++cursor_;
		if (yych == '_') goto yy1941;
		goto yy87;
yy1852:
		yych = *++cursor_;
		if (yych == '_') goto yy1942;
		goto yy87;
yy1853:
		yych = *++cursor_;
		if (yych == 'u') goto yy1943;
		goto yy87;
yy1854:
		yych = *++cursor_;
		if (yych == 'e') goto yy1944;
		goto yy87;
yy1855:
		yych = *++cursor_;
		if (yych == 'e') goto yy1946;
		goto yy87;
yy1856:
		yych = *++cursor_;
		if (yych == '_') goto yy1948;
		goto yy87;
yy1857:
		yych = *++cursor_;
		if (yych == '_') goto yy1949;
		goto yy87;
yy1858:
		yych = *++cursor_;
		if (yych == 'u') goto yy1950;
		goto yy87;
yy1859:
		yych = *++cursor_;
		if (yych == 'a') goto yy1951;
		goto yy87;
yy1860:
		yych = *++cursor_;
		if (yych == 'w') goto yy1952;
		goto yy87;
yy1861:
		yych = *++cursor_;
		if (yych == 'o') goto yy1953;
		goto yy87;
yy1862:
		yych = *++cursor_;
		if (yych == 'i') goto yy1954;
		goto yy87;
yy1863:
		yych = *++cursor_;
		if (yych == 's') goto yy1955;
		goto yy87;
yy1864:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 323 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32Extend8S); }
#line 9978 "src/prebuilt/wast-lexer-gen.cc"
yy1866:
		yych = *++cursor_;
		if (yych == 'e') goto yy1957;
		goto yy87;
yy1867:
		yych = *++cursor_;
		if (yych == '_') goto yy1958;
		goto yy87;
yy1868:
		yych = *++cursor_;
		if (yych == '_') goto yy1959;
		goto yy87;
yy1869:
		yych = *++cursor_;
		if (yych == '3') goto yy1960;
		if (yych == '6') goto yy1961;
		goto yy87;
yy1870:
		yych = *++cursor_;
		if (yych == 'a') goto yy1962;
		goto yy87;
yy1871:
		yych = *++cursor_;
		if (yych == '_') goto yy1963;
		goto yy87;
yy1872:
		yych = *++cursor_;
		if (yych == '3') goto yy1964;
		if (yych == '6') goto yy1965;
		goto yy87;
yy1873:
		yych = *++cursor_;
		if (yych == 'a') goto yy1966;
		goto yy87;
yy1874:
		yych = *++cursor_;
		if (yych == 'e') goto yy1967;
		goto yy87;
yy1875:
		yych = *++cursor_;
		if (yych == 'e') goto yy1969;
		goto yy87;
yy1876:
		yych = *++cursor_;
		if (yych == '_') goto yy1971;
		goto yy87;
yy1877:
		yych = *++cursor_;
		if (yych == '_') goto yy1972;
		goto yy87;
yy1878:
		yych = *++cursor_;
		if (yych == 'a') goto yy1973;
		goto yy87;
yy1879:
		yych = *++cursor_;
		if (yych == 'a') goto yy1974;
		goto yy87;
yy1880:
		yych = *++cursor_;
		if (yych == 'w') goto yy1975;
		goto yy87;
yy1881:
		yych = *++cursor_;
		if (yych == 'o') goto yy1976;
		goto yy87;
yy1882:
		yych = *++cursor_;
		if (yych == 'i') goto yy1977;
		goto yy87;
yy1883:
		yych = *++cursor_;
		if (yych == 's') goto yy1978;
		goto yy87;
yy1884:
		yych = *++cursor_;
		if (yych == 's') goto yy1980;
		goto yy87;
yy1885:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 325 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64Extend8S); }
#line 10064 "src/prebuilt/wast-lexer-gen.cc"
yy1887:
		yych = *++cursor_;
		if (yych == '2') goto yy1982;
		goto yy87;
yy1888:
		yych = *++cursor_;
		if (yych == 'i') goto yy1983;
		goto yy87;
yy1889:
		yych = *++cursor_;
		if (yych == 'i') goto yy1984;
		goto yy87;
yy1890:
		yych = *++cursor_;
		if (yych == 'e') goto yy1985;
		goto yy87;
yy1891:
		yych = *++cursor_;
		if (yych == '_') goto yy1986;
		goto yy87;
yy1892:
		yych = *++cursor_;
		if (yych == '_') goto yy1987;
		goto yy87;
yy1893:
		yych = *++cursor_;
		if (yych == '3') goto yy1988;
		if (yych == '6') goto yy1989;
		goto yy87;
yy1894:
		yych = *++cursor_;
		if (yych == 'a') goto yy1990;
		goto yy87;
yy1895:
		yych = *++cursor_;
		if (yych == '_') goto yy1991;
		goto yy87;
yy1896:
		yych = *++cursor_;
		if (yych == '3') goto yy1992;
		if (yych == '6') goto yy1993;
		goto yy87;
yy1897:
		yych = *++cursor_;
		if (yych == 'a') goto yy1994;
		goto yy87;
yy1898:
		yych = *++cursor_;
		if (yych == 'e') goto yy1995;
		goto yy87;
yy1899:
		yych = *++cursor_;
		if (yych == 'e') goto yy1997;
		goto yy87;
yy1900:
		yych = *++cursor_;
		if (yych == '_') goto yy1999;
		goto yy87;
yy1901:
		yych = *++cursor_;
		if (yych == '_') goto yy2000;
		goto yy87;
yy1902:
		yych = *++cursor_;
		if (yych == 'a') goto yy2001;
		goto yy87;
yy1903:
		yych = *++cursor_;
		if (yych == 'u') goto yy2002;
		goto yy87;
yy1904:
		yych = *++cursor_;
		if (yych == 'e') goto yy2003;
		goto yy87;
yy1905:
		yych = *++cursor_;
		if (yych == 'e') goto yy2005;
		goto yy87;
yy1906:
		yych = *++cursor_;
		if (yych == '_') goto yy2007;
		goto yy87;
yy1907:
		yych = *++cursor_;
		if (yych == '_') goto yy2008;
		goto yy87;
yy1908:
		yych = *++cursor_;
		if (yych == 'u') goto yy2009;
		goto yy87;
yy1909:
		yych = *++cursor_;
		if (yych == 'n') goto yy2010;
		goto yy87;
yy1910:
		yych = *++cursor_;
		if (yych == 't') goto yy2011;
		goto yy87;
yy1911:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 546 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdShuffleOp, V8X16Shuffle); }
#line 10170 "src/prebuilt/wast-lexer-gen.cc"
yy1913:
		yych = *++cursor_;
		if (yych == 'i') goto yy2013;
		goto yy87;
yy1914:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 730 "src/wast-lexer.cc"
		{ RETURN(AssertInvalid); }
#line 10182 "src/prebuilt/wast-lexer-gen.cc"
yy1916:
		yych = *++cursor_;
		if (yych == 'e') goto yy2014;
		goto yy87;
yy1917:
		yych = *++cursor_;
		if (yych == 'a') goto yy2015;
		if (yych == 'c') goto yy2016;
		goto yy87;
yy1918:
		yych = *++cursor_;
		if (yych == 'b') goto yy2017;
		goto yy87;
yy1919:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 445 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(MemorySize); }
#line 10203 "src/prebuilt/wast-lexer-gen.cc"
yy1921:
		yych = *++cursor_;
		if (yych == '2') goto yy2018;
		goto yy87;
yy1922:
		yych = *++cursor_;
		if (yych == '4') goto yy2019;
		goto yy87;
yy1923:
		yych = *++cursor_;
		if (yych == 'i') goto yy2020;
		goto yy87;
yy1924:
		yych = *++cursor_;
		if (yych == 'i') goto yy2021;
		goto yy87;
yy1925:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 693 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32DemoteF64); }
#line 10227 "src/prebuilt/wast-lexer-gen.cc"
yy1927:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 432 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32DemoteF64); }
#line 10235 "src/prebuilt/wast-lexer-gen.cc"
yy1929:
		yych = *++cursor_;
		if (yych == 't') goto yy2022;
		goto yy87;
yy1930:
		yych = *++cursor_;
		if (yych == 'i') goto yy2023;
		goto yy87;
yy1931:
		yych = *++cursor_;
		if (yych == 'l') goto yy2024;
		goto yy87;
yy1932:
		yych = *++cursor_;
		if (yych == 'l') goto yy2025;
		goto yy87;
yy1933:
		yych = *++cursor_;
		if (yych == '2') goto yy2026;
		goto yy87;
yy1934:
		yych = *++cursor_;
		if (yych == '4') goto yy2027;
		goto yy87;
yy1935:
		yych = *++cursor_;
		if (yych == 'i') goto yy2028;
		goto yy87;
yy1936:
		yych = *++cursor_;
		if (yych == 'i') goto yy2029;
		goto yy87;
yy1937:
		yych = *++cursor_;
		if (yych == '2') goto yy2030;
		goto yy87;
yy1938:
		yych = *++cursor_;
		if (yych == '2') goto yy2032;
		goto yy87;
yy1939:
		yych = *++cursor_;
		if (yych == 't') goto yy2034;
		goto yy87;
yy1940:
		yych = *++cursor_;
		if (yych == 'i') goto yy2035;
		goto yy87;
yy1941:
		yych = *++cursor_;
		if (yych == 'l') goto yy2036;
		goto yy87;
yy1942:
		yych = *++cursor_;
		if (yych == 'l') goto yy2037;
		goto yy87;
yy1943:
		yych = *++cursor_;
		if (yych == 'r') goto yy2038;
		goto yy87;
yy1944:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 592 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I16X8AllTrue); }
#line 10303 "src/prebuilt/wast-lexer-gen.cc"
yy1946:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 588 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I16X8AnyTrue); }
#line 10311 "src/prebuilt/wast-lexer-gen.cc"
yy1948:
		yych = *++cursor_;
		if (yych == 'l') goto yy2039;
		goto yy87;
yy1949:
		yych = *++cursor_;
		if (yych == 'l') goto yy2040;
		goto yy87;
yy1950:
		yych = *++cursor_;
		if (yych == 'r') goto yy2041;
		goto yy87;
yy1951:
		yych = *++cursor_;
		if (yych == 'd') goto yy2042;
		goto yy87;
yy1952:
		yych = *++cursor_;
		if (yych <= '0') {
			if (yych == '.') goto yy2044;
			goto yy87;
		} else {
			if (yych <= '1') goto yy2045;
			if (yych == '8') goto yy2046;
			goto yy87;
		}
yy1953:
		yych = *++cursor_;
		if (yych == 'r') goto yy2047;
		goto yy87;
yy1954:
		yych = *++cursor_;
		if (yych == 't') goto yy2048;
		goto yy87;
yy1955:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 324 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32Extend16S); }
#line 10353 "src/prebuilt/wast-lexer-gen.cc"
yy1957:
		yych = *++cursor_;
		if (yych == 't') goto yy2050;
		goto yy87;
yy1958:
		yych = *++cursor_;
		if (yych == 's') goto yy2051;
		if (yych == 'u') goto yy2053;
		goto yy87;
yy1959:
		yych = *++cursor_;
		if (yych == 's') goto yy2055;
		if (yych == 'u') goto yy2057;
		goto yy87;
yy1960:
		yych = *++cursor_;
		if (yych == '2') goto yy2059;
		goto yy87;
yy1961:
		yych = *++cursor_;
		if (yych == '4') goto yy2061;
		goto yy87;
yy1962:
		yych = *++cursor_;
		if (yych == 't') goto yy2063;
		goto yy87;
yy1963:
		yych = *++cursor_;
		if (yych == 'f') goto yy2064;
		goto yy87;
yy1964:
		yych = *++cursor_;
		if (yych == '2') goto yy2065;
		goto yy87;
yy1965:
		yych = *++cursor_;
		if (yych == '4') goto yy2067;
		goto yy87;
yy1966:
		yych = *++cursor_;
		if (yych == 't') goto yy2069;
		goto yy87;
yy1967:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 593 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I32X4AllTrue); }
#line 10403 "src/prebuilt/wast-lexer-gen.cc"
yy1969:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 589 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I32X4AnyTrue); }
#line 10411 "src/prebuilt/wast-lexer-gen.cc"
yy1971:
		yych = *++cursor_;
		if (yych == 'l') goto yy2070;
		goto yy87;
yy1972:
		yych = *++cursor_;
		if (yych == 'l') goto yy2071;
		goto yy87;
yy1973:
		yych = *++cursor_;
		if (yych == 't') goto yy2072;
		goto yy87;
yy1974:
		yych = *++cursor_;
		if (yych == 'd') goto yy2073;
		goto yy87;
yy1975:
		yych = *++cursor_;
		switch (yych) {
		case '.':	goto yy2075;
		case '1':	goto yy2076;
		case '3':	goto yy2077;
		case '8':	goto yy2078;
		default:	goto yy87;
		}
yy1976:
		yych = *++cursor_;
		if (yych == 'r') goto yy2079;
		goto yy87;
yy1977:
		yych = *++cursor_;
		if (yych == 't') goto yy2080;
		goto yy87;
yy1978:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 326 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64Extend16S); }
#line 10452 "src/prebuilt/wast-lexer-gen.cc"
yy1980:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 327 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64Extend32S); }
#line 10460 "src/prebuilt/wast-lexer-gen.cc"
yy1982:
		yych = *++cursor_;
		if (yych == '_') goto yy2082;
		goto yy87;
yy1983:
		yych = *++cursor_;
		if (yych == '3') goto yy2083;
		goto yy87;
yy1984:
		yych = *++cursor_;
		if (yych == '3') goto yy2084;
		goto yy87;
yy1985:
		yych = *++cursor_;
		if (yych == 't') goto yy2085;
		goto yy87;
yy1986:
		yych = *++cursor_;
		if (yych == 's') goto yy2086;
		if (yych == 'u') goto yy2088;
		goto yy87;
yy1987:
		yych = *++cursor_;
		if (yych == 's') goto yy2090;
		if (yych == 'u') goto yy2092;
		goto yy87;
yy1988:
		yych = *++cursor_;
		if (yych == '2') goto yy2094;
		goto yy87;
yy1989:
		yych = *++cursor_;
		if (yych == '4') goto yy2096;
		goto yy87;
yy1990:
		yych = *++cursor_;
		if (yych == 't') goto yy2098;
		goto yy87;
yy1991:
		yych = *++cursor_;
		if (yych == 'f') goto yy2099;
		goto yy87;
yy1992:
		yych = *++cursor_;
		if (yych == '2') goto yy2100;
		goto yy87;
yy1993:
		yych = *++cursor_;
		if (yych == '4') goto yy2102;
		goto yy87;
yy1994:
		yych = *++cursor_;
		if (yych == 't') goto yy2104;
		goto yy87;
yy1995:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 594 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I64X2AllTrue); }
#line 10522 "src/prebuilt/wast-lexer-gen.cc"
yy1997:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 590 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I64X2AnyTrue); }
#line 10530 "src/prebuilt/wast-lexer-gen.cc"
yy1999:
		yych = *++cursor_;
		if (yych == 'l') goto yy2105;
		goto yy87;
yy2000:
		yych = *++cursor_;
		if (yych == 'l') goto yy2106;
		goto yy87;
yy2001:
		yych = *++cursor_;
		if (yych == 't') goto yy2107;
		goto yy87;
yy2002:
		yych = *++cursor_;
		if (yych == 'r') goto yy2108;
		goto yy87;
yy2003:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 591 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I8X16AllTrue); }
#line 10554 "src/prebuilt/wast-lexer-gen.cc"
yy2005:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 587 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary,  I8X16AnyTrue); }
#line 10562 "src/prebuilt/wast-lexer-gen.cc"
yy2007:
		yych = *++cursor_;
		if (yych == 'l') goto yy2109;
		goto yy87;
yy2008:
		yych = *++cursor_;
		if (yych == 'l') goto yy2110;
		goto yy87;
yy2009:
		yych = *++cursor_;
		if (yych == 'r') goto yy2111;
		goto yy87;
yy2010:
		yych = *++cursor_;
		if (yych == 'd') goto yy2112;
		goto yy87;
yy2011:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 586 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Ternary, V128BitSelect); }
#line 10586 "src/prebuilt/wast-lexer-gen.cc"
yy2013:
		yych = *++cursor_;
		if (yych == 'o') goto yy2113;
		goto yy87;
yy2014:
		yych = *++cursor_;
		if (yych == 'd') goto yy2114;
		goto yy87;
yy2015:
		yych = *++cursor_;
		if (yych == 'r') goto yy2116;
		goto yy87;
yy2016:
		yych = *++cursor_;
		if (yych == 'a') goto yy2117;
		goto yy87;
yy2017:
		yych = *++cursor_;
		if (yych == 'l') goto yy2118;
		goto yy87;
yy2018:
		yych = *++cursor_;
		if (yych == '_') goto yy2119;
		goto yy87;
yy2019:
		yych = *++cursor_;
		if (yych == '_') goto yy2120;
		goto yy87;
yy2020:
		yych = *++cursor_;
		if (yych == '3') goto yy2121;
		if (yych == '6') goto yy2122;
		goto yy87;
yy2021:
		yych = *++cursor_;
		if (yych == '3') goto yy2123;
		if (yych == '6') goto yy2124;
		goto yy87;
yy2022:
		yych = *++cursor_;
		if (yych == '/') goto yy2125;
		if (yych == '_') goto yy2126;
		goto yy87;
yy2023:
		yych = *++cursor_;
		if (yych == '3') goto yy2127;
		goto yy87;
yy2024:
		yych = *++cursor_;
		if (yych == 'a') goto yy2128;
		goto yy87;
yy2025:
		yych = *++cursor_;
		if (yych == 'a') goto yy2129;
		goto yy87;
yy2026:
		yych = *++cursor_;
		if (yych == '_') goto yy2130;
		goto yy87;
yy2027:
		yych = *++cursor_;
		if (yych == '_') goto yy2131;
		goto yy87;
yy2028:
		yych = *++cursor_;
		if (yych == '3') goto yy2132;
		if (yych == '6') goto yy2133;
		goto yy87;
yy2029:
		yych = *++cursor_;
		if (yych == '3') goto yy2134;
		if (yych == '6') goto yy2135;
		goto yy87;
yy2030:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 692 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64PromoteF32); }
#line 10667 "src/prebuilt/wast-lexer-gen.cc"
yy2032:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 431 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64PromoteF32); }
#line 10675 "src/prebuilt/wast-lexer-gen.cc"
yy2034:
		yych = *++cursor_;
		if (yych == '/') goto yy2136;
		if (yych == '_') goto yy2137;
		goto yy87;
yy2035:
		yych = *++cursor_;
		if (yych == '6') goto yy2138;
		goto yy87;
yy2036:
		yych = *++cursor_;
		if (yych == 'a') goto yy2139;
		goto yy87;
yy2037:
		yych = *++cursor_;
		if (yych == 'a') goto yy2140;
		goto yy87;
yy2038:
		yych = *++cursor_;
		if (yych == 'a') goto yy2141;
		goto yy87;
yy2039:
		yych = *++cursor_;
		if (yych == 'a') goto yy2142;
		goto yy87;
yy2040:
		yych = *++cursor_;
		if (yych == 'a') goto yy2143;
		goto yy87;
yy2041:
		yych = *++cursor_;
		if (yych == 'a') goto yy2144;
		goto yy87;
yy2042:
		++cursor_;
		if ((yych = *cursor_) <= '0') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= '8') {
				if (yych <= '1') goto yy2145;
				if (yych <= '7') goto yy86;
				goto yy2146;
			} else {
				if (yych == ';') goto yy2043;
				if (yych <= '~') goto yy86;
			}
		}
yy2043:
#line 460 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I32AtomicLoad); }
#line 10731 "src/prebuilt/wast-lexer-gen.cc"
yy2044:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2147;
		case 'c':	goto yy2148;
		case 'o':	goto yy2149;
		case 's':	goto yy2150;
		case 'x':	goto yy2151;
		default:	goto yy87;
		}
yy2045:
		yych = *++cursor_;
		if (yych == '6') goto yy2152;
		goto yy87;
yy2046:
		yych = *++cursor_;
		if (yych == '.') goto yy2153;
		goto yy87;
yy2047:
		yych = *++cursor_;
		if (yych == 'e') goto yy2154;
		goto yy87;
yy2048:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 457 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicWait, I32AtomicWait); }
#line 10761 "src/prebuilt/wast-lexer-gen.cc"
yy2050:
		yych = *++cursor_;
		if (yych == '/') goto yy2156;
		if (yych == '_') goto yy2157;
		goto yy87;
yy2051:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 407 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF32S); }
#line 10774 "src/prebuilt/wast-lexer-gen.cc"
yy2053:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 411 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF32U); }
#line 10782 "src/prebuilt/wast-lexer-gen.cc"
yy2055:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 409 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF64S); }
#line 10790 "src/prebuilt/wast-lexer-gen.cc"
yy2057:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 413 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF64U); }
#line 10798 "src/prebuilt/wast-lexer-gen.cc"
yy2059:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 676 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF32S); }
#line 10806 "src/prebuilt/wast-lexer-gen.cc"
yy2061:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 678 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF64S); }
#line 10814 "src/prebuilt/wast-lexer-gen.cc"
yy2063:
		yych = *++cursor_;
		if (yych == '/') goto yy2158;
		goto yy87;
yy2064:
		yych = *++cursor_;
		if (yych == '3') goto yy2159;
		if (yych == '6') goto yy2160;
		goto yy87;
yy2065:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 680 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF32U); }
#line 10831 "src/prebuilt/wast-lexer-gen.cc"
yy2067:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 682 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncF64U); }
#line 10839 "src/prebuilt/wast-lexer-gen.cc"
yy2069:
		yych = *++cursor_;
		if (yych == '/') goto yy2161;
		goto yy87;
yy2070:
		yych = *++cursor_;
		if (yych == 'a') goto yy2162;
		goto yy87;
yy2071:
		yych = *++cursor_;
		if (yych == 'a') goto yy2163;
		goto yy87;
yy2072:
		yych = *++cursor_;
		if (yych == '_') goto yy2164;
		goto yy87;
yy2073:
		++cursor_;
		if ((yych = *cursor_) <= '1') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy2074;
				if (yych <= '0') goto yy86;
				goto yy2165;
			}
		} else {
			if (yych <= '8') {
				if (yych == '3') goto yy2166;
				if (yych <= '7') goto yy86;
				goto yy2167;
			} else {
				if (yych == ';') goto yy2074;
				if (yych <= '~') goto yy86;
			}
		}
yy2074:
#line 461 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I64AtomicLoad); }
#line 10880 "src/prebuilt/wast-lexer-gen.cc"
yy2075:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2168;
		case 'c':	goto yy2169;
		case 'o':	goto yy2170;
		case 's':	goto yy2171;
		case 'x':	goto yy2172;
		default:	goto yy87;
		}
yy2076:
		yych = *++cursor_;
		if (yych == '6') goto yy2173;
		goto yy87;
yy2077:
		yych = *++cursor_;
		if (yych == '2') goto yy2174;
		goto yy87;
yy2078:
		yych = *++cursor_;
		if (yych == '.') goto yy2175;
		goto yy87;
yy2079:
		yych = *++cursor_;
		if (yych == 'e') goto yy2176;
		goto yy87;
yy2080:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 458 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicWait, I64AtomicWait); }
#line 10914 "src/prebuilt/wast-lexer-gen.cc"
yy2082:
		yych = *++cursor_;
		if (yych == 's') goto yy2178;
		if (yych == 'u') goto yy2180;
		goto yy87;
yy2083:
		yych = *++cursor_;
		if (yych == '2') goto yy2182;
		goto yy87;
yy2084:
		yych = *++cursor_;
		if (yych == '2') goto yy2184;
		goto yy87;
yy2085:
		yych = *++cursor_;
		if (yych == '/') goto yy2186;
		if (yych == '_') goto yy2187;
		goto yy87;
yy2086:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 408 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF32S); }
#line 10940 "src/prebuilt/wast-lexer-gen.cc"
yy2088:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 412 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF32U); }
#line 10948 "src/prebuilt/wast-lexer-gen.cc"
yy2090:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 410 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF64S); }
#line 10956 "src/prebuilt/wast-lexer-gen.cc"
yy2092:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 414 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF64U); }
#line 10964 "src/prebuilt/wast-lexer-gen.cc"
yy2094:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 677 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF32S); }
#line 10972 "src/prebuilt/wast-lexer-gen.cc"
yy2096:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 679 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF64S); }
#line 10980 "src/prebuilt/wast-lexer-gen.cc"
yy2098:
		yych = *++cursor_;
		if (yych == '/') goto yy2188;
		goto yy87;
yy2099:
		yych = *++cursor_;
		if (yych == '3') goto yy2189;
		if (yych == '6') goto yy2190;
		goto yy87;
yy2100:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 681 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF32U); }
#line 10997 "src/prebuilt/wast-lexer-gen.cc"
yy2102:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 683 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncF64U); }
#line 11005 "src/prebuilt/wast-lexer-gen.cc"
yy2104:
		yych = *++cursor_;
		if (yych == '/') goto yy2191;
		goto yy87;
yy2105:
		yych = *++cursor_;
		if (yych == 'a') goto yy2192;
		goto yy87;
yy2106:
		yych = *++cursor_;
		if (yych == 'a') goto yy2193;
		goto yy87;
yy2107:
		yych = *++cursor_;
		if (yych == '_') goto yy2194;
		goto yy87;
yy2108:
		yych = *++cursor_;
		if (yych == 'a') goto yy2195;
		goto yy87;
yy2109:
		yych = *++cursor_;
		if (yych == 'a') goto yy2196;
		goto yy87;
yy2110:
		yych = *++cursor_;
		if (yych == 'a') goto yy2197;
		goto yy87;
yy2111:
		yych = *++cursor_;
		if (yych == 'a') goto yy2198;
		goto yy87;
yy2112:
		yych = *++cursor_;
		if (yych == 'i') goto yy2199;
		goto yy87;
yy2113:
		yych = *++cursor_;
		if (yych == 'n') goto yy2200;
		goto yy87;
yy2114:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 729 "src/wast-lexer.cc"
		{ RETURN(AssertMalformed); }
#line 11053 "src/prebuilt/wast-lexer-gen.cc"
yy2116:
		yych = *++cursor_;
		if (yych == 'i') goto yy2202;
		goto yy87;
yy2117:
		yych = *++cursor_;
		if (yych == 'n') goto yy2203;
		goto yy87;
yy2118:
		yych = *++cursor_;
		if (yych == 'e') goto yy2204;
		goto yy87;
yy2119:
		yych = *++cursor_;
		if (yych == 's') goto yy2206;
		if (yych == 'u') goto yy2208;
		goto yy87;
yy2120:
		yych = *++cursor_;
		if (yych == 's') goto yy2210;
		if (yych == 'u') goto yy2212;
		goto yy87;
yy2121:
		yych = *++cursor_;
		if (yych == '2') goto yy2214;
		goto yy87;
yy2122:
		yych = *++cursor_;
		if (yych == '4') goto yy2216;
		goto yy87;
yy2123:
		yych = *++cursor_;
		if (yych == '2') goto yy2218;
		goto yy87;
yy2124:
		yych = *++cursor_;
		if (yych == '4') goto yy2220;
		goto yy87;
yy2125:
		yych = *++cursor_;
		if (yych == 'i') goto yy2222;
		goto yy87;
yy2126:
		yych = *++cursor_;
		if (yych == 'i') goto yy2223;
		goto yy87;
yy2127:
		yych = *++cursor_;
		if (yych == '2') goto yy2224;
		goto yy87;
yy2128:
		yych = *++cursor_;
		if (yych == 'n') goto yy2225;
		goto yy87;
yy2129:
		yych = *++cursor_;
		if (yych == 'n') goto yy2226;
		goto yy87;
yy2130:
		yych = *++cursor_;
		if (yych == 's') goto yy2227;
		if (yych == 'u') goto yy2229;
		goto yy87;
yy2131:
		yych = *++cursor_;
		if (yych == 's') goto yy2231;
		if (yych == 'u') goto yy2233;
		goto yy87;
yy2132:
		yych = *++cursor_;
		if (yych == '2') goto yy2235;
		goto yy87;
yy2133:
		yych = *++cursor_;
		if (yych == '4') goto yy2237;
		goto yy87;
yy2134:
		yych = *++cursor_;
		if (yych == '2') goto yy2239;
		goto yy87;
yy2135:
		yych = *++cursor_;
		if (yych == '4') goto yy2241;
		goto yy87;
yy2136:
		yych = *++cursor_;
		if (yych == 'i') goto yy2243;
		goto yy87;
yy2137:
		yych = *++cursor_;
		if (yych == 'i') goto yy2244;
		goto yy87;
yy2138:
		yych = *++cursor_;
		if (yych == '4') goto yy2245;
		goto yy87;
yy2139:
		yych = *++cursor_;
		if (yych == 'n') goto yy2246;
		goto yy87;
yy2140:
		yych = *++cursor_;
		if (yych == 'n') goto yy2247;
		goto yy87;
yy2141:
		yych = *++cursor_;
		if (yych == 't') goto yy2248;
		goto yy87;
yy2142:
		yych = *++cursor_;
		if (yych == 'n') goto yy2249;
		goto yy87;
yy2143:
		yych = *++cursor_;
		if (yych == 'n') goto yy2250;
		goto yy87;
yy2144:
		yych = *++cursor_;
		if (yych == 't') goto yy2251;
		goto yy87;
yy2145:
		yych = *++cursor_;
		if (yych == '6') goto yy2252;
		goto yy87;
yy2146:
		yych = *++cursor_;
		if (yych == '_') goto yy2253;
		goto yy87;
yy2147:
		yych = *++cursor_;
		if (yych == 'd') goto yy2254;
		if (yych == 'n') goto yy2255;
		goto yy87;
yy2148:
		yych = *++cursor_;
		if (yych == 'm') goto yy2256;
		goto yy87;
yy2149:
		yych = *++cursor_;
		if (yych == 'r') goto yy2257;
		goto yy87;
yy2150:
		yych = *++cursor_;
		if (yych == 'u') goto yy2259;
		goto yy87;
yy2151:
		yych = *++cursor_;
		if (yych == 'c') goto yy2260;
		if (yych == 'o') goto yy2261;
		goto yy87;
yy2152:
		yych = *++cursor_;
		if (yych == '.') goto yy2262;
		goto yy87;
yy2153:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2263;
		case 'c':	goto yy2264;
		case 'o':	goto yy2265;
		case 's':	goto yy2266;
		case 'x':	goto yy2267;
		default:	goto yy87;
		}
yy2154:
		++cursor_;
		if ((yych = *cursor_) <= '0') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych >= '*') goto yy86;
			}
		} else {
			if (yych <= '8') {
				if (yych <= '1') goto yy2268;
				if (yych <= '7') goto yy86;
				goto yy2269;
			} else {
				if (yych == ';') goto yy2155;
				if (yych <= '~') goto yy86;
			}
		}
yy2155:
#line 467 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I32AtomicStore); }
#line 11240 "src/prebuilt/wast-lexer-gen.cc"
yy2156:
		yych = *++cursor_;
		if (yych == 'f') goto yy2271;
		goto yy87;
yy2157:
		yych = *++cursor_;
		if (yych == 'f') goto yy2272;
		goto yy87;
yy2158:
		yych = *++cursor_;
		if (yych == 'f') goto yy2273;
		goto yy87;
yy2159:
		yych = *++cursor_;
		if (yych == '2') goto yy2274;
		goto yy87;
yy2160:
		yych = *++cursor_;
		if (yych == '4') goto yy2275;
		goto yy87;
yy2161:
		yych = *++cursor_;
		if (yych == 'f') goto yy2276;
		goto yy87;
yy2162:
		yych = *++cursor_;
		if (yych == 'n') goto yy2277;
		goto yy87;
yy2163:
		yych = *++cursor_;
		if (yych == 'n') goto yy2278;
		goto yy87;
yy2164:
		yych = *++cursor_;
		if (yych == 'f') goto yy2279;
		goto yy87;
yy2165:
		yych = *++cursor_;
		if (yych == '6') goto yy2280;
		goto yy87;
yy2166:
		yych = *++cursor_;
		if (yych == '2') goto yy2281;
		goto yy87;
yy2167:
		yych = *++cursor_;
		if (yych == '_') goto yy2282;
		goto yy87;
yy2168:
		yych = *++cursor_;
		if (yych == 'd') goto yy2283;
		if (yych == 'n') goto yy2284;
		goto yy87;
yy2169:
		yych = *++cursor_;
		if (yych == 'm') goto yy2285;
		goto yy87;
yy2170:
		yych = *++cursor_;
		if (yych == 'r') goto yy2286;
		goto yy87;
yy2171:
		yych = *++cursor_;
		if (yych == 'u') goto yy2288;
		goto yy87;
yy2172:
		yych = *++cursor_;
		if (yych == 'c') goto yy2289;
		if (yych == 'o') goto yy2290;
		goto yy87;
yy2173:
		yych = *++cursor_;
		if (yych == '.') goto yy2291;
		goto yy87;
yy2174:
		yych = *++cursor_;
		if (yych == '.') goto yy2292;
		goto yy87;
yy2175:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2293;
		case 'c':	goto yy2294;
		case 'o':	goto yy2295;
		case 's':	goto yy2296;
		case 'x':	goto yy2297;
		default:	goto yy87;
		}
yy2176:
		++cursor_;
		if ((yych = *cursor_) <= '1') {
			if (yych <= '"') {
				if (yych == '!') goto yy86;
			} else {
				if (yych <= '\'') goto yy86;
				if (yych <= ')') goto yy2177;
				if (yych <= '0') goto yy86;
				goto yy2298;
			}
		} else {
			if (yych <= '8') {
				if (yych == '3') goto yy2299;
				if (yych <= '7') goto yy86;
				goto yy2300;
			} else {
				if (yych == ';') goto yy2177;
				if (yych <= '~') goto yy86;
			}
		}
yy2177:
#line 468 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I64AtomicStore); }
#line 11353 "src/prebuilt/wast-lexer-gen.cc"
yy2178:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 404 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64ExtendI32S); }
#line 11361 "src/prebuilt/wast-lexer-gen.cc"
yy2180:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 405 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64ExtendI32U); }
#line 11369 "src/prebuilt/wast-lexer-gen.cc"
yy2182:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 673 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64ExtendI32S); }
#line 11377 "src/prebuilt/wast-lexer-gen.cc"
yy2184:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 674 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64ExtendI32U); }
#line 11385 "src/prebuilt/wast-lexer-gen.cc"
yy2186:
		yych = *++cursor_;
		if (yych == 'f') goto yy2302;
		goto yy87;
yy2187:
		yych = *++cursor_;
		if (yych == 'f') goto yy2303;
		goto yy87;
yy2188:
		yych = *++cursor_;
		if (yych == 'f') goto yy2304;
		goto yy87;
yy2189:
		yych = *++cursor_;
		if (yych == '2') goto yy2305;
		goto yy87;
yy2190:
		yych = *++cursor_;
		if (yych == '4') goto yy2306;
		goto yy87;
yy2191:
		yych = *++cursor_;
		if (yych == 'f') goto yy2307;
		goto yy87;
yy2192:
		yych = *++cursor_;
		if (yych == 'n') goto yy2308;
		goto yy87;
yy2193:
		yych = *++cursor_;
		if (yych == 'n') goto yy2309;
		goto yy87;
yy2194:
		yych = *++cursor_;
		if (yych == 'f') goto yy2310;
		goto yy87;
yy2195:
		yych = *++cursor_;
		if (yych == 't') goto yy2311;
		goto yy87;
yy2196:
		yych = *++cursor_;
		if (yych == 'n') goto yy2312;
		goto yy87;
yy2197:
		yych = *++cursor_;
		if (yych == 'n') goto yy2313;
		goto yy87;
yy2198:
		yych = *++cursor_;
		if (yych == 't') goto yy2314;
		goto yy87;
yy2199:
		yych = *++cursor_;
		if (yych == 'r') goto yy2315;
		goto yy87;
yy2200:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 736 "src/wast-lexer.cc"
		{ RETURN(AssertExhaustion); }
#line 11449 "src/prebuilt/wast-lexer-gen.cc"
yy2202:
		yych = *++cursor_;
		if (yych == 't') goto yy2316;
		goto yy87;
yy2203:
		yych = *++cursor_;
		if (yych == 'o') goto yy2317;
		goto yy87;
yy2204:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 731 "src/wast-lexer.cc"
		{ RETURN(AssertUnlinkable); }
#line 11465 "src/prebuilt/wast-lexer-gen.cc"
yy2206:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 423 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI32S); }
#line 11473 "src/prebuilt/wast-lexer-gen.cc"
yy2208:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 427 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI32U); }
#line 11481 "src/prebuilt/wast-lexer-gen.cc"
yy2210:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 425 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI64S); }
#line 11489 "src/prebuilt/wast-lexer-gen.cc"
yy2212:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 429 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI64U); }
#line 11497 "src/prebuilt/wast-lexer-gen.cc"
yy2214:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 684 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI32S); }
#line 11505 "src/prebuilt/wast-lexer-gen.cc"
yy2216:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 686 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI64S); }
#line 11513 "src/prebuilt/wast-lexer-gen.cc"
yy2218:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 688 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI32U); }
#line 11521 "src/prebuilt/wast-lexer-gen.cc"
yy2220:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 690 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ConvertI64U); }
#line 11529 "src/prebuilt/wast-lexer-gen.cc"
yy2222:
		yych = *++cursor_;
		if (yych == '3') goto yy2318;
		goto yy87;
yy2223:
		yych = *++cursor_;
		if (yych == '3') goto yy2319;
		goto yy87;
yy2224:
		yych = *++cursor_;
		if (yych == 'x') goto yy2320;
		goto yy87;
yy2225:
		yych = *++cursor_;
		if (yych == 'e') goto yy2321;
		goto yy87;
yy2226:
		yych = *++cursor_;
		if (yych == 'e') goto yy2323;
		goto yy87;
yy2227:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 424 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI32S); }
#line 11557 "src/prebuilt/wast-lexer-gen.cc"
yy2229:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 428 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI32U); }
#line 11565 "src/prebuilt/wast-lexer-gen.cc"
yy2231:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 426 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI64S); }
#line 11573 "src/prebuilt/wast-lexer-gen.cc"
yy2233:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 430 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI64U); }
#line 11581 "src/prebuilt/wast-lexer-gen.cc"
yy2235:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 685 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI32S); }
#line 11589 "src/prebuilt/wast-lexer-gen.cc"
yy2237:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 687 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI64S); }
#line 11597 "src/prebuilt/wast-lexer-gen.cc"
yy2239:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 689 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI32U); }
#line 11605 "src/prebuilt/wast-lexer-gen.cc"
yy2241:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 691 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ConvertI64U); }
#line 11613 "src/prebuilt/wast-lexer-gen.cc"
yy2243:
		yych = *++cursor_;
		if (yych == '6') goto yy2325;
		goto yy87;
yy2244:
		yych = *++cursor_;
		if (yych == '6') goto yy2326;
		goto yy87;
yy2245:
		yych = *++cursor_;
		if (yych == 'x') goto yy2327;
		goto yy87;
yy2246:
		yych = *++cursor_;
		if (yych == 'e') goto yy2328;
		goto yy87;
yy2247:
		yych = *++cursor_;
		if (yych == 'e') goto yy2330;
		goto yy87;
yy2248:
		yych = *++cursor_;
		if (yych == 'e') goto yy2332;
		goto yy87;
yy2249:
		yych = *++cursor_;
		if (yych == 'e') goto yy2333;
		goto yy87;
yy2250:
		yych = *++cursor_;
		if (yych == 'e') goto yy2334;
		goto yy87;
yy2251:
		yych = *++cursor_;
		if (yych == 'e') goto yy2336;
		goto yy87;
yy2252:
		yych = *++cursor_;
		if (yych == '_') goto yy2337;
		goto yy87;
yy2253:
		yych = *++cursor_;
		if (yych == 'u') goto yy2338;
		goto yy87;
yy2254:
		yych = *++cursor_;
		if (yych == 'd') goto yy2340;
		goto yy87;
yy2255:
		yych = *++cursor_;
		if (yych == 'd') goto yy2342;
		goto yy87;
yy2256:
		yych = *++cursor_;
		if (yych == 'p') goto yy2344;
		goto yy87;
yy2257:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 495 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmwOr); }
#line 11677 "src/prebuilt/wast-lexer-gen.cc"
yy2259:
		yych = *++cursor_;
		if (yych == 'b') goto yy2345;
		goto yy87;
yy2260:
		yych = *++cursor_;
		if (yych == 'h') goto yy2347;
		goto yy87;
yy2261:
		yych = *++cursor_;
		if (yych == 'r') goto yy2348;
		goto yy87;
yy2262:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2350;
		case 'c':	goto yy2351;
		case 'o':	goto yy2352;
		case 's':	goto yy2353;
		case 'x':	goto yy2354;
		default:	goto yy87;
		}
yy2263:
		yych = *++cursor_;
		if (yych == 'd') goto yy2355;
		if (yych == 'n') goto yy2356;
		goto yy87;
yy2264:
		yych = *++cursor_;
		if (yych == 'm') goto yy2357;
		goto yy87;
yy2265:
		yych = *++cursor_;
		if (yych == 'r') goto yy2358;
		goto yy87;
yy2266:
		yych = *++cursor_;
		if (yych == 'u') goto yy2359;
		goto yy87;
yy2267:
		yych = *++cursor_;
		if (yych == 'c') goto yy2360;
		if (yych == 'o') goto yy2361;
		goto yy87;
yy2268:
		yych = *++cursor_;
		if (yych == '6') goto yy2362;
		goto yy87;
yy2269:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 469 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I32AtomicStore8); }
#line 11733 "src/prebuilt/wast-lexer-gen.cc"
yy2271:
		yych = *++cursor_;
		if (yych == '3') goto yy2364;
		goto yy87;
yy2272:
		yych = *++cursor_;
		if (yych == '3') goto yy2365;
		goto yy87;
yy2273:
		yych = *++cursor_;
		if (yych == '3') goto yy2366;
		if (yych == '6') goto yy2367;
		goto yy87;
yy2274:
		yych = *++cursor_;
		if (yych == '_') goto yy2368;
		goto yy87;
yy2275:
		yych = *++cursor_;
		if (yych == '_') goto yy2369;
		goto yy87;
yy2276:
		yych = *++cursor_;
		if (yych == '3') goto yy2370;
		if (yych == '6') goto yy2371;
		goto yy87;
yy2277:
		yych = *++cursor_;
		if (yych == 'e') goto yy2372;
		goto yy87;
yy2278:
		yych = *++cursor_;
		if (yych == 'e') goto yy2374;
		goto yy87;
yy2279:
		yych = *++cursor_;
		if (yych == '3') goto yy2376;
		goto yy87;
yy2280:
		yych = *++cursor_;
		if (yych == '_') goto yy2377;
		goto yy87;
yy2281:
		yych = *++cursor_;
		if (yych == '_') goto yy2378;
		goto yy87;
yy2282:
		yych = *++cursor_;
		if (yych == 'u') goto yy2379;
		goto yy87;
yy2283:
		yych = *++cursor_;
		if (yych == 'd') goto yy2381;
		goto yy87;
yy2284:
		yych = *++cursor_;
		if (yych == 'd') goto yy2383;
		goto yy87;
yy2285:
		yych = *++cursor_;
		if (yych == 'p') goto yy2385;
		goto yy87;
yy2286:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 496 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmwOr); }
#line 11803 "src/prebuilt/wast-lexer-gen.cc"
yy2288:
		yych = *++cursor_;
		if (yych == 'b') goto yy2386;
		goto yy87;
yy2289:
		yych = *++cursor_;
		if (yych == 'h') goto yy2388;
		goto yy87;
yy2290:
		yych = *++cursor_;
		if (yych == 'r') goto yy2389;
		goto yy87;
yy2291:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2391;
		case 'c':	goto yy2392;
		case 'o':	goto yy2393;
		case 's':	goto yy2394;
		case 'x':	goto yy2395;
		default:	goto yy87;
		}
yy2292:
		yych = *++cursor_;
		switch (yych) {
		case 'a':	goto yy2396;
		case 'c':	goto yy2397;
		case 'o':	goto yy2398;
		case 's':	goto yy2399;
		case 'x':	goto yy2400;
		default:	goto yy87;
		}
yy2293:
		yych = *++cursor_;
		if (yych == 'd') goto yy2401;
		if (yych == 'n') goto yy2402;
		goto yy87;
yy2294:
		yych = *++cursor_;
		if (yych == 'm') goto yy2403;
		goto yy87;
yy2295:
		yych = *++cursor_;
		if (yych == 'r') goto yy2404;
		goto yy87;
yy2296:
		yych = *++cursor_;
		if (yych == 'u') goto yy2405;
		goto yy87;
yy2297:
		yych = *++cursor_;
		if (yych == 'c') goto yy2406;
		if (yych == 'o') goto yy2407;
		goto yy87;
yy2298:
		yych = *++cursor_;
		if (yych == '6') goto yy2408;
		goto yy87;
yy2299:
		yych = *++cursor_;
		if (yych == '2') goto yy2410;
		goto yy87;
yy2300:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 471 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I64AtomicStore8); }
#line 11873 "src/prebuilt/wast-lexer-gen.cc"
yy2302:
		yych = *++cursor_;
		if (yych == '6') goto yy2412;
		goto yy87;
yy2303:
		yych = *++cursor_;
		if (yych == '6') goto yy2413;
		goto yy87;
yy2304:
		yych = *++cursor_;
		if (yych == '3') goto yy2414;
		if (yych == '6') goto yy2415;
		goto yy87;
yy2305:
		yych = *++cursor_;
		if (yych == '_') goto yy2416;
		goto yy87;
yy2306:
		yych = *++cursor_;
		if (yych == '_') goto yy2417;
		goto yy87;
yy2307:
		yych = *++cursor_;
		if (yych == '3') goto yy2418;
		if (yych == '6') goto yy2419;
		goto yy87;
yy2308:
		yych = *++cursor_;
		if (yych == 'e') goto yy2420;
		goto yy87;
yy2309:
		yych = *++cursor_;
		if (yych == 'e') goto yy2422;
		goto yy87;
yy2310:
		yych = *++cursor_;
		if (yych == '6') goto yy2424;
		goto yy87;
yy2311:
		yych = *++cursor_;
		if (yych == 'e') goto yy2425;
		goto yy87;
yy2312:
		yych = *++cursor_;
		if (yych == 'e') goto yy2426;
		goto yy87;
yy2313:
		yych = *++cursor_;
		if (yych == 'e') goto yy2427;
		goto yy87;
yy2314:
		yych = *++cursor_;
		if (yych == 'e') goto yy2429;
		goto yy87;
yy2315:
		yych = *++cursor_;
		if (yych == 'e') goto yy2430;
		goto yy87;
yy2316:
		yych = *++cursor_;
		if (yych == 'h') goto yy2431;
		goto yy87;
yy2317:
		yych = *++cursor_;
		if (yych == 'n') goto yy2432;
		goto yy87;
yy2318:
		yych = *++cursor_;
		if (yych == '2') goto yy2433;
		goto yy87;
yy2319:
		yych = *++cursor_;
		if (yych == '2') goto yy2435;
		goto yy87;
yy2320:
		yych = *++cursor_;
		if (yych == '4') goto yy2437;
		goto yy87;
yy2321:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 538 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, F32X4ExtractLane); }
#line 11959 "src/prebuilt/wast-lexer-gen.cc"
yy2323:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 544 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, F32X4ReplaceLane); }
#line 11967 "src/prebuilt/wast-lexer-gen.cc"
yy2325:
		yych = *++cursor_;
		if (yych == '4') goto yy2438;
		goto yy87;
yy2326:
		yych = *++cursor_;
		if (yych == '4') goto yy2440;
		goto yy87;
yy2327:
		yych = *++cursor_;
		if (yych == '2') goto yy2442;
		goto yy87;
yy2328:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 539 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, F64X2ExtractLane); }
#line 11987 "src/prebuilt/wast-lexer-gen.cc"
yy2330:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 545 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, F64X2ReplaceLane); }
#line 11995 "src/prebuilt/wast-lexer-gen.cc"
yy2332:
		yych = *++cursor_;
		if (yych == '_') goto yy2443;
		goto yy87;
yy2333:
		yych = *++cursor_;
		if (yych == '_') goto yy2444;
		goto yy87;
yy2334:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 541 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I16X8ReplaceLane); }
#line 12011 "src/prebuilt/wast-lexer-gen.cc"
yy2336:
		yych = *++cursor_;
		if (yych == '_') goto yy2445;
		goto yy87;
yy2337:
		yych = *++cursor_;
		if (yych == 'u') goto yy2446;
		goto yy87;
yy2338:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 462 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I32AtomicLoad8U); }
#line 12027 "src/prebuilt/wast-lexer-gen.cc"
yy2340:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 474 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmwAdd); }
#line 12035 "src/prebuilt/wast-lexer-gen.cc"
yy2342:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 488 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmwAnd); }
#line 12043 "src/prebuilt/wast-lexer-gen.cc"
yy2344:
		yych = *++cursor_;
		if (yych == 'x') goto yy2448;
		goto yy87;
yy2345:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 481 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmwSub); }
#line 12055 "src/prebuilt/wast-lexer-gen.cc"
yy2347:
		yych = *++cursor_;
		if (yych == 'g') goto yy2449;
		goto yy87;
yy2348:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 502 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmwXor); }
#line 12067 "src/prebuilt/wast-lexer-gen.cc"
yy2350:
		yych = *++cursor_;
		if (yych == 'd') goto yy2451;
		if (yych == 'n') goto yy2452;
		goto yy87;
yy2351:
		yych = *++cursor_;
		if (yych == 'm') goto yy2453;
		goto yy87;
yy2352:
		yych = *++cursor_;
		if (yych == 'r') goto yy2454;
		goto yy87;
yy2353:
		yych = *++cursor_;
		if (yych == 'u') goto yy2455;
		goto yy87;
yy2354:
		yych = *++cursor_;
		if (yych == 'c') goto yy2456;
		if (yych == 'o') goto yy2457;
		goto yy87;
yy2355:
		yych = *++cursor_;
		if (yych == 'd') goto yy2458;
		goto yy87;
yy2356:
		yych = *++cursor_;
		if (yych == 'd') goto yy2459;
		goto yy87;
yy2357:
		yych = *++cursor_;
		if (yych == 'p') goto yy2460;
		goto yy87;
yy2358:
		yych = *++cursor_;
		if (yych == '_') goto yy2461;
		goto yy87;
yy2359:
		yych = *++cursor_;
		if (yych == 'b') goto yy2462;
		goto yy87;
yy2360:
		yych = *++cursor_;
		if (yych == 'h') goto yy2463;
		goto yy87;
yy2361:
		yych = *++cursor_;
		if (yych == 'r') goto yy2464;
		goto yy87;
yy2362:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 470 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I32AtomicStore16); }
#line 12125 "src/prebuilt/wast-lexer-gen.cc"
yy2364:
		yych = *++cursor_;
		if (yych == '2') goto yy2465;
		goto yy87;
yy2365:
		yych = *++cursor_;
		if (yych == '2') goto yy2467;
		goto yy87;
yy2366:
		yych = *++cursor_;
		if (yych == '2') goto yy2469;
		goto yy87;
yy2367:
		yych = *++cursor_;
		if (yych == '4') goto yy2471;
		goto yy87;
yy2368:
		yych = *++cursor_;
		if (yych == 's') goto yy2473;
		if (yych == 'u') goto yy2475;
		goto yy87;
yy2369:
		yych = *++cursor_;
		if (yych == 's') goto yy2477;
		if (yych == 'u') goto yy2479;
		goto yy87;
yy2370:
		yych = *++cursor_;
		if (yych == '2') goto yy2481;
		goto yy87;
yy2371:
		yych = *++cursor_;
		if (yych == '4') goto yy2483;
		goto yy87;
yy2372:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 536 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I32X4ExtractLane); }
#line 12167 "src/prebuilt/wast-lexer-gen.cc"
yy2374:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 542 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I32X4ReplaceLane); }
#line 12175 "src/prebuilt/wast-lexer-gen.cc"
yy2376:
		yych = *++cursor_;
		if (yych == '2') goto yy2485;
		goto yy87;
yy2377:
		yych = *++cursor_;
		if (yych == 'u') goto yy2486;
		goto yy87;
yy2378:
		yych = *++cursor_;
		if (yych == 'u') goto yy2488;
		goto yy87;
yy2379:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 464 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I64AtomicLoad8U); }
#line 12195 "src/prebuilt/wast-lexer-gen.cc"
yy2381:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 475 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmwAdd); }
#line 12203 "src/prebuilt/wast-lexer-gen.cc"
yy2383:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 489 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmwAnd); }
#line 12211 "src/prebuilt/wast-lexer-gen.cc"
yy2385:
		yych = *++cursor_;
		if (yych == 'x') goto yy2490;
		goto yy87;
yy2386:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 482 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmwSub); }
#line 12223 "src/prebuilt/wast-lexer-gen.cc"
yy2388:
		yych = *++cursor_;
		if (yych == 'g') goto yy2491;
		goto yy87;
yy2389:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 503 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmwXor); }
#line 12235 "src/prebuilt/wast-lexer-gen.cc"
yy2391:
		yych = *++cursor_;
		if (yych == 'd') goto yy2493;
		if (yych == 'n') goto yy2494;
		goto yy87;
yy2392:
		yych = *++cursor_;
		if (yych == 'm') goto yy2495;
		goto yy87;
yy2393:
		yych = *++cursor_;
		if (yych == 'r') goto yy2496;
		goto yy87;
yy2394:
		yych = *++cursor_;
		if (yych == 'u') goto yy2497;
		goto yy87;
yy2395:
		yych = *++cursor_;
		if (yych == 'c') goto yy2498;
		if (yych == 'o') goto yy2499;
		goto yy87;
yy2396:
		yych = *++cursor_;
		if (yych == 'd') goto yy2500;
		if (yych == 'n') goto yy2501;
		goto yy87;
yy2397:
		yych = *++cursor_;
		if (yych == 'm') goto yy2502;
		goto yy87;
yy2398:
		yych = *++cursor_;
		if (yych == 'r') goto yy2503;
		goto yy87;
yy2399:
		yych = *++cursor_;
		if (yych == 'u') goto yy2504;
		goto yy87;
yy2400:
		yych = *++cursor_;
		if (yych == 'c') goto yy2505;
		if (yych == 'o') goto yy2506;
		goto yy87;
yy2401:
		yych = *++cursor_;
		if (yych == 'd') goto yy2507;
		goto yy87;
yy2402:
		yych = *++cursor_;
		if (yych == 'd') goto yy2508;
		goto yy87;
yy2403:
		yych = *++cursor_;
		if (yych == 'p') goto yy2509;
		goto yy87;
yy2404:
		yych = *++cursor_;
		if (yych == '_') goto yy2510;
		goto yy87;
yy2405:
		yych = *++cursor_;
		if (yych == 'b') goto yy2511;
		goto yy87;
yy2406:
		yych = *++cursor_;
		if (yych == 'h') goto yy2512;
		goto yy87;
yy2407:
		yych = *++cursor_;
		if (yych == 'r') goto yy2513;
		goto yy87;
yy2408:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 472 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I64AtomicStore16); }
#line 12315 "src/prebuilt/wast-lexer-gen.cc"
yy2410:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 473 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicStore, I64AtomicStore32); }
#line 12323 "src/prebuilt/wast-lexer-gen.cc"
yy2412:
		yych = *++cursor_;
		if (yych == '4') goto yy2514;
		goto yy87;
yy2413:
		yych = *++cursor_;
		if (yych == '4') goto yy2516;
		goto yy87;
yy2414:
		yych = *++cursor_;
		if (yych == '2') goto yy2518;
		goto yy87;
yy2415:
		yych = *++cursor_;
		if (yych == '4') goto yy2520;
		goto yy87;
yy2416:
		yych = *++cursor_;
		if (yych == 's') goto yy2522;
		if (yych == 'u') goto yy2524;
		goto yy87;
yy2417:
		yych = *++cursor_;
		if (yych == 's') goto yy2526;
		if (yych == 'u') goto yy2528;
		goto yy87;
yy2418:
		yych = *++cursor_;
		if (yych == '2') goto yy2530;
		goto yy87;
yy2419:
		yych = *++cursor_;
		if (yych == '4') goto yy2532;
		goto yy87;
yy2420:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 537 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I64X2ExtractLane); }
#line 12365 "src/prebuilt/wast-lexer-gen.cc"
yy2422:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 543 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I64X2ReplaceLane); }
#line 12373 "src/prebuilt/wast-lexer-gen.cc"
yy2424:
		yych = *++cursor_;
		if (yych == '4') goto yy2534;
		goto yy87;
yy2425:
		yych = *++cursor_;
		if (yych == '_') goto yy2535;
		goto yy87;
yy2426:
		yych = *++cursor_;
		if (yych == '_') goto yy2536;
		goto yy87;
yy2427:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 540 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I8X16ReplaceLane); }
#line 12393 "src/prebuilt/wast-lexer-gen.cc"
yy2429:
		yych = *++cursor_;
		if (yych == '_') goto yy2537;
		goto yy87;
yy2430:
		yych = *++cursor_;
		if (yych == 'c') goto yy2538;
		goto yy87;
yy2431:
		yych = *++cursor_;
		if (yych == 'm') goto yy2539;
		goto yy87;
yy2432:
		yych = *++cursor_;
		if (yych == 'i') goto yy2540;
		goto yy87;
yy2433:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 694 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ReinterpretI32); }
#line 12417 "src/prebuilt/wast-lexer-gen.cc"
yy2435:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 433 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F32ReinterpretI32); }
#line 12425 "src/prebuilt/wast-lexer-gen.cc"
yy2437:
		yych = *++cursor_;
		if (yych == '_') goto yy2541;
		goto yy87;
yy2438:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 696 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ReinterpretI64); }
#line 12437 "src/prebuilt/wast-lexer-gen.cc"
yy2440:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 435 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, F64ReinterpretI64); }
#line 12445 "src/prebuilt/wast-lexer-gen.cc"
yy2442:
		yych = *++cursor_;
		if (yych == '_') goto yy2542;
		goto yy87;
yy2443:
		yych = *++cursor_;
		if (yych == 's') goto yy2543;
		if (yych == 'u') goto yy2545;
		goto yy87;
yy2444:
		yych = *++cursor_;
		if (yych == 's') goto yy2547;
		if (yych == 'u') goto yy2549;
		goto yy87;
yy2445:
		yych = *++cursor_;
		if (yych == 's') goto yy2551;
		if (yych == 'u') goto yy2553;
		goto yy87;
yy2446:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 463 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I32AtomicLoad16U); }
#line 12472 "src/prebuilt/wast-lexer-gen.cc"
yy2448:
		yych = *++cursor_;
		if (yych == 'c') goto yy2555;
		goto yy87;
yy2449:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 509 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmwXchg); }
#line 12484 "src/prebuilt/wast-lexer-gen.cc"
yy2451:
		yych = *++cursor_;
		if (yych == 'd') goto yy2556;
		goto yy87;
yy2452:
		yych = *++cursor_;
		if (yych == 'd') goto yy2557;
		goto yy87;
yy2453:
		yych = *++cursor_;
		if (yych == 'p') goto yy2558;
		goto yy87;
yy2454:
		yych = *++cursor_;
		if (yych == '_') goto yy2559;
		goto yy87;
yy2455:
		yych = *++cursor_;
		if (yych == 'b') goto yy2560;
		goto yy87;
yy2456:
		yych = *++cursor_;
		if (yych == 'h') goto yy2561;
		goto yy87;
yy2457:
		yych = *++cursor_;
		if (yych == 'r') goto yy2562;
		goto yy87;
yy2458:
		yych = *++cursor_;
		if (yych == '_') goto yy2563;
		goto yy87;
yy2459:
		yych = *++cursor_;
		if (yych == '_') goto yy2564;
		goto yy87;
yy2460:
		yych = *++cursor_;
		if (yych == 'x') goto yy2565;
		goto yy87;
yy2461:
		yych = *++cursor_;
		if (yych == 'u') goto yy2566;
		goto yy87;
yy2462:
		yych = *++cursor_;
		if (yych == '_') goto yy2568;
		goto yy87;
yy2463:
		yych = *++cursor_;
		if (yych == 'g') goto yy2569;
		goto yy87;
yy2464:
		yych = *++cursor_;
		if (yych == '_') goto yy2570;
		goto yy87;
yy2465:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 695 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32ReinterpretF32); }
#line 12548 "src/prebuilt/wast-lexer-gen.cc"
yy2467:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 434 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32ReinterpretF32); }
#line 12556 "src/prebuilt/wast-lexer-gen.cc"
yy2469:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 698 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF32S); }
#line 12564 "src/prebuilt/wast-lexer-gen.cc"
yy2471:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 700 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF64S); }
#line 12572 "src/prebuilt/wast-lexer-gen.cc"
yy2473:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 415 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF32S); }
#line 12580 "src/prebuilt/wast-lexer-gen.cc"
yy2475:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 419 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF32U); }
#line 12588 "src/prebuilt/wast-lexer-gen.cc"
yy2477:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 417 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF64S); }
#line 12596 "src/prebuilt/wast-lexer-gen.cc"
yy2479:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 421 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF64U); }
#line 12604 "src/prebuilt/wast-lexer-gen.cc"
yy2481:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 702 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF32U); }
#line 12612 "src/prebuilt/wast-lexer-gen.cc"
yy2483:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 704 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I32TruncSatF64U); }
#line 12620 "src/prebuilt/wast-lexer-gen.cc"
yy2485:
		yych = *++cursor_;
		if (yych == 'x') goto yy2571;
		goto yy87;
yy2486:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 465 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I64AtomicLoad16U); }
#line 12632 "src/prebuilt/wast-lexer-gen.cc"
yy2488:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 466 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicLoad, I64AtomicLoad32U); }
#line 12640 "src/prebuilt/wast-lexer-gen.cc"
yy2490:
		yych = *++cursor_;
		if (yych == 'c') goto yy2572;
		goto yy87;
yy2491:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 510 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmwXchg); }
#line 12652 "src/prebuilt/wast-lexer-gen.cc"
yy2493:
		yych = *++cursor_;
		if (yych == 'd') goto yy2573;
		goto yy87;
yy2494:
		yych = *++cursor_;
		if (yych == 'd') goto yy2574;
		goto yy87;
yy2495:
		yych = *++cursor_;
		if (yych == 'p') goto yy2575;
		goto yy87;
yy2496:
		yych = *++cursor_;
		if (yych == '_') goto yy2576;
		goto yy87;
yy2497:
		yych = *++cursor_;
		if (yych == 'b') goto yy2577;
		goto yy87;
yy2498:
		yych = *++cursor_;
		if (yych == 'h') goto yy2578;
		goto yy87;
yy2499:
		yych = *++cursor_;
		if (yych == 'r') goto yy2579;
		goto yy87;
yy2500:
		yych = *++cursor_;
		if (yych == 'd') goto yy2580;
		goto yy87;
yy2501:
		yych = *++cursor_;
		if (yych == 'd') goto yy2581;
		goto yy87;
yy2502:
		yych = *++cursor_;
		if (yych == 'p') goto yy2582;
		goto yy87;
yy2503:
		yych = *++cursor_;
		if (yych == '_') goto yy2583;
		goto yy87;
yy2504:
		yych = *++cursor_;
		if (yych == 'b') goto yy2584;
		goto yy87;
yy2505:
		yych = *++cursor_;
		if (yych == 'h') goto yy2585;
		goto yy87;
yy2506:
		yych = *++cursor_;
		if (yych == 'r') goto yy2586;
		goto yy87;
yy2507:
		yych = *++cursor_;
		if (yych == '_') goto yy2587;
		goto yy87;
yy2508:
		yych = *++cursor_;
		if (yych == '_') goto yy2588;
		goto yy87;
yy2509:
		yych = *++cursor_;
		if (yych == 'x') goto yy2589;
		goto yy87;
yy2510:
		yych = *++cursor_;
		if (yych == 'u') goto yy2590;
		goto yy87;
yy2511:
		yych = *++cursor_;
		if (yych == '_') goto yy2592;
		goto yy87;
yy2512:
		yych = *++cursor_;
		if (yych == 'g') goto yy2593;
		goto yy87;
yy2513:
		yych = *++cursor_;
		if (yych == '_') goto yy2594;
		goto yy87;
yy2514:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 697 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64ReinterpretF64); }
#line 12744 "src/prebuilt/wast-lexer-gen.cc"
yy2516:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 436 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64ReinterpretF64); }
#line 12752 "src/prebuilt/wast-lexer-gen.cc"
yy2518:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 699 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF32S); }
#line 12760 "src/prebuilt/wast-lexer-gen.cc"
yy2520:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 701 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF64S); }
#line 12768 "src/prebuilt/wast-lexer-gen.cc"
yy2522:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 416 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF32S); }
#line 12776 "src/prebuilt/wast-lexer-gen.cc"
yy2524:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 420 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF32U); }
#line 12784 "src/prebuilt/wast-lexer-gen.cc"
yy2526:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 418 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF64S); }
#line 12792 "src/prebuilt/wast-lexer-gen.cc"
yy2528:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 422 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF64U); }
#line 12800 "src/prebuilt/wast-lexer-gen.cc"
yy2530:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 703 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF32U); }
#line 12808 "src/prebuilt/wast-lexer-gen.cc"
yy2532:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 705 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Convert, I64TruncSatF64U); }
#line 12816 "src/prebuilt/wast-lexer-gen.cc"
yy2534:
		yych = *++cursor_;
		if (yych == 'x') goto yy2595;
		goto yy87;
yy2535:
		yych = *++cursor_;
		if (yych == 's') goto yy2596;
		if (yych == 'u') goto yy2598;
		goto yy87;
yy2536:
		yych = *++cursor_;
		if (yych == 's') goto yy2600;
		if (yych == 'u') goto yy2602;
		goto yy87;
yy2537:
		yych = *++cursor_;
		if (yych == 's') goto yy2604;
		if (yych == 'u') goto yy2606;
		goto yy87;
yy2538:
		yych = *++cursor_;
		if (yych == 't') goto yy2608;
		goto yy87;
yy2539:
		yych = *++cursor_;
		if (yych == 'e') goto yy2610;
		goto yy87;
yy2540:
		yych = *++cursor_;
		if (yych == 'c') goto yy2611;
		goto yy87;
yy2541:
		yych = *++cursor_;
		if (yych == 's') goto yy2612;
		if (yych == 'u') goto yy2614;
		goto yy87;
yy2542:
		yych = *++cursor_;
		if (yych == 's') goto yy2616;
		if (yych == 'u') goto yy2618;
		goto yy87;
yy2543:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 564 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8AddSaturateS); }
#line 12865 "src/prebuilt/wast-lexer-gen.cc"
yy2545:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 565 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8AddSaturateU); }
#line 12873 "src/prebuilt/wast-lexer-gen.cc"
yy2547:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 534 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I16X8ExtractLaneS); }
#line 12881 "src/prebuilt/wast-lexer-gen.cc"
yy2549:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 535 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I16X8ExtractLaneU); }
#line 12889 "src/prebuilt/wast-lexer-gen.cc"
yy2551:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 568 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8SubSaturateS); }
#line 12897 "src/prebuilt/wast-lexer-gen.cc"
yy2553:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 569 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I16X8SubSaturateU); }
#line 12905 "src/prebuilt/wast-lexer-gen.cc"
yy2555:
		yych = *++cursor_;
		if (yych == 'h') goto yy2620;
		goto yy87;
yy2556:
		yych = *++cursor_;
		if (yych == '_') goto yy2621;
		goto yy87;
yy2557:
		yych = *++cursor_;
		if (yych == '_') goto yy2622;
		goto yy87;
yy2558:
		yych = *++cursor_;
		if (yych == 'x') goto yy2623;
		goto yy87;
yy2559:
		yych = *++cursor_;
		if (yych == 'u') goto yy2624;
		goto yy87;
yy2560:
		yych = *++cursor_;
		if (yych == '_') goto yy2626;
		goto yy87;
yy2561:
		yych = *++cursor_;
		if (yych == 'g') goto yy2627;
		goto yy87;
yy2562:
		yych = *++cursor_;
		if (yych == '_') goto yy2628;
		goto yy87;
yy2563:
		yych = *++cursor_;
		if (yych == 'u') goto yy2629;
		goto yy87;
yy2564:
		yych = *++cursor_;
		if (yych == 'u') goto yy2631;
		goto yy87;
yy2565:
		yych = *++cursor_;
		if (yych == 'c') goto yy2633;
		goto yy87;
yy2566:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 497 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw8OrU); }
#line 12957 "src/prebuilt/wast-lexer-gen.cc"
yy2568:
		yych = *++cursor_;
		if (yych == 'u') goto yy2634;
		goto yy87;
yy2569:
		yych = *++cursor_;
		if (yych == '_') goto yy2636;
		goto yy87;
yy2570:
		yych = *++cursor_;
		if (yych == 'u') goto yy2637;
		goto yy87;
yy2571:
		yych = *++cursor_;
		if (yych == '4') goto yy2639;
		goto yy87;
yy2572:
		yych = *++cursor_;
		if (yych == 'h') goto yy2640;
		goto yy87;
yy2573:
		yych = *++cursor_;
		if (yych == '_') goto yy2641;
		goto yy87;
yy2574:
		yych = *++cursor_;
		if (yych == '_') goto yy2642;
		goto yy87;
yy2575:
		yych = *++cursor_;
		if (yych == 'x') goto yy2643;
		goto yy87;
yy2576:
		yych = *++cursor_;
		if (yych == 'u') goto yy2644;
		goto yy87;
yy2577:
		yych = *++cursor_;
		if (yych == '_') goto yy2646;
		goto yy87;
yy2578:
		yych = *++cursor_;
		if (yych == 'g') goto yy2647;
		goto yy87;
yy2579:
		yych = *++cursor_;
		if (yych == '_') goto yy2648;
		goto yy87;
yy2580:
		yych = *++cursor_;
		if (yych == '_') goto yy2649;
		goto yy87;
yy2581:
		yych = *++cursor_;
		if (yych == '_') goto yy2650;
		goto yy87;
yy2582:
		yych = *++cursor_;
		if (yych == 'x') goto yy2651;
		goto yy87;
yy2583:
		yych = *++cursor_;
		if (yych == 'u') goto yy2652;
		goto yy87;
yy2584:
		yych = *++cursor_;
		if (yych == '_') goto yy2654;
		goto yy87;
yy2585:
		yych = *++cursor_;
		if (yych == 'g') goto yy2655;
		goto yy87;
yy2586:
		yych = *++cursor_;
		if (yych == '_') goto yy2656;
		goto yy87;
yy2587:
		yych = *++cursor_;
		if (yych == 'u') goto yy2657;
		goto yy87;
yy2588:
		yych = *++cursor_;
		if (yych == 'u') goto yy2659;
		goto yy87;
yy2589:
		yych = *++cursor_;
		if (yych == 'c') goto yy2661;
		goto yy87;
yy2590:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 499 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw8OrU); }
#line 13053 "src/prebuilt/wast-lexer-gen.cc"
yy2592:
		yych = *++cursor_;
		if (yych == 'u') goto yy2662;
		goto yy87;
yy2593:
		yych = *++cursor_;
		if (yych == '_') goto yy2664;
		goto yy87;
yy2594:
		yych = *++cursor_;
		if (yych == 'u') goto yy2665;
		goto yy87;
yy2595:
		yych = *++cursor_;
		if (yych == '2') goto yy2667;
		goto yy87;
yy2596:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 562 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16AddSaturateS); }
#line 13077 "src/prebuilt/wast-lexer-gen.cc"
yy2598:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 563 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16AddSaturateU); }
#line 13085 "src/prebuilt/wast-lexer-gen.cc"
yy2600:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 532 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I8X16ExtractLaneS); }
#line 13093 "src/prebuilt/wast-lexer-gen.cc"
yy2602:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 533 "src/wast-lexer.cc"
		{ RETURN_OPCODE(SimdLaneOp, I8X16ExtractLaneU); }
#line 13101 "src/prebuilt/wast-lexer-gen.cc"
yy2604:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 566 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16SubSaturateS); }
#line 13109 "src/prebuilt/wast-lexer-gen.cc"
yy2606:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 567 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Binary, I8X16SubSaturateU); }
#line 13117 "src/prebuilt/wast-lexer-gen.cc"
yy2608:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 664 "src/wast-lexer.cc"
		{ RETURN_OPCODE0(ReturnCallIndirect); }
#line 13125 "src/prebuilt/wast-lexer-gen.cc"
yy2610:
		yych = *++cursor_;
		if (yych == 't') goto yy2668;
		goto yy87;
yy2611:
		yych = *++cursor_;
		if (yych == 'a') goto yy2669;
		goto yy87;
yy2612:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 655 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32X4ConvertI32X4S); }
#line 13141 "src/prebuilt/wast-lexer-gen.cc"
yy2614:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 656 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F32X4ConvertI32X4U); }
#line 13149 "src/prebuilt/wast-lexer-gen.cc"
yy2616:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 657 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64X2ConvertI64X2S); }
#line 13157 "src/prebuilt/wast-lexer-gen.cc"
yy2618:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 658 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, F64X2ConvertI64X2U); }
#line 13165 "src/prebuilt/wast-lexer-gen.cc"
yy2620:
		yych = *++cursor_;
		if (yych == 'g') goto yy2670;
		goto yy87;
yy2621:
		yych = *++cursor_;
		if (yych == 'u') goto yy2672;
		goto yy87;
yy2622:
		yych = *++cursor_;
		if (yych == 'u') goto yy2674;
		goto yy87;
yy2623:
		yych = *++cursor_;
		if (yych == 'c') goto yy2676;
		goto yy87;
yy2624:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 498 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw16OrU); }
#line 13189 "src/prebuilt/wast-lexer-gen.cc"
yy2626:
		yych = *++cursor_;
		if (yych == 'u') goto yy2677;
		goto yy87;
yy2627:
		yych = *++cursor_;
		if (yych == '_') goto yy2679;
		goto yy87;
yy2628:
		yych = *++cursor_;
		if (yych == 'u') goto yy2680;
		goto yy87;
yy2629:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 476 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw8AddU); }
#line 13209 "src/prebuilt/wast-lexer-gen.cc"
yy2631:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 490 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw8AndU); }
#line 13217 "src/prebuilt/wast-lexer-gen.cc"
yy2633:
		yych = *++cursor_;
		if (yych == 'h') goto yy2682;
		goto yy87;
yy2634:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 483 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw8SubU); }
#line 13229 "src/prebuilt/wast-lexer-gen.cc"
yy2636:
		yych = *++cursor_;
		if (yych == 'u') goto yy2683;
		goto yy87;
yy2637:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 504 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw8XorU); }
#line 13241 "src/prebuilt/wast-lexer-gen.cc"
yy2639:
		yych = *++cursor_;
		if (yych == '_') goto yy2685;
		goto yy87;
yy2640:
		yych = *++cursor_;
		if (yych == 'g') goto yy2686;
		goto yy87;
yy2641:
		yych = *++cursor_;
		if (yych == 'u') goto yy2688;
		goto yy87;
yy2642:
		yych = *++cursor_;
		if (yych == 'u') goto yy2690;
		goto yy87;
yy2643:
		yych = *++cursor_;
		if (yych == 'c') goto yy2692;
		goto yy87;
yy2644:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 500 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw16OrU); }
#line 13269 "src/prebuilt/wast-lexer-gen.cc"
yy2646:
		yych = *++cursor_;
		if (yych == 'u') goto yy2693;
		goto yy87;
yy2647:
		yych = *++cursor_;
		if (yych == '_') goto yy2695;
		goto yy87;
yy2648:
		yych = *++cursor_;
		if (yych == 'u') goto yy2696;
		goto yy87;
yy2649:
		yych = *++cursor_;
		if (yych == 'u') goto yy2698;
		goto yy87;
yy2650:
		yych = *++cursor_;
		if (yych == 'u') goto yy2700;
		goto yy87;
yy2651:
		yych = *++cursor_;
		if (yych == 'c') goto yy2702;
		goto yy87;
yy2652:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 501 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw32OrU); }
#line 13301 "src/prebuilt/wast-lexer-gen.cc"
yy2654:
		yych = *++cursor_;
		if (yych == 'u') goto yy2703;
		goto yy87;
yy2655:
		yych = *++cursor_;
		if (yych == '_') goto yy2705;
		goto yy87;
yy2656:
		yych = *++cursor_;
		if (yych == 'u') goto yy2706;
		goto yy87;
yy2657:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 478 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw8AddU); }
#line 13321 "src/prebuilt/wast-lexer-gen.cc"
yy2659:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 492 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw8AndU); }
#line 13329 "src/prebuilt/wast-lexer-gen.cc"
yy2661:
		yych = *++cursor_;
		if (yych == 'h') goto yy2708;
		goto yy87;
yy2662:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 485 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw8SubU); }
#line 13341 "src/prebuilt/wast-lexer-gen.cc"
yy2664:
		yych = *++cursor_;
		if (yych == 'u') goto yy2709;
		goto yy87;
yy2665:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 506 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw8XorU); }
#line 13353 "src/prebuilt/wast-lexer-gen.cc"
yy2667:
		yych = *++cursor_;
		if (yych == '_') goto yy2711;
		goto yy87;
yy2668:
		yych = *++cursor_;
		if (yych == 'i') goto yy2712;
		goto yy87;
yy2669:
		yych = *++cursor_;
		if (yych == 'l') goto yy2713;
		goto yy87;
yy2670:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 516 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I32AtomicRmwCmpxchg); }
#line 13373 "src/prebuilt/wast-lexer-gen.cc"
yy2672:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 477 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw16AddU); }
#line 13381 "src/prebuilt/wast-lexer-gen.cc"
yy2674:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 491 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw16AndU); }
#line 13389 "src/prebuilt/wast-lexer-gen.cc"
yy2676:
		yych = *++cursor_;
		if (yych == 'h') goto yy2714;
		goto yy87;
yy2677:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 484 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw16SubU); }
#line 13401 "src/prebuilt/wast-lexer-gen.cc"
yy2679:
		yych = *++cursor_;
		if (yych == 'u') goto yy2715;
		goto yy87;
yy2680:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 505 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw16XorU); }
#line 13413 "src/prebuilt/wast-lexer-gen.cc"
yy2682:
		yych = *++cursor_;
		if (yych == 'g') goto yy2717;
		goto yy87;
yy2683:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 511 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw8XchgU); }
#line 13425 "src/prebuilt/wast-lexer-gen.cc"
yy2685:
		yych = *++cursor_;
		if (yych == 's') goto yy2718;
		if (yych == 'u') goto yy2720;
		goto yy87;
yy2686:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 517 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I64AtomicRmwCmpxchg); }
#line 13438 "src/prebuilt/wast-lexer-gen.cc"
yy2688:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 479 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw16AddU); }
#line 13446 "src/prebuilt/wast-lexer-gen.cc"
yy2690:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 493 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw16AndU); }
#line 13454 "src/prebuilt/wast-lexer-gen.cc"
yy2692:
		yych = *++cursor_;
		if (yych == 'h') goto yy2722;
		goto yy87;
yy2693:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 486 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw16SubU); }
#line 13466 "src/prebuilt/wast-lexer-gen.cc"
yy2695:
		yych = *++cursor_;
		if (yych == 'u') goto yy2723;
		goto yy87;
yy2696:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 507 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw16XorU); }
#line 13478 "src/prebuilt/wast-lexer-gen.cc"
yy2698:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 480 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw32AddU); }
#line 13486 "src/prebuilt/wast-lexer-gen.cc"
yy2700:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 494 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw32AndU); }
#line 13494 "src/prebuilt/wast-lexer-gen.cc"
yy2702:
		yych = *++cursor_;
		if (yych == 'h') goto yy2725;
		goto yy87;
yy2703:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 487 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw32SubU); }
#line 13506 "src/prebuilt/wast-lexer-gen.cc"
yy2705:
		yych = *++cursor_;
		if (yych == 'u') goto yy2726;
		goto yy87;
yy2706:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 508 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw32XorU); }
#line 13518 "src/prebuilt/wast-lexer-gen.cc"
yy2708:
		yych = *++cursor_;
		if (yych == 'g') goto yy2728;
		goto yy87;
yy2709:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 513 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw8XchgU); }
#line 13530 "src/prebuilt/wast-lexer-gen.cc"
yy2711:
		yych = *++cursor_;
		if (yych == 's') goto yy2729;
		if (yych == 'u') goto yy2731;
		goto yy87;
yy2712:
		yych = *++cursor_;
		if (yych == 'c') goto yy2733;
		goto yy87;
yy2713:
		yych = *++cursor_;
		if (yych == '_') goto yy2734;
		goto yy87;
yy2714:
		yych = *++cursor_;
		if (yych == 'g') goto yy2735;
		goto yy87;
yy2715:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 512 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I32AtomicRmw16XchgU); }
#line 13555 "src/prebuilt/wast-lexer-gen.cc"
yy2717:
		yych = *++cursor_;
		if (yych == '_') goto yy2736;
		goto yy87;
yy2718:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 659 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32X4TruncSatF32X4S); }
#line 13567 "src/prebuilt/wast-lexer-gen.cc"
yy2720:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 660 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I32X4TruncSatF32X4U); }
#line 13575 "src/prebuilt/wast-lexer-gen.cc"
yy2722:
		yych = *++cursor_;
		if (yych == 'g') goto yy2737;
		goto yy87;
yy2723:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 514 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw16XchgU); }
#line 13587 "src/prebuilt/wast-lexer-gen.cc"
yy2725:
		yych = *++cursor_;
		if (yych == 'g') goto yy2738;
		goto yy87;
yy2726:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 515 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmw, I64AtomicRmw32XchgU); }
#line 13599 "src/prebuilt/wast-lexer-gen.cc"
yy2728:
		yych = *++cursor_;
		if (yych == '_') goto yy2739;
		goto yy87;
yy2729:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 661 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64X2TruncSatF64X2S); }
#line 13611 "src/prebuilt/wast-lexer-gen.cc"
yy2731:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 662 "src/wast-lexer.cc"
		{ RETURN_OPCODE(Unary, I64X2TruncSatF64X2U); }
#line 13619 "src/prebuilt/wast-lexer-gen.cc"
yy2733:
		yych = *++cursor_;
		if (yych == '_') goto yy2740;
		goto yy87;
yy2734:
		yych = *++cursor_;
		if (yych == 'n') goto yy2741;
		goto yy87;
yy2735:
		yych = *++cursor_;
		if (yych == '_') goto yy2742;
		goto yy87;
yy2736:
		yych = *++cursor_;
		if (yych == 'u') goto yy2743;
		goto yy87;
yy2737:
		yych = *++cursor_;
		if (yych == '_') goto yy2745;
		goto yy87;
yy2738:
		yych = *++cursor_;
		if (yych == '_') goto yy2746;
		goto yy87;
yy2739:
		yych = *++cursor_;
		if (yych == 'u') goto yy2747;
		goto yy87;
yy2740:
		yych = *++cursor_;
		if (yych == 'n') goto yy2749;
		goto yy87;
yy2741:
		yych = *++cursor_;
		if (yych == 'a') goto yy2750;
		goto yy87;
yy2742:
		yych = *++cursor_;
		if (yych == 'u') goto yy2751;
		goto yy87;
yy2743:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 518 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I32AtomicRmw8CmpxchgU); }
#line 13667 "src/prebuilt/wast-lexer-gen.cc"
yy2745:
		yych = *++cursor_;
		if (yych == 'u') goto yy2753;
		goto yy87;
yy2746:
		yych = *++cursor_;
		if (yych == 'u') goto yy2755;
		goto yy87;
yy2747:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 520 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I64AtomicRmw8CmpxchgU); }
#line 13683 "src/prebuilt/wast-lexer-gen.cc"
yy2749:
		yych = *++cursor_;
		if (yych == 'a') goto yy2757;
		goto yy87;
yy2750:
		yych = *++cursor_;
		if (yych == 'n') goto yy2758;
		goto yy87;
yy2751:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 519 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I32AtomicRmw16CmpxchgU); }
#line 13699 "src/prebuilt/wast-lexer-gen.cc"
yy2753:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 521 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I64AtomicRmw16CmpxchgU); }
#line 13707 "src/prebuilt/wast-lexer-gen.cc"
yy2755:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 522 "src/wast-lexer.cc"
		{ RETURN_OPCODE(AtomicRmwCmpxchg, I64AtomicRmw32CmpxchgU); }
#line 13715 "src/prebuilt/wast-lexer-gen.cc"
yy2757:
		yych = *++cursor_;
		if (yych == 'n') goto yy2760;
		goto yy87;
yy2758:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 733 "src/wast-lexer.cc"
		{ RETURN(AssertReturnCanonicalNan); }
#line 13727 "src/prebuilt/wast-lexer-gen.cc"
yy2760:
		++cursor_;
		if (yybm[0+(yych = *cursor_)] & 8) {
			goto yy86;
		}
#line 734 "src/wast-lexer.cc"
		{ RETURN(AssertReturnArithmeticNan); }
#line 13735 "src/prebuilt/wast-lexer-gen.cc"
	}
}
#line 768 "src/wast-lexer.cc"

  }
}

}  // namespace wabt
